@@ -382,10 +382,8 @@ Magick::Geometry::operator MagickCore::RectangleInfo() const
   RectangleInfo rectangle;
   rectangle.width=_width;
   rectangle.height=_height;
-  _xNegative ? rectangle.x=static_cast<ssize_t>(0-_xOff) :
-    rectangle.x=static_cast<ssize_t>(_xOff);
-  _yNegative ? rectangle.y=static_cast<ssize_t>(0-_yOff) :
-    rectangle.y=static_cast<ssize_t>(_yOff);
+  rectangle.x=_xOff;
+  rectangle.y=_yOff;
   return(rectangle);
 }
 
@@ -447,8 +445,8 @@ const Magick::Offset& Magick::Offset::operator=(const char *offset_)
     flags;
 
   flags=ParseGeometry(offset_,&geometry_info);
-  _x=geometry_info.rho;
-  _y=geometry_info.sigma;
+  _x=(ssize_t) geometry_info.rho;
+  _y=(ssize_t) geometry_info.sigma;
   if ((flags & MagickCore::SigmaValue) == 0)
     _y=_x;
   return(*this);
@@ -487,4 +485,4 @@ Magick::Offset::operator MagickCore::OffsetInfo() const
   offset.x=_x;
   offset.y=_y;
   return(offset);
-}
\ No newline at end of file
+}
