@@ -225,9 +225,13 @@ FILE *Magick::Options::file(void) const
 
 void Magick::Options::fileName(const std::string &fileName_)
 {
-  fileName_.copy(_imageInfo->filename,MaxTextExtent-1);
-  if (fileName_.length() > MaxTextExtent-1)
-    _imageInfo->filename[MaxTextExtent-1]=0;
+  ssize_t
+    max_length;
+
+  max_length=sizeof(_imageInfo->filename)-1;
+  fileName_.copy(_imageInfo->filename,max_length);
+  if ((ssize_t) fileName_.length() > max_length)
+    _imageInfo->filename[max_length]=0;
   else
     _imageInfo->filename[fileName_.length()]=0;
 }
@@ -602,6 +606,9 @@ void Magick::Options::strokeDashArray(const double *strokeDashArray_)
       // Allocate elements
       _drawInfo->dash_pattern=static_cast<double*>(AcquireMagickMemory((x+1)*
         sizeof(double)));
+      if (!_drawInfo->dash_pattern)
+        throwExceptionExplicit(MagickCore::ResourceLimitError,
+          "Unable to allocate dash-pattern memory");
       // Copy elements
       memcpy(_drawInfo->dash_pattern,strokeDashArray_,(x+1)*sizeof(double));
       _drawInfo->dash_pattern[x]=0.0;
