@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,13 +100,13 @@ static Image *ReadAAIImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -271,7 +271,7 @@ ModuleExport size_t RegisterAAIImage(void)
   entry->decoder=(DecodeImageHandler *) ReadAAIImage;
   entry->encoder=(EncodeImageHandler *) WriteAAIImage;
   entry->description=ConstantString("AAI Dune image");
-  entry->module=ConstantString("AAI");
+  entry->magick_module=ConstantString("AAI");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -332,13 +332,13 @@ static MagickBooleanType WriteAAIImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
-  register const PixelPacket
+  const PixelPacket
     *magick_restrict p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *magick_restrict q;
 
   size_t
