@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -164,7 +164,7 @@ static Image *ReadCALSImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   unsigned long
@@ -332,7 +332,7 @@ ModuleExport size_t RegisterCALSImage(void)
   entry->magick=(IsImageFormatHandler *) IsCALS;
   entry->description=ConstantString(CALSDescription);
   entry->note=ConstantString(CALSNote);
-  entry->module=ConstantString("CALS");
+  entry->magick_module=ConstantString("CALS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CALS");
   entry->decoder=(DecodeImageHandler *) ReadCALSImage;
@@ -343,7 +343,7 @@ ModuleExport size_t RegisterCALSImage(void)
   entry->magick=(IsImageFormatHandler *) IsCALS;
   entry->description=ConstantString(CALSDescription);
   entry->note=ConstantString(CALSNote);
-  entry->module=ConstantString("CALS");
+  entry->magick_module=ConstantString("CALS");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -406,10 +406,10 @@ static ssize_t WriteCALSRecord(Image *image,const char *data)
   char
     pad[128];
 
-  register const char
+  const char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -447,7 +447,7 @@ static MagickBooleanType WriteCALSImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
