@@ -107,13 +107,13 @@ static MagickBooleanType InsertRow(int bpp,unsigned char *p,ssize_t y,
   ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   IndexPacket
     index;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
   exception=(&image->exception);
@@ -277,7 +277,7 @@ static int GetCutColors(Image *image)
     intensity,
     scale_intensity;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   ssize_t
@@ -568,6 +568,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
   BImgBuff=(unsigned char *) AcquireQuantumMemory((size_t) ldblk,
     sizeof(*BImgBuff));  /*Ldblk was set in the check phase*/
   if(BImgBuff==NULL) goto NoMemory;
+  (void) memset(BImgBuff,0,(size_t) ldblk*sizeof(*BImgBuff));
 
   offset=SeekBlob(image,6 /*sizeof(Header)*/,SEEK_SET);
   if (offset < 0)
@@ -633,7 +634,7 @@ static Image *ReadCUTImage(const ImageInfo *image_info,ExceptionInfo *exception)
             {
               for (i=0; i < (ssize_t)image->colors; i++)
                 {
-                  register Quantum
+                  Quantum
                     sample;
                   sample=ScaleCharToQuantum((unsigned char) i);
                   if(image->colormap[i].red!=sample) goto Finish;
@@ -710,7 +711,7 @@ ModuleExport size_t RegisterCUTImage(void)
   entry->decoder=(DecodeImageHandler *) ReadCUTImage;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("DR Halo");
-  entry->module=ConstantString("CUT");
+  entry->magick_module=ConstantString("CUT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
