@@ -19,7 +19,7 @@
 %                              September 2013                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -933,7 +933,7 @@ static size_t CompressAlpha(const size_t min, const size_t max,
   unsigned char
     codes[8];
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1236,7 +1236,7 @@ static void CompressRangeFit(const size_t count,
     half,
     dist;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1325,7 +1325,7 @@ static void ComputeEndPoints(const DDSSingleColourLookup *lookup[],
   const unsigned char *color, DDSVector3 *start, DDSVector3 *end,
   unsigned char *index)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1371,7 +1371,7 @@ static void ComputePrincipleComponent(const float *covariance,
     row2,
     v;
 
-  register ssize_t
+  ssize_t
     i;
 
   row0.x = covariance[0];
@@ -1489,7 +1489,7 @@ static MagickBooleanType ConstructOrdering(const size_t count,
      dps[16],
      f;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1812,6 +1812,8 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((decoder)(image, &dds_info, exception) != MagickTrue)
       {
         (void) CloseBlob(image);
+        if (n == 0)
+          return(DestroyImageList(image));
         return(GetFirstImageInList(image));
       }
   }
@@ -1879,7 +1881,7 @@ static MagickBooleanType ReadDXT1(Image *image,DDSInfo *dds_info,
   PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1960,7 +1962,7 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info,
   PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -2052,7 +2054,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info,
   PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -2327,7 +2329,7 @@ ModuleExport size_t RegisterDDSImage(void)
   entry->magick = (IsImageFormatHandler *) IsDDS;
   entry->seekable_stream=MagickTrue;
   entry->description = ConstantString("Microsoft DirectDraw Surface");
-  entry->module = ConstantString("DDS");
+  entry->magick_module = ConstantString("DDS");
   (void) RegisterMagickInfo(entry);
   entry = SetMagickInfo("DXT1");
   entry->decoder = (DecodeImageHandler *) ReadDDSImage;
@@ -2335,7 +2337,7 @@ ModuleExport size_t RegisterDDSImage(void)
   entry->magick = (IsImageFormatHandler *) IsDDS;
   entry->seekable_stream=MagickTrue;
   entry->description = ConstantString("Microsoft DirectDraw Surface");
-  entry->module = ConstantString("DDS");
+  entry->magick_module = ConstantString("DDS");
   (void) RegisterMagickInfo(entry);
   entry = SetMagickInfo("DXT5");
   entry->decoder = (DecodeImageHandler *) ReadDDSImage;
@@ -2343,7 +2345,7 @@ ModuleExport size_t RegisterDDSImage(void)
   entry->magick = (IsImageFormatHandler *) IsDDS;
   entry->seekable_stream=MagickTrue;
   entry->description = ConstantString("Microsoft DirectDraw Surface");
-  entry->module = ConstantString("DDS");
+  entry->magick_module = ConstantString("DDS");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -2351,7 +2353,7 @@ ModuleExport size_t RegisterDDSImage(void)
 static void RemapIndices(const ssize_t *map, const unsigned char *source,
   unsigned char *target)
 {
-  register ssize_t
+  ssize_t
     i;
 
   for (i = 0; i < 16; i++)
@@ -2369,7 +2371,7 @@ static void RemapIndices(const ssize_t *map, const unsigned char *source,
 static MagickBooleanType SkipDXTMipmaps(Image *image,DDSInfo *dds_info,
   int texel_size,ExceptionInfo *exception)
 {
-  register ssize_t
+  ssize_t
     i;
 
   MagickOffsetType
@@ -2421,7 +2423,7 @@ static MagickBooleanType SkipRGBMipmaps(Image *image,DDSInfo *dds_info,
   MagickOffsetType
     offset;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2490,7 +2492,7 @@ ModuleExport void UnregisterDDSImage(void)
 static void WriteAlphas(Image *image, const ssize_t* alphas, size_t min5,
   size_t max5, size_t min7, size_t max7)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2637,13 +2639,14 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
   (void) TransformImageColorspace(image,sRGBColorspace);
   pixelFormat=DDPF_FOURCC;
   compression=FOURCC_DXT5;
-
   if (!image->matte)
     compression=FOURCC_DXT1;
-
   if (LocaleCompare(image_info->magick,"dxt1") == 0)
     compression=FOURCC_DXT1;
-
+  if (image_info->compression == DXT1Compression)
+    compression=FOURCC_DXT1;
+  else if (image_info->compression == NoCompression)
+    pixelFormat=DDPF_RGB;
   option=GetImageOption(image_info,"dds:compression");
   if (option != (char *) NULL)
     {
@@ -2652,10 +2655,8 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
        if (LocaleCompare(option,"none") == 0)
          pixelFormat=DDPF_RGB;
     }
-
   clusterFit=MagickFalse;
   weightByAlpha=MagickFalse;
-
   if (pixelFormat == DDPF_FOURCC)
     {
       option=GetImageOption(image_info,"dds:cluster-fit");
@@ -2670,7 +2671,6 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
             }
         }
     }
-
   maxMipmaps=SIZE_MAX;
   mipmaps=0;
   if ((image->columns & (image->columns - 1)) == 0 &&
@@ -2692,16 +2692,12 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
           }
         }
     }
-
   WriteDDSInfo(image,pixelFormat,compression,mipmaps);
-
   WriteImageData(image,pixelFormat,compression,clusterFit,weightByAlpha,
     &image->exception);
-
   if (mipmaps > 0 && WriteMipmaps(image,pixelFormat,compression,mipmaps,
         clusterFit,weightByAlpha,&image->exception) == MagickFalse)
     return(MagickFalse);
-
   (void) CloseBlob(image);
   return(MagickTrue);
 }
@@ -2712,7 +2708,7 @@ static void WriteDDSInfo(Image *image, const size_t pixelFormat,
   char
     software[MaxTextExtent];
 
-  register ssize_t
+  ssize_t
     i;
 
   unsigned int
@@ -2809,10 +2805,10 @@ static void WriteFourCC(Image *image, const size_t compression,
   const MagickBooleanType clusterFit, const MagickBooleanType weightByAlpha,
   ExceptionInfo *exception)
 {
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -2969,7 +2965,7 @@ static inline size_t ColorTo565(const DDSVector3 point)
 static void WriteIndices(Image *image, const DDSVector3 start,
   const DDSVector3 end, unsigned char* indices)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -3019,7 +3015,7 @@ static MagickBooleanType WriteMipmaps(Image *image, const size_t pixelFormat,
   Image*
     resize_image;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -3059,7 +3055,7 @@ static void WriteSingleColorFit(Image *image, const DDSVector4* points,
     start,
     end;
 
-  register ssize_t
+  ssize_t
     i;
 
   unsigned char
@@ -3083,10 +3079,10 @@ static void WriteSingleColorFit(Image *image, const DDSVector4* points,
 
 static void WriteUncompressed(Image *image, ExceptionInfo *exception)
 {
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
