@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -107,7 +107,7 @@ ModuleExport size_t RegisterDEBUGImage(void)
   entry->raw=MagickTrue;
   entry->stealth=MagickTrue;
   entry->description=ConstantString("Image pixel values for debugging");
-  entry->module=ConstantString("DEBUG");
+  entry->magick_module=ConstantString("DEBUG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -181,13 +181,13 @@ static MagickBooleanType WriteDEBUGImage(const ImageInfo *image_info,
   MagickPixelPacket
     pixel;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   size_t
