@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -113,16 +113,16 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Pixmap
     pixmap;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register size_t
+  size_t
     pixel;
 
   Screen
@@ -177,7 +177,10 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
-    return((Image *) NULL);
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Get user defaults from X resource database.
   */
@@ -339,7 +342,7 @@ static Image *ReadDPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     case DirectClass:
     default:
     {
-      register size_t
+      size_t
         color,
         index;
 
@@ -564,7 +567,7 @@ ModuleExport size_t RegisterDPSImage(void)
 #endif
   entry->blob_support=MagickFalse;
   entry->description=ConstantString("Display Postscript Interpreter");
-  entry->module=ConstantString("DPS");
+  entry->magick_module=ConstantString("DPS");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
