@@ -18,7 +18,7 @@
 %                               Dirk Lemstra                                  %
 %                               January 2014                                  %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -174,12 +174,12 @@ static MagickBooleanType IsWMF(const unsigned char *magick,const size_t length)
 #    if defined(MAGICKCORE_HAVE__WFOPEN)
 static size_t UTF8ToUTF16(const unsigned char *utf8,wchar_t *utf16)
 {
-  register const unsigned char
+  const unsigned char
     *p;
 
   if (utf16 != (wchar_t *) NULL)
     {
-      register wchar_t
+      wchar_t
         *q;
 
       wchar_t
@@ -266,7 +266,7 @@ static wchar_t *ConvertUTF8ToUTF16(const unsigned char *source)
   length=UTF8ToUTF16(source,(wchar_t *) NULL);
   if (length == 0)
     {
-      register ssize_t
+      ssize_t
         i;
 
       /*
@@ -460,10 +460,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   RECT
     rect;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   RGBQUAD
@@ -521,7 +521,7 @@ static Image *ReadEMFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       char
         *geometry;
 
-      register char
+      char
         *p;
 
       MagickStatusType
@@ -685,10 +685,10 @@ static Image *ReadEMFImage(const ImageInfo *image_info,
   MagickStatusType
     flags;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -858,7 +858,7 @@ ModuleExport size_t RegisterEMFImage(void)
     "Windows Enhanced Meta File");
   entry->magick=(IsImageFormatHandler *) IsEMF;
   entry->blob_support=MagickFalse;
-  entry->module=ConstantString("EMF");
+  entry->magick_module=ConstantString("EMF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("WMF");
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
@@ -867,7 +867,7 @@ ModuleExport size_t RegisterEMFImage(void)
   entry->description=ConstantString("Windows Meta File");
   entry->magick=(IsImageFormatHandler *) IsWMF;
   entry->blob_support=MagickFalse;
-  entry->module=ConstantString("EMF");
+  entry->magick_module=ConstantString("EMF");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
