@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -179,7 +179,7 @@ static MagickOffsetType GetFITSPixelExtrema(Image *image,
   MagickSizeType
     number_pixels;
 
-  register MagickOffsetType
+  MagickOffsetType
     i;
 
   offset=TellBlob(image);
@@ -207,7 +207,7 @@ static inline double GetFITSPixelRange(const size_t depth)
 static void SetFITSUnsignedPixels(const size_t length,
   const size_t bits_per_pixel,const EndianType endian,unsigned char *pixels)
 {
-  register ssize_t
+  ssize_t
     i;
 
   if (endian != MSBEndian)
@@ -270,11 +270,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   ssize_t
@@ -321,7 +321,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   {
     for ( ; EOFBlob(image) == MagickFalse; )
     {
-      register char
+      char
         *p;
 
       count=ReadBlob(image,8,(unsigned char *) keyword);
@@ -331,7 +331,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       {
         if (isspace((int) ((unsigned char) keyword[i])) != 0)
           break;
-        keyword[i]=tolower((int) ((unsigned char) keyword[i]));
+        keyword[i]=LocaleLowercase((int) ((unsigned char) keyword[i]));
       }
       keyword[i]='\0';
       count=ReadBlob(image,72,(unsigned char *) value);
@@ -556,7 +556,7 @@ ModuleExport size_t RegisterFITSImage(void)
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Flexible Image Transport System");
-  entry->module=ConstantString("FITS");
+  entry->magick_module=ConstantString("FITS");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FTS");
   entry->decoder=(DecodeImageHandler *) ReadFITSImage;
@@ -565,7 +565,7 @@ ModuleExport size_t RegisterFITSImage(void)
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Flexible Image Transport System");
-  entry->module=ConstantString("FITS");
+  entry->magick_module=ConstantString("FITS");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -640,8 +640,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
 {
   char
     *fits_info,
-    header[FITSBlocksize],
-    *url;
+    header[FITSBlocksize];
 
   MagickBooleanType
     status;
@@ -649,7 +648,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   QuantumInfo
     *quantum_info;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   size_t
@@ -727,7 +726,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) FormatLocaleString(header,FITSBlocksize,"BZERO   =         %E",
-    image->depth > 8 ? GetFITSPixelRange(image->depth)/2.0 : 0.0);
+    image->depth > 8 ? (GetFITSPixelRange(image->depth)+1)/2.0 : 0.0);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) FormatLocaleString(header,FITSBlocksize,"DATAMAX =         %E",
@@ -743,9 +742,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
       CopyFitsRecord(fits_info,header,offset);
       offset+=80;
     }
-  url=GetMagickHomeURL();
-  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",url);
-  url=DestroyString(url);
+  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",
+    MagickAuthoritativeURL);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) strncpy(header,"END",FITSBlocksize);
