@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -110,10 +110,10 @@ static Image *ReadXCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   PixelPacket
     pixel;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   ssize_t
@@ -291,7 +291,7 @@ ModuleExport size_t RegisterGRADIENTImage(void)
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Gradual linear passing from one shade to "
     "another");
-  entry->module=ConstantString("GRADIENT");
+  entry->magick_module=ConstantString("GRADIENT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RADIAL-GRADIENT");
   entry->decoder=(DecodeImageHandler *) ReadGRADIENTImage;
@@ -300,7 +300,7 @@ ModuleExport size_t RegisterGRADIENTImage(void)
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Gradual radial passing from one shade to "
     "another");
-  entry->module=ConstantString("GRADIENT");
+  entry->magick_module=ConstantString("GRADIENT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
