@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -153,13 +153,13 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status,
     value_expected;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -176,12 +176,12 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -201,14 +201,14 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  while (isgraph(c) && (image->columns == 0) && (image->rows == 0))
+  while (isgraph((int) ((unsigned char) c)) && (image->columns == 0) && (image->rows == 0))
   {
     if (c == (int) '#')
       {
         char
           *comment;
 
-        register char
+        char
           *p;
 
         size_t
@@ -244,11 +244,11 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
         c=ReadBlobByte(image);
       }
     else
-      if (isalnum(c) == MagickFalse)
+      if (isalnum((int) ((unsigned char) c)) == MagickFalse)
         c=ReadBlobByte(image);
       else
         {
-          register char
+          char
             *p;
 
           /*
@@ -260,7 +260,7 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if ((size_t) (p-keyword) < (MaxTextExtent-1))
               *p++=c;
             c=ReadBlobByte(image);
-          } while (isalnum(c) || (c == '_'));
+          } while (isalnum((int) ((unsigned char) c)) || (c == '_'));
           *p='\0';
           value_expected=MagickFalse;
           while ((isspace((int) ((unsigned char) c)) != 0) || (c == '='))
@@ -406,6 +406,7 @@ static Image *ReadHDRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(pixels,0,4*image->columns*sizeof(*pixels));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     if (image->compression != RLECompression)
@@ -540,7 +541,7 @@ ModuleExport size_t RegisterHDRImage(void)
   entry->decoder=(DecodeImageHandler *) ReadHDRImage;
   entry->encoder=(EncodeImageHandler *) WriteHDRImage;
   entry->description=ConstantString("Radiance RGBE image format");
-  entry->module=ConstantString("HDR");
+  entry->magick_module=ConstantString("HDR");
   entry->magick=(IsImageFormatHandler *) IsHDR;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -600,7 +601,7 @@ static size_t HDRWriteRunlengthPixels(Image *image,unsigned char *pixels)
 {
 #define MinimumRunlength 4
 
-  register size_t
+  size_t
     p,
     q;
 
@@ -671,10 +672,10 @@ static MagickBooleanType WriteHDRImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -693,9 +694,9 @@ static MagickBooleanType WriteHDRImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -706,14 +707,15 @@ static MagickBooleanType WriteHDRImage(const ImageInfo *image_info,Image *image)
   /*
     Write header.
   */
-  (void) ResetMagickMemory(header,' ',MaxTextExtent);
-  length=CopyMagickString(header,"#?RGBE\n",MaxTextExtent);
+  (void) memset(header,' ',MaxTextExtent);
+  length=CopyMagickString(header,"#?RADIANCE\n",MaxTextExtent);
   (void) WriteBlob(image,length,(unsigned char *) header);
   property=GetImageProperty(image,"comment");
   if ((property != (const char *) NULL) &&
       (strchr(property,'\n') == (char *) NULL))
     {
-      count=FormatLocaleString(header,MaxTextExtent,"#%s\n",property);
+      count=FormatLocaleString(header,MagickPathExtent,"#%.*s\n",
+        MagickPathExtent-3,property); 
       (void) WriteBlob(image,(size_t) count,(unsigned char *) header);
     }
   property=GetImageProperty(image,"hdr:exposure");
@@ -747,7 +749,7 @@ static MagickBooleanType WriteHDRImage(const ImageInfo *image_info,Image *image)
     sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(pixels,0,4*(image->columns+128)*sizeof(*pixels));
+  (void) memset(pixels,0,4*(image->columns+128)*sizeof(*pixels));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
@@ -784,9 +786,12 @@ static MagickBooleanType WriteHDRImage(const ImageInfo *image_info,Image *image)
             exponent;
 
           gamma=frexp(gamma,&exponent)*256.0/gamma;
-          pixel[0]=(unsigned char) (gamma*QuantumScale*GetPixelRed(p));
-          pixel[1]=(unsigned char) (gamma*QuantumScale*GetPixelGreen(p));
-          pixel[2]=(unsigned char) (gamma*QuantumScale*GetPixelBlue(p));
+          if (GetPixelRed(p) > 0)
+            pixel[0]=(unsigned char) (gamma*QuantumScale*GetPixelRed(p));
+          if (GetPixelGreen(p) > 0)
+            pixel[1]=(unsigned char) (gamma*QuantumScale*GetPixelGreen(p));
+          if (GetPixelBlue(p) > 0)
+            pixel[2]=(unsigned char) (gamma*QuantumScale*GetPixelBlue(p));
           pixel[3]=(unsigned char) (exponent+128);
         }
       if ((image->columns >= 8) && (image->columns <= 0x7ffff))
