@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,13 +100,13 @@ static Image *ReadHRZImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   ssize_t
@@ -218,7 +218,7 @@ ModuleExport size_t RegisterHRZImage(void)
   entry->encoder=(EncodeImageHandler *) WriteHRZImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Slow Scan TeleVision");
-  entry->module=ConstantString("HRZ");
+  entry->magick_module=ConstantString("HRZ");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -279,14 +279,14 @@ static MagickBooleanType WriteHRZImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x,
     y;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
