@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,7 +95,7 @@ static MagickBooleanType
 */
 static MagickBooleanType IsHTML(const unsigned char *magick,const size_t length)
 {
-  if (length < 5)
+  if (length < 6)
     return(MagickFalse);
   if (LocaleNCompare((char *) magick+1,"html",5) == 0)
     return(MagickTrue);
@@ -136,7 +136,7 @@ ModuleExport size_t RegisterHTMLImage(void)
   entry->adjoin=MagickFalse;
   entry->description=ConstantString(
     "Hypertext Markup Language and a client-side image map");
-  entry->module=ConstantString("HTML");
+  entry->magick_module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTML");
   entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
@@ -144,7 +144,7 @@ ModuleExport size_t RegisterHTMLImage(void)
   entry->adjoin=MagickFalse;
   entry->description=ConstantString(
     "Hypertext Markup Language and a client-side image map");
-  entry->module=ConstantString("HTML");
+  entry->magick_module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SHTML");
   entry->encoder=(EncodeImageHandler *) WriteHTMLImage;
@@ -152,7 +152,7 @@ ModuleExport size_t RegisterHTMLImage(void)
   entry->adjoin=MagickFalse;
   entry->description=ConstantString(
     "Hypertext Markup Language and a client-side image map");
-  entry->module=ConstantString("HTML");
+  entry->magick_module=ConstantString("HTML");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -230,7 +230,7 @@ static MagickBooleanType WriteHTMLImage(const ImageInfo *image_info,
   RectangleInfo
     geometry;
 
-  register char
+  char
     *p;
 
   /*
