@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -101,7 +101,7 @@ ModuleExport size_t RegisterINFOImage(void)
   entry->encoder=(EncodeImageHandler *) WriteINFOImage;
   entry->blob_support=MagickFalse;
   entry->description=ConstantString("The image format and characteristics");
-  entry->module=ConstantString("INFO");
+  entry->magick_module=ConstantString("INFO");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
