@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -71,7 +71,7 @@
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
-#include "jbig.h"
+#include <jbig.h>
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
@@ -125,16 +125,16 @@ static Image *ReadJBIGImage(const ImageInfo *image_info,
   MagickStatusType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   ssize_t
@@ -327,7 +327,7 @@ ModuleExport size_t RegisterJBIGImage(void)
   entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
-  entry->module=ConstantString("JBIG");
+  entry->magick_module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBG");
 #if defined(MAGICKCORE_JBIG_DELEGATE)
@@ -337,7 +337,7 @@ ModuleExport size_t RegisterJBIGImage(void)
   entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
-  entry->module=ConstantString("JBIG");
+  entry->magick_module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JBIG");
 #if defined(MAGICKCORE_JBIG_DELEGATE)
@@ -347,7 +347,7 @@ ModuleExport size_t RegisterJBIGImage(void)
   entry->description=ConstantString(JBIGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
-  entry->module=ConstantString("JBIG");
+  entry->magick_module=ConstantString("JBIG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -429,13 +429,13 @@ static MagickBooleanType WriteJBIGImage(const ImageInfo *image_info,
   MemoryInfo
     *pixel_info;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
