@@ -16,7 +16,7 @@
 %                                 July 2012                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -141,7 +141,7 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -202,7 +202,7 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *jnx_level_info[i].copyright='\0';
     if (jnx_info.version > 3)
       {
-        register ssize_t
+        ssize_t
           j;
 
         unsigned short
@@ -227,7 +227,7 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     MagickOffsetType
       offset;
 
-    register ssize_t
+    ssize_t
       j;
 
     offset=SeekBlob(image,(MagickOffsetType) jnx_level_info[i].offset,SEEK_SET);
@@ -267,7 +267,10 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) ReadBlobLSBShort(image); /* width */
       (void) ReadBlobLSBShort(image); /* height */
       if (EOFBlob(image) != MagickFalse)
-        ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
+        {
+          images=DestroyImageList(images);
+          ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
+        }
       tile_length=ReadBlobLSBLong(image);
       tile_offset=ReadBlobLSBSignedLong(image);
       if (tile_offset == -1)
@@ -282,13 +285,15 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         Read a tile.
       */
       if (((MagickSizeType) tile_length) > GetBlobSize(image))
-        ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+        {
+          images=DestroyImageList(images);
+          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+        }
       blob=(unsigned char *) AcquireQuantumMemory((size_t) tile_length+2,
         sizeof(*blob));
       if (blob == (unsigned char *) NULL)
         {
-          if (images != (Image *) NULL)
-            images=DestroyImageList(images);
+          images=DestroyImageList(images);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       blob[0]=0xFF;
@@ -296,8 +301,7 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       count=ReadBlob(image,tile_length,blob+2);
       if (count != (ssize_t) tile_length)
         {
-          if (images != (Image *) NULL)
-            images=DestroyImageList(images);
+          images=DestroyImageList(images);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
         }
@@ -330,8 +334,6 @@ static Image *ReadJNXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   (void) CloseBlob(image);
   image=DestroyImage(image);
-  if (images == (Image *) NULL)
-    return((Image *) NULL);
   return(GetFirstImageInList(images));
 }
 
@@ -366,7 +368,7 @@ ModuleExport size_t RegisterJNXImage(void)
   entry->decoder=(DecodeImageHandler *) ReadJNXImage;
   entry->description=ConstantString("Garmin tile format");
   entry->seekable_stream=MagickTrue;
-  entry->module=ConstantString("JNX");
+  entry->magick_module=ConstantString("JNX");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
