@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,16 +95,16 @@ static Image *ReadMACImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -172,9 +172,10 @@ static Image *ReadMACImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Convert MAC raster image to pixel packets.
   */
   length=(image->columns+7)/8;
-  pixels=(unsigned char *) AcquireQuantumMemory(length+1,sizeof(*pixels));
+  pixels=(unsigned char *) AcquireQuantumMemory(length+257,sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(pixels,0,(length+257)*sizeof(*pixels));
   p=pixels;
   offset=0;
   for (y=0; y < (ssize_t) image->rows; )
@@ -289,7 +290,7 @@ ModuleExport size_t RegisterMACImage(void)
   entry=SetMagickInfo("MAC");
   entry->decoder=(DecodeImageHandler *) ReadMACImage;
   entry->description=ConstantString("MAC Paint");
-  entry->module=ConstantString("MAC");
+  entry->magick_module=ConstantString("MAC");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
