@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -108,19 +108,19 @@ static Image *ReadMAPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -295,7 +295,7 @@ ModuleExport size_t RegisterMAPImage(void)
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
   entry->description=ConstantString("Colormap intensities and indices");
-  entry->module=ConstantString("MAP");
+  entry->magick_module=ConstantString("MAP");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -355,17 +355,17 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -452,7 +452,7 @@ static MagickBooleanType WriteMAPImage(const ImageInfo *image_info,Image *image)
     {
       if (image->colors > 256)
         *q++=(unsigned char) ((size_t) GetPixelIndex(indexes+x) >> 8);
-      *q++=(unsigned char) GetPixelIndex(indexes+x);
+      *q++=(unsigned char) ((size_t) GetPixelIndex(indexes+x));
     }
     (void) WriteBlob(image,(size_t) (q-pixels),pixels);
   }
