@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -103,13 +103,13 @@ static Image *ReadMTVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   ssize_t
@@ -276,7 +276,7 @@ ModuleExport size_t RegisterMTVImage(void)
   entry->decoder=(DecodeImageHandler *) ReadMTVImage;
   entry->encoder=(EncodeImageHandler *) WriteMTVImage;
   entry->description=ConstantString("MTV Raytracing image format");
-  entry->module=ConstantString("MTV");
+  entry->magick_module=ConstantString("MTV");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -341,13 +341,13 @@ static MagickBooleanType WriteMTVImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
