@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -105,13 +105,13 @@ static Image *ReadNULLImage(const ImageInfo *image_info,
   MagickPixelPacket
     background;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   ssize_t
@@ -195,7 +195,7 @@ ModuleExport size_t RegisterNULLImage(void)
   entry->adjoin=MagickFalse;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Constant image of uniform color");
-  entry->module=ConstantString("NULL");
+  entry->magick_module=ConstantString("NULL");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
