@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -126,7 +126,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
         count=ReadBlob(image,0x800,buffer); \
         p=buffer; \
       } \
-    sum|=((unsigned int) (*p) << (24-bits)); \
+    sum|=(((unsigned int) (*p)) << (24-bits)); \
     bits+=8; \
     p++; \
   } \
@@ -148,14 +148,14 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   PCDTable
     *pcd_table[3];
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
-  register PCDTable
+  PCDTable
     *r;
 
-  register unsigned char
+  unsigned char
     *p,
     *q;
 
@@ -196,7 +196,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     pcd_table[i]=(PCDTable *) NULL;
     pcd_length[i]=0;
   }
-  for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
   {
     PCDGetBits(8);
     length=(sum & 0xff)+1;
@@ -205,6 +205,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     if (pcd_table[i] == (PCDTable *) NULL)
       {
         buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+        for (j=0; j < i; j++)
+          pcd_table[j]=(PCDTable *) RelinquishMagickMemory(pcd_table[j]);
         ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
       }
@@ -216,6 +218,8 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
       if (r->length > 16)
         {
           buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+          for (j=0; j <= i; j++)
+            pcd_table[j]=(PCDTable *) RelinquishMagickMemory(pcd_table[j]);
           return(MagickFalse);
         }
       PCDGetBits(16);
@@ -227,17 +231,20 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
     }
     pcd_length[i]=(size_t) length;
   }
-  /*
-    Search for Sync byte.
-  */
-  for (i=0; i < 1; i++)
-    PCDGetBits(16);
-  for (i=0; i < 1; i++)
-    PCDGetBits(16);
-  while ((sum & 0x00fff000UL) != 0x00fff000UL)
-    PCDGetBits(8);
-  while (IsSync(sum) == 0)
-    PCDGetBits(1);
+  if (EOFBlob(image) == MagickFalse)
+    {
+      /*
+        Search for Sync byte.
+      */
+      for (i=0; i < 1; i++)
+        PCDGetBits(16);
+      for (i=0; i < 1; i++)
+        PCDGetBits(16);
+      while ((sum & 0x00fff000UL) != 0x00fff000UL)
+        PCDGetBits(8);
+      while (IsSync(sum) == 0)
+        PCDGetBits(1);
+    }
   /*
     Recover the Huffman encoded luminance and chrominance deltas.
   */
@@ -245,8 +252,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   length=0;
   plane=0;
   row=0;
-  q=luma;
-  for ( ; ; )
+  for (q=luma; EOFBlob(image) == MagickFalse; )
   {
     if (IsSync(sum) != 0)
       {
@@ -284,7 +290,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
           }
           default:
           {
-            for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+            for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
               pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
             buffer=(unsigned char *) RelinquishMagickMemory(buffer);
             ThrowBinaryImageException(CorruptImageError,"CorruptImage",
@@ -322,7 +328,7 @@ static MagickBooleanType DecodeImage(Image *image,unsigned char *luma,
   /*
     Relinquish resources.
   */
-  for (i=0; i < (image->columns > 1536 ? 3 : 1); i++)
+  for (i=0; i < (ssize_t) (image->columns > 1536 ? 3 : 1); i++)
     pcd_table[i]=(PCDTable *) RelinquishMagickMemory(pcd_table[i]);
   buffer=(unsigned char *) RelinquishMagickMemory(buffer);
   return(MagickTrue);
@@ -399,7 +405,7 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
   MontageInfo
     *montage_info;
 
-  register Image
+  Image
     *p;
 
   /*
@@ -424,11 +430,11 @@ static Image *OverviewImage(const ImageInfo *image_info,Image *image,
 static void Upsample(const size_t width,const size_t height,
   const size_t scaled_width,unsigned char *pixels)
 {
-  register ssize_t
+  ssize_t
     x,
     y;
 
-  register unsigned char
+  unsigned char
     *p,
     *q,
     *r;
@@ -479,12 +485,8 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
 { \
   if (header != (unsigned char *) NULL) \
     header=(unsigned char *) RelinquishMagickMemory(header); \
-  if (luma != (unsigned char *) NULL) \
-    luma=(unsigned char *) RelinquishMagickMemory(luma); \
-  if (chroma2 != (unsigned char *) NULL) \
-    chroma2=(unsigned char *) RelinquishMagickMemory(chroma2); \
-  if (chroma1 != (unsigned char *) NULL) \
-    chroma1=(unsigned char *) RelinquishMagickMemory(chroma1); \
+  if (pixel_info != (MemoryInfo *) NULL) \
+    pixel_info=RelinquishVirtualMemory(pixel_info); \
   ThrowReaderException((exception),(message)); \
 }
 
@@ -497,17 +499,17 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  MagickSizeType
-    number_pixels;
+  MemoryInfo
+    *pixel_info;
 
-  register ssize_t
+  ssize_t
     i,
     y;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *c1,
     *c2,
     *yy;
@@ -515,6 +517,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   size_t
     height,
     number_images,
+    number_pixels,
     rotate,
     scene,
     width;
@@ -555,9 +558,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   header=(unsigned char *) AcquireQuantumMemory(0x800,3UL*sizeof(*header));
   if (header == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  chroma1=(unsigned char *) NULL;
-  chroma2=(unsigned char *) NULL;
-  luma=(unsigned char *) NULL;
+  pixel_info=(MemoryInfo *) NULL;
   count=ReadBlob(image,3*0x800,header);
   if (count != (3*0x800))
     ThrowPCDException(CorruptImageError,"ImproperImageHeader");
@@ -627,24 +628,15 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Allocate luma and chroma memory.
   */
-  number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (number_pixels != (size_t) number_pixels)
+  pixel_info=AcquireVirtualMemory(image->columns+1UL,30*image->rows*
+    sizeof(*luma));
+  if (pixel_info == (MemoryInfo *) NULL)
     ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
-  chroma1=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    10*sizeof(*chroma1));
-  chroma2=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    10*sizeof(*chroma2));
-  luma=(unsigned char *) AcquireQuantumMemory(image->columns+1UL,image->rows*
-    10*sizeof(*luma));
-  if ((chroma1 == (unsigned char *) NULL) ||
-      (chroma2 == (unsigned char *) NULL) || (luma == (unsigned char *) NULL))
-    ThrowPCDException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) memset(chroma1,0,(image->columns+1UL)*image->rows*
-    10*sizeof(*chroma1));
-  (void) memset(chroma2,0,(image->columns+1UL)*image->rows*
-    10*sizeof(*chroma2));
-  (void) memset(luma,0,(image->columns+1UL)*image->rows*
-    10*sizeof(*luma));
+  number_pixels=(image->columns+1UL)*10*image->rows*sizeof(*luma);
+  luma=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
+  chroma1=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+number_pixels;
+  chroma2=(unsigned char *) GetVirtualMemoryBlob(pixel_info)+2*number_pixels;
+  (void) memset(luma,0,3*number_pixels);
   /*
     Advance to image data.
   */
@@ -665,7 +657,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
       MagickProgressMonitor
         progress_monitor;
 
-      register ssize_t
+      ssize_t
         j;
 
       /*
@@ -744,14 +736,13 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->client_data);
         if (image->previous == (Image *) NULL)
           {
-            status=SetImageProgress(image,LoadImageTag,j-1,number_images);
+            status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) j-1,
+              number_images);
             if (status == MagickFalse)
               break;
           }
       }
-      chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
-      chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
-      luma=(unsigned char *) RelinquishMagickMemory(luma);
+      pixel_info=RelinquishVirtualMemory(pixel_info);
       if (status == MagickFalse)
         return(DestroyImageList(image));
       return(OverviewImage(image_info,GetFirstImageInList(image),exception));
@@ -841,9 +832,7 @@ static Image *ReadPCDImage(const ImageInfo *image_info,ExceptionInfo *exception)
           break;
       }
   }
-  chroma2=(unsigned char *) RelinquishMagickMemory(chroma2);
-  chroma1=(unsigned char *) RelinquishMagickMemory(chroma1);
-  luma=(unsigned char *) RelinquishMagickMemory(luma);
+  pixel_info=RelinquishVirtualMemory(pixel_info);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
@@ -924,7 +913,7 @@ ModuleExport size_t RegisterPCDImage(void)
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photo CD");
-  entry->module=ConstantString("PCD");
+  entry->magick_module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PCDS");
   entry->decoder=(DecodeImageHandler *) ReadPCDImage;
@@ -932,7 +921,7 @@ ModuleExport size_t RegisterPCDImage(void)
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photo CD");
-  entry->module=ConstantString("PCD");
+  entry->magick_module=ConstantString("PCD");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1006,11 +995,11 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
   RectangleInfo
     geometry;
 
-  register const PixelPacket
+  const PixelPacket
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1094,7 +1083,8 @@ static MagickBooleanType WritePCDTile(Image *image,const char *page_geometry,
       (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(q)));
       q++;
     }
-    status=SetImageProgress(image,SaveImageTag,y,tile_image->rows);
+    status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+      tile_image->rows);
     if (status == MagickFalse)
       break;
   }
@@ -1113,7 +1103,7 @@ static MagickBooleanType WritePCDImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(image_info != (const ImageInfo *) NULL);
