@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -182,7 +182,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     bounding_box,
     page;
 
-  register char
+  char
     *p;
 
   SegmentInfo
@@ -319,7 +319,10 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
      else
        delegate_info=GetDelegateInfo("pcl:color",(char *) NULL,exception);
   if (delegate_info == (const DelegateInfo *) NULL)
-    return((Image *) NULL);
+    {
+      image=DestroyImage(image);
+      return((Image *) NULL);
+    }
   if ((page.width == 0) || (page.height == 0))
     (void) ParseAbsoluteGeometry(PSPageGeometry,&page);
   if (image_info->page != (char *) NULL)
@@ -328,6 +331,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   options=AcquireString("");
   (void) FormatLocaleString(density,MaxTextExtent,"%gx%g",
     image->x_resolution,image->y_resolution);
+  if (image_info->ping != MagickFalse)
+    (void) FormatLocaleString(density,MagickPathExtent,"2.0x2.0");
   page.width=(size_t) floor((double) page.width*image->x_resolution/delta.x+
     0.5);
   page.height=(size_t) floor((double) page.height*image->y_resolution/delta.y+
@@ -383,6 +388,13 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   {
     (void) CopyMagickString(image->filename,filename,MaxTextExtent);
     image->page=page;
+    if (image_info->ping != MagickFalse)
+      {
+        image->magick_columns*=image->x_resolution/2.0;
+        image->magick_rows*=image->y_resolution/2.0;
+        image->columns*=image->x_resolution/2.0;
+        image->rows*=image->y_resolution/2.0;
+      }
     next_image=SyncNextImageInList(image);
     if (next_image != (Image *) NULL)
       image=next_image;
@@ -426,7 +438,7 @@ ModuleExport size_t RegisterPCLImage(void)
   entry->seekable_stream=MagickTrue;
   entry->thread_support=EncoderThreadSupport;
   entry->description=ConstantString("Printer Control Language");
-  entry->module=ConstantString("PCL");
+  entry->magick_module=ConstantString("PCL");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -490,11 +502,11 @@ static size_t PCLDeltaCompressImage(const size_t length,
     j,
     replacement;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   q=compress_pixels;
@@ -558,10 +570,10 @@ static size_t PCLPackbitsCompressImage(const size_t length,
   int
     count;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -670,17 +682,17 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   MagickOffsetType
     scene;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -868,7 +880,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
       {
         case 1:
         {
-          register unsigned char
+          unsigned char
             bit,
             byte;
 
@@ -901,7 +913,7 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
             Colormapped image.
           */
           for (x=0; x < (ssize_t) image->columns; x++)
-            *q++=(unsigned char) GetPixelIndex(indexes+x);
+            *q++=(unsigned char) ((size_t) GetPixelIndex(indexes+x));
           break;
         }
         case 24:
