@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -326,19 +326,19 @@ static MagickBooleanType
 %
 */
 
-static unsigned char *ExpandBuffer(unsigned char *pixels,
-  MagickSizeType *bytes_per_line,const unsigned int bits_per_pixel)
+static const unsigned char *UnpackScanline(
+  const unsigned char *magick_restrict pixels,const unsigned int bits_per_pixel,
+    unsigned char *scanline,MagickSizeType *bytes_per_line)
 {
-  register ssize_t
+  const unsigned char
+    *p;
+
+  ssize_t
     i;
 
-  register unsigned char
-    *p,
+  unsigned char
     *q;
 
-  static unsigned char
-    scanline[8*256];
-
   p=pixels;
   q=scanline;
   switch (bits_per_pixel)
@@ -403,11 +403,13 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
   MagickSizeType
     number_pixels;
 
-  register ssize_t
+  const unsigned char
+    *p;
+
+  ssize_t
     i;
 
-  register unsigned char
-    *p,
+  unsigned char
     *q;
 
   size_t
@@ -424,7 +426,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
 
   unsigned char
     *pixels,
-    *scanline;
+    *scanline,
+    unpack_buffer[8*256];
 
   /*
     Determine pixel buffer size.
@@ -463,6 +466,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
       return((unsigned char *) NULL);
     }
   (void) memset(scanline,0,2*row_bytes*sizeof(*scanline));
+  (void) memset(unpack_buffer,0,sizeof(unpack_buffer));
   status=MagickTrue;
   if (bytes_per_line < 8)
     {
@@ -479,7 +483,7 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
             status=MagickFalse;
             break;
           }
-        p=ExpandBuffer(scanline,&number_pixels,bits_per_pixel);
+        p=UnpackScanline(scanline,bits_per_pixel,unpack_buffer,&number_pixels);
         if ((q+number_pixels) > (pixels+(*extent)))
           {
             status=MagickFalse;
@@ -518,7 +522,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
         {
           length=(size_t) ((scanline[j] & 0xff)+1);
           number_pixels=length*bytes_per_pixel;
-          p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
+          p=UnpackScanline(scanline+j+1,bits_per_pixel,unpack_buffer,
+            &number_pixels);
           if ((q-pixels+number_pixels) <= *extent)
             (void) memcpy(q,p,(size_t) number_pixels);
           q+=number_pixels;
@@ -528,7 +533,8 @@ static unsigned char *DecodeImage(Image *blob,Image *image,
         {
           length=(size_t) (((scanline[j] ^ 0xff) & 0xff)+2);
           number_pixels=bytes_per_pixel;
-          p=ExpandBuffer(scanline+j+1,&number_pixels,bits_per_pixel);
+          p=UnpackScanline(scanline+j+1,bits_per_pixel,unpack_buffer,
+            &number_pixels);
           for (i=0; i < (ssize_t) length; i++)
           {
             if ((q-pixels+number_pixels) <= *extent)
@@ -581,13 +587,13 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 #define MaxCount  128
 #define MaxPackbitsRunlength  128
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -803,9 +809,11 @@ static MagickBooleanType ReadRectangle(Image *image,PICTRectangle *rectangle)
   rectangle->left=(short) ReadBlobMSBShort(image);
   rectangle->bottom=(short) ReadBlobMSBShort(image);
   rectangle->right=(short) ReadBlobMSBShort(image);
-  if ((EOFBlob(image) != MagickFalse) ||
-      ((rectangle->bottom-rectangle->top) <= 0) ||
-      ((rectangle->right-rectangle->left) <= 0))
+  if (((EOFBlob(image) != MagickFalse) ||
+      (((rectangle->bottom | rectangle->top |
+         rectangle->right | rectangle->left ) & 0x8000) != 0) ||
+      (rectangle->bottom < rectangle->top) ||
+      (rectangle->right < rectangle->left)))
     return(MagickFalse);
   return(MagickTrue);
 }
@@ -850,16 +858,16 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   PICTPixmap
     pixmap;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -969,6 +977,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       code=ReadBlobByte(image);
     if (version == 2)
       code=ReadBlobMSBSignedShort(image);
+    code&=0xffff;
     if (code < 0)
       break;
     if (code == 0)
@@ -991,7 +1000,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               Clipping rectangle.
             */
             length=ReadBlobMSBShort(image);
-            if (length > GetBlobSize(image))
+            if ((MagickSizeType) length > GetBlobSize(image))
               ThrowPICTException(CorruptImageError,
                 "InsufficientImageDataInFile");
             if (length != 0x000a)
@@ -1007,6 +1016,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               break;
             image->columns=(size_t) (frame.right-frame.left);
             image->rows=(size_t) (frame.bottom-frame.top);
+            if (image_info->ping != MagickFalse)
+              break;
             status=SetImageExtent(image,image->columns,image->rows);
             if (status != MagickFalse)
               status=ResetImagePixels(image,&image->exception);
@@ -1045,7 +1056,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if (pattern != 1)
               ThrowPICTException(CorruptImageError,"UnknownPatternType");
             length=ReadBlobMSBShort(image);
-            if (length > GetBlobSize(image))
+            if ((MagickSizeType) length > GetBlobSize(image))
               ThrowPICTException(CorruptImageError,
                 "InsufficientImageDataInFile");
             if (ReadRectangle(image,&frame) == MagickFalse)
@@ -1059,7 +1070,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             (void) ReadBlobMSBLong(image);
             flags=(ssize_t) ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
-            if (length > GetBlobSize(image))
+            if ((MagickSizeType) length > GetBlobSize(image))
               ThrowPICTException(CorruptImageError,
                 "InsufficientImageDataInFile");
             for (i=0; i <= (ssize_t) length; i++)
@@ -1081,18 +1092,21 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             else
               for (i=0; i < (ssize_t) height; i++)
               {
+                 size_t
+                  scanline_length;
+
                 if (EOFBlob(image) != MagickFalse)
                   break;
                 if (length > 200)
-                  {
-                    for (j=0; j < (ssize_t) ReadBlobMSBShort(image); j++)
-                      if (ReadBlobByte(image) == EOF)
-                        break;
-                  }
+                  scanline_length=ReadBlobMSBShort(image);
                 else
-                  for (j=0; j < (ssize_t) ReadBlobByte(image); j++)
-                    if (ReadBlobByte(image) == EOF)
-                      break;
+                  scanline_length=ReadBlobByte(image);
+                if ((MagickSizeType) scanline_length > GetBlobSize(image))
+                  ThrowPICTException(CorruptImageError,
+                    "InsufficientImageDataInFile");
+                for (j=0; j < (ssize_t) scanline_length; j++)
+                  if (ReadBlobByte(image) == EOF)
+                    break;
               }
             break;
           }
@@ -1122,7 +1136,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               Skip polygon or region.
             */
             length=ReadBlobMSBShort(image);
-            if (length > GetBlobSize(image))
+            if ((MagickSizeType) length > GetBlobSize(image))
               ThrowPICTException(CorruptImageError,
                 "InsufficientImageDataInFile");
             for (i=0; i < (ssize_t) (length-2); i++)
@@ -1144,7 +1158,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               source,
               destination;
 
-            register unsigned char
+            unsigned char
               *p;
 
             size_t
@@ -1174,6 +1188,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               (size_t) (frame.bottom-frame.top),MagickTrue,exception);
             if (tile_image == (Image *) NULL)
               ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+            status=ResetImagePixels(tile_image,exception);
             if ((code == 0x9a) || (code == 0x9b) ||
                 ((bytes_per_line & 0x8000) != 0))
               {
@@ -1246,7 +1261,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   Skip region.
                 */
                 length=ReadBlobMSBShort(image);
-                if (length > GetBlobSize(image))
+                if ((MagickSizeType) length > GetBlobSize(image))
                   ThrowPICTException(CorruptImageError,
                     "InsufficientImageDataInFile");
                 for (i=0; i < (ssize_t) (length-2); i++)
@@ -1282,7 +1297,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 if (tile_image->storage_class == PseudoClass)
                   {
-                    index=ConstrainColormapIndex(tile_image,*p);
+                    index=ConstrainColormapIndex(tile_image,(ssize_t) *p);
                     SetPixelIndex(indexes+x,index);
                     SetPixelRed(q,
                       tile_image->colormap[(ssize_t) index].red);
@@ -1370,7 +1385,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             */
             type=ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
-            if (length > GetBlobSize(image))
+            if ((MagickSizeType) length > GetBlobSize(image))
               ThrowPICTException(CorruptImageError,
                 "InsufficientImageDataInFile");
             if (length == 0)
@@ -1453,64 +1468,49 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
       continue;
     if (code == 0x8200)
       {
-        char
-          filename[MaxTextExtent];
-
-        FILE
-          *file;
-
-        int
-          unique_file;
-
         /*
           Embedded JPEG.
         */
         jpeg=MagickTrue;
-        read_info=CloneImageInfo(image_info);
-        SetImageInfoBlob(read_info,(void *) NULL,0);
-        file=(FILE *) NULL;
-        unique_file=AcquireUniqueFileResource(filename);
-        (void) FormatLocaleString(read_info->filename,MaxTextExtent,"jpeg:%s",
-          filename);
-        if (unique_file != -1)
-          file=fdopen(unique_file,"wb");
-        if ((unique_file == -1) || (file == (FILE *) NULL))
-          {
-            (void) RelinquishUniqueFileResource(read_info->filename);
-            (void) CopyMagickString(image->filename,read_info->filename,
-              MaxTextExtent);
-            ThrowPICTException(FileOpenError,"UnableToCreateTemporaryFile");
-          }
         length=ReadBlobMSBLong(image);
-        if (length > GetBlobSize(image))
+        if ((MagickSizeType) length > GetBlobSize(image))
           ThrowPICTException(CorruptImageError,"InsufficientImageDataInFile");
         if (length > 154)
           {
+            const void
+              *stream;
+
+            ssize_t
+              count;
+
+            unsigned char
+              *pixels;
+
             for (i=0; i < 6; i++)
               (void) ReadBlobMSBLong(image);
             if (ReadRectangle(image,&frame) == MagickFalse)
-              {
-                (void) fclose(file);
-                (void) RelinquishUniqueFileResource(read_info->filename);
-                ThrowPICTException(CorruptImageError,"ImproperImageHeader");
-              }
+              ThrowPICTException(CorruptImageError,"ImproperImageHeader");
             for (i=0; i < 122; i++)
               if (ReadBlobByte(image) == EOF)
                 break;
-            for (i=0; i < (ssize_t) (length-154); i++)
-            {
-              c=ReadBlobByte(image);
-              if (c == EOF)
-                break;
-              if (fputc(c,file) != c)
-                break;
-            }
+            length-=154;
+            pixels=(unsigned char *) AcquireQuantumMemory(length,
+              sizeof(*pixels));
+            if (pixels == (unsigned char *) NULL)
+              ThrowPICTException(ResourceLimitError,"MemoryAllocationFailed");
+            stream=ReadBlobStream(image,length,pixels,&count);
+            if (count != (ssize_t) length)
+              {
+                pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+                ThrowPICTException(CorruptImageError,"ImproperImageHeader");
+              }
+            read_info=AcquireImageInfo();
+            (void) FormatLocaleString(read_info->filename,MaxTextExtent,
+              "jpeg:%s",image_info->filename);
+            tile_image=BlobToImage(read_info,stream,count,exception);
+            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+            read_info=DestroyImageInfo(read_info);
           }
-        (void) fclose(file);
-        (void) close(unique_file);
-        tile_image=ReadImage(read_info,exception);
-        (void) RelinquishUniqueFileResource(filename);
-        read_info=DestroyImageInfo(read_info);
         if (tile_image == (Image *) NULL)
           continue;
         (void) FormatLocaleString(geometry,MaxTextExtent,"%.20gx%.20g",
@@ -1535,7 +1535,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           Skip reserved.
         */
         length=ReadBlobMSBShort(image);
-        if (length > GetBlobSize(image))
+        if ((MagickSizeType) length > GetBlobSize(image))
           ThrowPICTException(CorruptImageError,"InsufficientImageDataInFile");
         for (i=0; i < (ssize_t) length; i++)
           if (ReadBlobByte(image) == EOF)
@@ -1548,7 +1548,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
           Skip reserved.
         */
         length=(size_t) ((code >> 7) & 0xff);
-        if (length > GetBlobSize(image))
+        if ((MagickSizeType) length > GetBlobSize(image))
           ThrowPICTException(CorruptImageError,"InsufficientImageDataInFile");
         for (i=0; i < (ssize_t) length; i++)
           if (ReadBlobByte(image) == EOF)
@@ -1596,7 +1596,7 @@ ModuleExport size_t RegisterPICTImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(IsImageFormatHandler *) IsPICT;
-  entry->module=ConstantString("PICT");
+  entry->magick_module=ConstantString("PICT");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PICT");
   entry->decoder=(DecodeImageHandler *) ReadPICTImage;
@@ -1605,7 +1605,7 @@ ModuleExport size_t RegisterPICTImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Apple Macintosh QuickDraw/PICT");
   entry->magick=(IsImageFormatHandler *) IsPICT;
-  entry->module=ConstantString("PICT");
+  entry->magick_module=ConstantString("PICT");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1698,13 +1698,13 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
     size_rectangle,
     source_rectangle;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1771,10 +1771,19 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   pixmap.table=0;
   pixmap.reserved=0;
   transfer_mode=0;
-  x_resolution=image->x_resolution != 0.0 ? image->x_resolution :
-    DefaultResolution;
-  y_resolution=image->y_resolution != 0.0 ? image->y_resolution :
-    DefaultResolution;
+  x_resolution=0.0;
+  y_resolution=0.0;
+  if ((image->x_resolution > MagickEpsilon) &&
+      (image->y_resolution > MagickEpsilon))
+    {
+      x_resolution=image->x_resolution;
+      y_resolution=image->y_resolution;
+      if (image->units == PixelsPerCentimeterResolution)
+        {
+          x_resolution*=2.54;
+          y_resolution*=2.54;
+        }
+    }
   storage_class=image->storage_class;
   if (image_info->compression == JPEGCompression)
     storage_class=DirectClass;
@@ -1793,6 +1802,9 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
   bytes_per_line=image->columns;
   if (storage_class == DirectClass)
     bytes_per_line*=image->matte != MagickFalse ? 4 : 3;
+  if ((bytes_per_line == 0) || (bytes_per_line > 0x7FFFU) ||
+      ((row_bytes+MaxCount*2U) >= 0x7FFFU))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   buffer=(unsigned char *) AcquireQuantumMemory(PictInfoSize,sizeof(*buffer));
   packed_scanline=(unsigned char *) AcquireQuantumMemory((size_t)
    (row_bytes+2*MaxCount),sizeof(*packed_scanline));
@@ -1940,7 +1952,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       (void) WriteBlobMSBShort(image,0x0000);
       (void) WriteBlobMSBShort(image,(unsigned short) y_resolution);
       (void) WriteBlobMSBShort(image,0x0000);
-      (void) WriteBlobMSBLong(image,length);
+      (void) WriteBlobMSBLong(image,(unsigned int) length);
       (void) WriteBlobMSBShort(image,0x0001);
       (void) WriteBlobMSBLong(image,0x0B466F74U);
       (void) WriteBlobMSBLong(image,0x6F202D20U);
@@ -2034,7 +2046,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
         break;
       indexes=GetVirtualIndexQueue(image);
       for (x=0; x < (ssize_t) image->columns; x++)
-        scanline[x]=(unsigned char) GetPixelIndex(indexes+x);
+        scanline[x]=(unsigned char) ((size_t) GetPixelIndex(indexes+x));
       count+=EncodeImage(image,scanline,(size_t) (row_bytes & 0x7FFF),
         packed_scanline);
       if (image->previous == (Image *) NULL)
@@ -2055,7 +2067,7 @@ static MagickBooleanType WritePICTImage(const ImageInfo *image_info,
       }
     else
       {
-        register unsigned char
+        unsigned char
           *blue,
           *green,
           *opacity,
