@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,6 +60,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/module.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/option.h"
@@ -69,7 +70,7 @@
 #include "magick/resource_.h"
 #include "magick/static.h"
 #include "magick/string_.h"
-#include "magick/module.h"
+#include "magick/timer-private.h"
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/module.h"
@@ -133,7 +134,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->encoder=(EncodeImageHandler *) WritePS3Image;
   entry->description=ConstantString("Level III Encapsulated PostScript");
   entry->mime_type=ConstantString("application/postscript");
-  entry->module=ConstantString("PS3");
+  entry->magick_module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
@@ -141,7 +142,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->encoder=(EncodeImageHandler *) WritePS3Image;
   entry->description=ConstantString("Level III PostScript");
   entry->mime_type=ConstantString("application/postscript");
-  entry->module=ConstantString("PS3");
+  entry->magick_module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
@@ -219,19 +220,19 @@ static MagickBooleanType Huffman2DEncodeImage(const ImageInfo *image_info,
   unsigned char
     *group4;
 
+  group4_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
+  if (group4_image == (Image *) NULL)
+    return(MagickFalse);
   status=MagickTrue;
   write_info=CloneImageInfo(image_info);
   (void) CopyMagickString(write_info->filename,"GROUP4:",MaxTextExtent);
   (void) CopyMagickString(write_info->magick,"GROUP4",MaxTextExtent);
-  group4_image=CloneImage(inject_image,0,0,MagickTrue,&image->exception);
-  if (group4_image == (Image *) NULL)
-    return(MagickFalse);
   group4=(unsigned char *) ImageToBlob(write_info,group4_image,&length,
     &image->exception);
   group4_image=DestroyImage(group4_image);
+  write_info=DestroyImageInfo(write_info);
   if (group4 == (unsigned char *) NULL)
     return(MagickFalse);
-  write_info=DestroyImageInfo(write_info);
   if (WriteBlob(image,length,group4) != (ssize_t) length)
     status=MagickFalse;
   group4=(unsigned char *) RelinquishMagickMemory(group4);
@@ -244,16 +245,16 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -270,6 +271,7 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   if (*pixel_info == (MemoryInfo *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   q=(unsigned char *) GetVirtualMemoryBlob(*pixel_info);
+  (void) memset(q,0,*length*sizeof(*q));
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
@@ -312,13 +314,13 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -389,16 +391,16 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -421,7 +423,7 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
       break;
     indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (ssize_t) image->columns; x++)
-      *q++=(unsigned char) GetPixelIndex(indexes+x);
+      *q++=(unsigned char) ((size_t) GetPixelIndex(indexes+x));
     if (image->previous == (Image *) NULL)
       {
         status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -455,7 +457,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   MemoryInfo
     *pixel_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -625,196 +627,192 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
 static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
 {
   static const char
-    *const PostscriptProlog[]=
-    {
-      "/ByteStreamDecodeFilter",
-      "{",
-      "  /z exch def",
-      "  /r exch def",
-      "  /c exch def",
-      "  z " PS3_NoCompression " eq { /ASCII85Decode filter } if",
-      "  z " PS3_FaxCompression " eq",
-      "  {",
-      "    <<",
-      "      /K " CCITTParam,
-      "      /Columns c",
-      "      /Rows r",
-      "    >>",
-      "    /CCITTFaxDecode filter",
-      "  } if",
-      "  z " PS3_JPEGCompression " eq { /DCTDecode filter } if",
-      "  z " PS3_LZWCompression " eq { /LZWDecode filter } if",
-      "  z " PS3_RLECompression " eq { /RunLengthDecode filter } if",
-      "  z " PS3_ZipCompression " eq { /FlateDecode filter } if",
-      "} bind def",
-      "",
-      "/DirectClassImageDict",
-      "{",
-      "  colorspace " PS3_RGBColorspace " eq",
-      "  {",
-      "    /DeviceRGB setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /DataSource pixel_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /Decode [0 1 0 1 0 1]",
-      "    >>",
-      "  }",
-      "  {",
-      "    /DeviceCMYK setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /DataSource pixel_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /Decode",
-      "        compression " PS3_JPEGCompression " eq",
-      "        { [1 0 1 0 1 0 1 0] }",
-      "        { [0 1 0 1 0 1 0 1] }",
-      "        ifelse",
-      "    >>",
-      "  }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/PseudoClassImageDict",
-      "{",
-      "  % Colors in colormap image.",
-      "  currentfile buffer readline pop",
-      "  token pop /colors exch def pop",
-      "  colors 0 eq",
-      "  {",
-      "    % Depth of grayscale image.",
-      "    currentfile buffer readline pop",
-      "    token pop /bits exch def pop",
-      "    /DeviceGray setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent bits",
-      "      /Decode [0 1]",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /DataSource pixel_stream",
-      "    >>",
-      "  }",
-      "  {",
-      "    % RGB colormap.",
-      "    /colormap colors 3 mul string def",
-      "    compression " PS3_NoCompression " eq",
-      "    { currentfile /ASCII85Decode filter colormap readstring pop pop }",
-      "    { currentfile colormap readstring pop pop }",
-      "    ifelse",
-      "    [ /Indexed /DeviceRGB colors 1 sub colormap ] setcolorspace",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 8",
-      "      /Decode [0 255]",
-      "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
-      "      /DataSource pixel_stream",
-      "    >>",
-      "  }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/NonMaskedImageDict",
-      "{",
-      "  class " PS3_PseudoClass " eq",
-      "  { PseudoClassImageDict }",
-      "  { DirectClassImageDict }",
-      "  ifelse",
-      "} bind def",
-      "",
-      "/MaskedImageDict",
-      "{",
-      "  <<",
-      "    /ImageType 3",
-      "    /InterleaveType 3",
-      "    /DataDict NonMaskedImageDict",
-      "    /MaskDict",
-      "    <<",
-      "      /ImageType 1",
-      "      /Width columns",
-      "      /Height rows",
-      "      /BitsPerComponent 1",
-      "      /DataSource mask_stream",
-      "      /MultipleDataSources false",
-      "      /ImageMatrix [ columns 0 0 rows neg 0 rows]",
-      "      /Decode [ 0 1 ]",
-      "    >>",
-      "  >>",
-      "} bind def",
-      "",
-      "/ClipImage",
-      "{} def",
-      "",
-      "/DisplayImage",
-      "{",
-      "  gsave",
-      "  /buffer 512 string def",
-      "  % Translation.",
-      "  currentfile buffer readline pop",
-      "  token pop /x exch def",
-      "  token pop /y exch def pop",
-      "  x y translate",
-      "  % Image size and font size.",
-      "  currentfile buffer readline pop",
-      "  token pop /x exch def",
-      "  token pop /y exch def pop",
-      "  currentfile buffer readline pop",
-      "  token pop /pointsize exch def pop",
-      (const char *) NULL
-    },
-    *const PostscriptEpilog[]=
-    {
-      "  x y scale",
-      "  % Clipping path.",
-      "  currentfile buffer readline pop",
-      "  token pop /clipped exch def pop",
-      "  % Showpage.",
-      "  currentfile buffer readline pop",
-      "  token pop /sp exch def pop",
-      "  % Image pixel size.",
-      "  currentfile buffer readline pop",
-      "  token pop /columns exch def",
-      "  token pop /rows exch def pop",
-      "  % Colorspace (RGB/CMYK).",
-      "  currentfile buffer readline pop",
-      "  token pop /colorspace exch def pop",
-      "  % Transparency.",
-      "  currentfile buffer readline pop",
-      "  token pop /alpha exch def pop",
-      "  % Stencil mask?",
-      "  currentfile buffer readline pop",
-      "  token pop /stencil exch def pop",
-      "  % Image class (direct/pseudo).",
-      "  currentfile buffer readline pop",
-      "  token pop /class exch def pop",
-      "  % Compression type.",
-      "  currentfile buffer readline pop",
-      "  token pop /compression exch def pop",
-      "  % Clip and render.",
-      "  /pixel_stream currentfile columns rows compression ByteStreamDecodeFilter def",
-      "  clipped { ClipImage } if",
-      "  alpha stencil not and",
-      "  { MaskedImageDict mask_stream resetfile }",
-      "  { NonMaskedImageDict }",
-      "  ifelse",
-      "  stencil { 0 setgray imagemask } { image } ifelse",
-      "  grestore",
-      "  sp { showpage } if",
-      "} bind def",
-      (const char *) NULL
-    };
+    PostscriptProlog[]=
+      "/ByteStreamDecodeFilter\n"
+      "{\n"
+      "  /z exch def\n"
+      "  /r exch def\n"
+      "  /c exch def\n"
+      "  z " PS3_NoCompression " eq { /ASCII85Decode filter } if\n"
+      "  z " PS3_FaxCompression " eq\n"
+      "  {\n"
+      "    <<\n"
+      "      /K " CCITTParam "\n"
+      "      /Columns c\n"
+      "      /Rows r\n"
+      "    >>\n"
+      "    /CCITTFaxDecode filter\n"
+      "  } if\n"
+      "  z " PS3_JPEGCompression " eq { /DCTDecode filter } if\n"
+      "  z " PS3_LZWCompression " eq { /LZWDecode filter } if\n"
+      "  z " PS3_RLECompression " eq { /RunLengthDecode filter } if\n"
+      "  z " PS3_ZipCompression " eq { /FlateDecode filter } if\n"
+      "} bind def\n"
+      "\n"
+      "/DirectClassImageDict\n"
+      "{\n"
+      "  colorspace " PS3_RGBColorspace " eq\n"
+      "  {\n"
+      "    /DeviceRGB setcolorspace\n"
+      "    <<\n"
+      "      /ImageType 1\n"
+      "      /Width columns\n"
+      "      /Height rows\n"
+      "      /BitsPerComponent 8\n"
+      "      /DataSource pixel_stream\n"
+      "      /MultipleDataSources false\n"
+      "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+      "      /Decode [0 1 0 1 0 1]\n"
+      "    >>\n"
+      "  }\n"
+      "  {\n"
+      "    /DeviceCMYK setcolorspace\n"
+      "    <<\n"
+      "      /ImageType 1\n"
+      "      /Width columns\n"
+      "      /Height rows\n"
+      "      /BitsPerComponent 8\n"
+      "      /DataSource pixel_stream\n"
+      "      /MultipleDataSources false\n"
+      "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+      "      /Decode\n"
+      "        compression " PS3_JPEGCompression " eq\n"
+      "        { [1 0 1 0 1 0 1 0] }\n"
+      "        { [0 1 0 1 0 1 0 1] }\n"
+      "        ifelse\n"
+      "    >>\n"
+      "  }\n"
+      "  ifelse\n"
+      "} bind def\n"
+      "\n"
+      "/PseudoClassImageDict\n"
+      "{\n"
+      "  % Colors in colormap image.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /colors exch def pop\n"
+      "  colors 0 eq\n"
+      "  {\n"
+      "    % Depth of grayscale image.\n"
+      "    currentfile buffer readline pop\n"
+      "    token pop /bits exch def pop\n"
+      "    /DeviceGray setcolorspace\n"
+      "    <<\n"
+      "      /ImageType 1\n"
+      "      /Width columns\n"
+      "      /Height rows\n"
+      "      /BitsPerComponent bits\n"
+      "      /Decode [0 1]\n"
+      "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+      "      /DataSource pixel_stream\n"
+      "    >>\n"
+      "  }\n"
+      "  {\n"
+      "    % RGB colormap.\n"
+      "    /colormap colors 3 mul string def\n"
+      "    compression " PS3_NoCompression " eq\n"
+      "    { currentfile /ASCII85Decode filter colormap readstring pop pop }\n"
+      "    { currentfile colormap readstring pop pop }\n"
+      "    ifelse\n"
+      "    [ /Indexed /DeviceRGB colors 1 sub colormap ] setcolorspace\n"
+      "    <<\n"
+      "      /ImageType 1\n"
+      "      /Width columns\n"
+      "      /Height rows\n"
+      "      /BitsPerComponent 8\n"
+      "      /Decode [0 255]\n"
+      "      /ImageMatrix [columns 0 0 rows neg 0 rows]\n"
+      "      /DataSource pixel_stream\n"
+      "    >>\n"
+      "  }\n"
+      "  ifelse\n"
+      "} bind def\n"
+      "\n"
+      "/NonMaskedImageDict\n"
+      "{\n"
+      "  class " PS3_PseudoClass " eq\n"
+      "  { PseudoClassImageDict }\n"
+      "  { DirectClassImageDict }\n"
+      "  ifelse\n"
+      "} bind def\n"
+      "\n"
+      "/MaskedImageDict\n"
+      "{\n"
+      "  <<\n"
+      "    /ImageType 3\n"
+      "    /InterleaveType 3\n"
+      "    /DataDict NonMaskedImageDict\n"
+      "    /MaskDict\n"
+      "    <<\n"
+      "      /ImageType 1\n"
+      "      /Width columns\n"
+      "      /Height rows\n"
+      "      /BitsPerComponent 1\n"
+      "      /DataSource mask_stream\n"
+      "      /MultipleDataSources false\n"
+      "      /ImageMatrix [ columns 0 0 rows neg 0 rows]\n"
+      "      /Decode [ 0 1 ]\n"
+      "    >>\n"
+      "  >>\n"
+      "} bind def\n"
+      "\n"
+      "/ClipImage\n"
+      "{} def\n"
+      "\n"
+      "/DisplayImage\n"
+      "{\n"
+      "  gsave\n"
+      "  /buffer 512 string def\n"
+      "  % Translation.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /x exch def\n"
+      "  token pop /y exch def pop\n"
+      "  x y translate\n"
+      "  % Image size and font size.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /x exch def\n"
+      "  token pop /y exch def pop\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /pointsize exch def pop\n";
+
+  static const char
+    PostscriptEpilog[]=
+      "  x y scale\n"
+      "  % Clipping path.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /clipped exch def pop\n"
+      "  % Showpage.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /sp exch def pop\n"
+      "  % Image pixel size.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /columns exch def\n"
+      "  token pop /rows exch def pop\n"
+      "  % Colorspace (RGB/CMYK).\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /colorspace exch def pop\n"
+      "  % Transparency.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /alpha exch def pop\n"
+      "  % Stencil mask?\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /stencil exch def pop\n"
+      "  % Image class (direct/pseudo).\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /class exch def pop\n"
+      "  % Compression type.\n"
+      "  currentfile buffer readline pop\n"
+      "  token pop /compression exch def pop\n"
+      "  % Clip and render.\n"
+      "  /pixel_stream currentfile columns rows compression ByteStreamDecodeFilter def\n"
+      "  clipped { ClipImage } if\n"
+      "  alpha stencil not and\n"
+      "  { MaskedImageDict mask_stream resetfile }\n"
+      "  { NonMaskedImageDict }\n"
+      "  ifelse\n"
+      "  stencil { 0 setgray imagemask } { image } ifelse\n"
+      "  grestore\n"
+      "  sp { showpage } if\n"
+      "} bind def\n";
 
   char
     buffer[MaxTextExtent],
@@ -826,7 +824,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     compression;
 
   const char
-    *const *q,
     *option,
     *value;
 
@@ -861,7 +858,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     media_info,
     page_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   SegmentInfo
@@ -962,8 +959,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       }
     if (image->units == PixelsPerCentimeterResolution)
       {
-        resolution.x=(size_t) (100.0*2.54*resolution.x+0.5)/100.0;
-        resolution.y=(size_t) (100.0*2.54*resolution.y+0.5)/100.0;
+        resolution.x=(100.0*2.54*resolution.x+0.5)/100.0;
+        resolution.y=(100.0*2.54*resolution.y+0.5)/100.0;
       }
     SetGeometry(image,&geometry);
     (void) FormatLocaleString(page_geometry,MaxTextExtent,"%.20gx%.20g",
@@ -982,9 +979,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     (void) ConcatenateMagickString(page_geometry,">",MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
-    scale.x=(double) (geometry.width*delta.x)/resolution.x;
+    scale.x=PerceptibleReciprocal(resolution.x)*geometry.width*delta.x;
     geometry.width=(size_t) floor(scale.x+0.5);
-    scale.y=(double) (geometry.height*delta.y)/resolution.y;
+    scale.y=PerceptibleReciprocal(resolution.y)*geometry.height*delta.y;
     geometry.height=(size_t) floor(scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
     (void) ParseGravityGeometry(image,page_geometry,&page_info,
@@ -1019,7 +1016,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) FormatLocaleString(buffer,MaxTextExtent,"%%%%Title: %s\n",
           image->filename);
         (void) WriteBlobString(image,buffer);
-        timer=time((time_t *) NULL);
+        timer=GetMagickTime();
         (void) FormatMagickTime(timer,MaxTextExtent,date);
         (void) FormatLocaleString(buffer,MaxTextExtent,
           "%%%%CreationDate: %s\n",date);
@@ -1083,12 +1080,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           The static postscript procedures prolog.
         */
-        (void)WriteBlobString(image,"%%BeginProlog\n");
-        for (q=PostscriptProlog; *q; q++)
-        {
-          (void) WriteBlobString(image,*q);
-          (void) WriteBlobByte(image,'\n');
-        }
+        (void) WriteBlobString(image,"%%BeginProlog\n");
+        (void) WriteBlob(image,sizeof(PostscriptProlog)-1,
+          (const unsigned char *) PostscriptProlog);
         /*
           One label line for each line in label string.
         */
@@ -1109,12 +1103,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         /*
           The static postscript procedures epilog.
         */
-        for (q=PostscriptEpilog; *q; q++)
-        {
-          (void) WriteBlobString(image,*q);
-          (void) WriteBlobByte(image,'\n');
-        }
-        (void)WriteBlobString(image,"%%EndProlog\n");
+        (void) WriteBlob(image,sizeof(PostscriptEpilog)-1,
+          (const unsigned char *) PostscriptEpilog);
+        (void) WriteBlobString(image,"%%EndProlog\n");
       }
     (void) FormatLocaleString(buffer,MaxTextExtent,"%%%%Page: 1 %.20g\n",
       (double) page);
