@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,8 +95,8 @@ static MagickBooleanType
 */
 static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  const unsigned char
-    *pixels;
+  const void
+    *stream;
 
   Image
     *canvas_image,
@@ -121,6 +121,9 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count,
     y;
 
+  unsigned char
+    *pixels;
+
   /*
     Open image file.
   */
@@ -164,7 +167,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
       canvas_image=DestroyImage(canvas_image);
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  pixels=(const unsigned char *) NULL;
+  pixels=GetQuantumPixels(quantum_info);
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -175,8 +178,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
       length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        pixels=(const unsigned char *) ReadBlobStream(image,length,
-          GetQuantumPixels(quantum_info),&count);
+        stream=ReadBlobStream(image,length,pixels,&count);
         if (count != (ssize_t) length)
           break;
       }
@@ -185,6 +187,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   count=0;
   length=0;
   status=MagickTrue;
+  stream=NULL;
   do
   {
     /*
@@ -199,20 +202,19 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (scene == 0)
       {
         length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-        pixels=(const unsigned char *) ReadBlobStream(image,length,
-          GetQuantumPixels(quantum_info),&count);
+        stream=ReadBlobStream(image,length,pixels,&count);
         if (count != (ssize_t) length)
           break;
       }
     for (y=0; y < (ssize_t) image->extract_info.height; y++)
     {
-      register const PixelPacket
+      const PixelPacket
         *magick_restrict p;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
-      register ssize_t
+      ssize_t
         x;
 
       if (count != (ssize_t) length)
@@ -226,7 +228,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (q == (PixelPacket *) NULL)
         break;
       length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
-        quantum_type,pixels,exception);
+        quantum_type,(unsigned char *) stream,exception);
       if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
       if (((y-image->extract_info.y) >= 0) &&
@@ -256,8 +258,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (status == MagickFalse)
             break;
         }
-      pixels=(const unsigned char *) ReadBlobStream(image,length,
-        GetQuantumPixels(quantum_info),&count);
+      stream=ReadBlobStream(image,length,pixels,&count);
       if (count != (ssize_t) length)
         break;
     }
@@ -331,7 +332,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw red samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("C");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -340,7 +341,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw cyan samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -349,7 +350,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw green samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("M");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -358,7 +359,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw magenta samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("B");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -367,7 +368,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw blue samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("Y");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -376,7 +377,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw yellow samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("A");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -385,7 +386,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw alpha samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("O");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -394,7 +395,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw opacity samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("K");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
@@ -403,7 +404,7 @@ ModuleExport size_t RegisterRAWImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Raw black samples");
-  entry->module=ConstantString("RAW");
+  entry->magick_module=ConstantString("RAW");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -478,7 +479,7 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   size_t
