@@ -17,7 +17,7 @@
 %                               August 2013                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -104,17 +104,17 @@ static Image *ReadRGFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -260,7 +260,7 @@ ModuleExport size_t RegisterRGFImage(void)
   entry->adjoin=MagickFalse;
   entry->description=ConstantString(
     "LEGO Mindstorms EV3 Robot Graphic Format (black and white)");
-  entry->module=ConstantString("RGF");
+  entry->magick_module=ConstantString("RGF");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -324,10 +324,10 @@ static MagickBooleanType WriteRGFImage(const ImageInfo *image_info,Image *image)
   int
     bit;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
