@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -124,7 +124,7 @@ static MagickBooleanType IsSFW(const unsigned char *magick,const size_t length)
 static unsigned char *SFWScan(unsigned char *p,const unsigned char *q,
   const unsigned char *target,const size_t length)
 {
-  register ssize_t
+  ssize_t
     i;
 
   while ((p+length) < q)
@@ -156,7 +156,7 @@ static void TranslateSFWMarker(unsigned char *marker)
 
 static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  static unsigned char
+  static const unsigned char
     HuffmanTable[] =
     {
       0xFF, 0xC4, 0x01, 0xA2, 0x00, 0x00, 0x01, 0x05, 0x01, 0x01, 0x01,
@@ -217,7 +217,7 @@ static Image *ReadSFWImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register unsigned char
+  unsigned char
     *header,
     *data;
 
@@ -411,7 +411,7 @@ ModuleExport size_t RegisterSFWImage(void)
   entry->seekable_stream=MagickTrue;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Seattle Film Works");
-  entry->module=ConstantString("SFW");
+  entry->magick_module=ConstantString("SFW");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
