@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -178,7 +178,7 @@ ModuleExport size_t RegisterTILEImage(void)
   entry->endian_support=MagickTrue;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Tile image with a texture");
-  entry->module=ConstantString("TILE");
+  entry->magick_module=ConstantString("TILE");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
