@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -109,19 +109,19 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -469,7 +469,7 @@ ModuleExport size_t RegisterTIMImage(void)
   entry=SetMagickInfo("TIM");
   entry->decoder=(DecodeImageHandler *) ReadTIMImage;
   entry->description=ConstantString("PSX TIM");
-  entry->module=ConstantString("TIM");
+  entry->magick_module=ConstantString("TIM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
