@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -101,7 +101,7 @@ ModuleExport size_t RegisterUILImage(void)
   entry->encoder=(EncodeImageHandler *) WriteUILImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("X-Motif UIL table");
-  entry->module=ConstantString("UIL");
+  entry->magick_module=ConstantString("UIL");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -181,13 +181,13 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
   MagickSizeType
     number_pixels;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -263,10 +263,10 @@ static MagickBooleanType WriteUILImage(const ImageInfo *image_info,Image *image)
       colors=image->colors;
       if (transparent != MagickFalse)
         {
-          register IndexPacket
+          IndexPacket
             *indexes;
 
-          register PixelPacket
+          PixelPacket
             *q;
 
           i=0;
