@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -148,9 +148,6 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     keyword[MaxTextExtent],
     value[MaxTextExtent];
 
-  const unsigned char
-    *pixels;
-
   Image
     *image;
 
@@ -167,7 +164,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   size_t
@@ -177,6 +174,9 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
     count,
     y;
 
+  unsigned char
+    *pixels;
+
   /*
     Open image file.
   */
@@ -207,16 +207,16 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   length=0;
   image->columns=0;
   image->rows=0;
-  while (isgraph(c) && ((image->columns == 0) || (image->rows == 0)))
+  while (isgraph((int) ((unsigned char) c)) && ((image->columns == 0) || (image->rows == 0)))
   {
-    if (isalnum(c) == MagickFalse)
+    if (isalnum((int) ((unsigned char) c)) == MagickFalse)
       {
         c=ReadBlobByte(image);
         count++;
       }
     else
       {
-        register char
+        char
           *p;
 
         /*
@@ -229,7 +229,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
             *p++=c;
           c=ReadBlobByte(image);
           count++;
-        } while (isalnum(c) || (c == '_'));
+        } while (isalnum((int) ((unsigned char) c)) || (c == '_'));
         *p='\0';
         value_expected=MagickFalse;
         while ((isspace((int) ((unsigned char) c)) != 0) || (c == '='))
@@ -242,7 +242,7 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         if (value_expected == MagickFalse)
           continue;
         p=value;
-        while (isalnum(c))
+        while (isalnum((int) ((unsigned char) c)))
         {
           if ((size_t) (p-value) < (MaxTextExtent-1))
             *p++=c;
@@ -253,12 +253,15 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
         /*
           Assign a value to the specified keyword.
         */
-        if (LocaleCompare(keyword,"Label_RECORDS") == 0)
-          length=(ssize_t) StringToLong(value);
+        if (LocaleCompare(keyword,"LABEL_RECORDS") == 0)
+          length*=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"LBLSIZE") == 0)
           length=(ssize_t) StringToLong(value);
         if (LocaleCompare(keyword,"RECORD_BYTES") == 0)
-          image->columns=StringToUnsignedLong(value);
+          {
+            image->columns=StringToUnsignedLong(value);
+            length=(ssize_t) image->columns;
+          }
         if (LocaleCompare(keyword,"NS") == 0)
           image->columns=StringToUnsignedLong(value);
         if (LocaleCompare(keyword,"LINES") == 0)
@@ -302,18 +305,20 @@ static Image *ReadVICARImage(const ImageInfo *image_info,
   if (quantum_info == (QuantumInfo *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   length=GetQuantumExtent(image,quantum_info,quantum_type);
-  pixels=(const unsigned char *) NULL;
+  pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
+    const void
+      *stream;
+
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    pixels=(const unsigned char *) ReadBlobStream(image,length,
-      GetQuantumPixels(quantum_info),&count);
+    stream=ReadBlobStream(image,length,pixels,&count);
     if (count != (ssize_t) length)
       break;
     (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
-      quantum_type,pixels,exception);
+      quantum_type,(unsigned char *) stream,exception);
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
@@ -364,7 +369,7 @@ ModuleExport size_t RegisterVICARImage(void)
   entry->magick=(IsImageFormatHandler *) IsVICAR;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("VICAR rasterfile format");
-  entry->module=ConstantString("VICAR");
+  entry->magick_module=ConstantString("VICAR");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -439,7 +444,7 @@ static MagickBooleanType WriteVICARImage(const ImageInfo *image_info,
   QuantumInfo
     *quantum_info;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   size_t
