@@ -17,7 +17,7 @@
 %                                 April 2014                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -309,13 +309,13 @@ static MagickBooleanType ReadVIPSPixelsNONE(Image *image,
   Quantum
     pixel;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -488,6 +488,7 @@ static Image *ReadVIPSImage(const ImageInfo *image_info,
     case VIPSTypeYXY:
       ThrowReaderException(CoderError,"Unsupported colorspace");
   }
+  (void) SetImageBackgroundColor(image);
   image->units=PixelsPerCentimeterResolution;
   image->x_resolution=ReadBlobFloat(image)*10;
   image->y_resolution=ReadBlobFloat(image)*10;
@@ -561,7 +562,7 @@ ModuleExport size_t RegisterVIPSImage(void)
   entry->magick=(IsImageFormatHandler *) IsVIPS;
   entry->description=ConstantString("VIPS image");
   entry->endian_support=MagickTrue;
-  entry->module=ConstantString("VIPS");
+  entry->magick_module=ConstantString("VIPS");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -633,13 +634,13 @@ static MagickBooleanType WriteVIPSImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
