@@ -17,7 +17,7 @@
 %                               January 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -127,13 +127,13 @@ static Image *ReadWBMPImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   ssize_t
@@ -265,7 +265,7 @@ ModuleExport size_t RegisterWBMPImage(void)
   entry->encoder=(EncodeImageHandler *) WriteWBMPImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Wireless Bitmap (level 0) image");
-  entry->module=ConstantString("WBMP");
+  entry->magick_module=ConstantString("WBMP");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -331,7 +331,7 @@ static void WBMPWriteInteger(Image *image,const size_t value)
     flag,
     n;
 
-  register ssize_t
+  ssize_t
     i;
 
   unsigned char
@@ -361,10 +361,10 @@ static MagickBooleanType WriteWBMPImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
