@@ -16,7 +16,7 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -276,13 +276,13 @@ static MagickBooleanType InsertRow(unsigned char *p,ssize_t y,Image *image,
   ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   IndexPacket
     index;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
   exception=(&image->exception);
@@ -571,12 +571,14 @@ static int UnpackWPG2Raster(Image *image,int bpp)
   int
     RunCount;
 
+  ssize_t
+    i;
+
   size_t
     x,
     y;
 
   ssize_t
-    i,
     ldblk;
 
   unsigned int
@@ -625,7 +627,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
           RunCount=ReadBlobByte(image);   /* BLK */
           if (RunCount < 0)
             break;
-          for(i=0; i < SampleSize*(RunCount+1); i++)
+          for(i=0; i < ((ssize_t) SampleSize*(RunCount+1)); i++)
             {
               InsertByte6(0);
             }
@@ -654,8 +656,12 @@ static int UnpackWPG2Raster(Image *image,int bpp)
             /* duplicate the previous row RunCount x */
             for(i=0;i<=RunCount;i++)
               {
-                if (InsertRow(BImgBuff,(ssize_t) (image->rows > y ? y : image->rows-1),image,bpp) != MagickFalse)
-                  y++;
+                if (InsertRow(BImgBuff,(ssize_t) (image->rows > y ? y : image->rows-1),image,bpp) == MagickFalse)
+                  {
+                    BImgBuff=(unsigned char *) RelinquishMagickMemory(BImgBuff);
+                    return(-3);
+                  }
+                y++;
               }
           }
           break;
@@ -663,7 +669,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
           RunCount=ReadBlobByte(image);   /* WHT */
           if (RunCount < 0)
             break;
-          for (i=0; i < SampleSize*(RunCount+1); i++)
+          for (i=0; i < ((ssize_t) SampleSize*(RunCount+1)); i++)
           {
             InsertByte6(0xFF);
           }
@@ -680,7 +686,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
                   InsertByte6(SampleBuffer[bbuf]);
             }
           else {      /* NRP */
-            for(i=0; i< SampleSize*(RunCount+1);i++)
+            for(i=0; i < (ssize_t) (SampleSize*(RunCount+1)); i++)
               {
                 bbuf=ReadBlobByte(image);
                 InsertByte6(bbuf);
@@ -899,7 +905,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
     } while (p != (Image *) NULL);
   }
 
-  if ((image->rows == 0 || image->columns == 0) && 
+  if ((image->rows == 0 || image->columns == 0) &&
       (image->previous != NULL || image->next != NULL))
   {
     DeleteImageFromList(&image);
@@ -1111,6 +1117,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   image->columns = 1;
   image->rows = 1;
   image->colors = 0;
+  image->storage_class=DirectClass;
   (void) ResetImagePixels(image,exception);
   bpp=0;
   BitmapHeader2.RotAngle=0;
@@ -1169,7 +1176,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if (WPG_Palette.StartIndex > WPG_Palette.NumOfEntries)
                 ThrowReaderException(CorruptImageError,"InvalidColormapIndex");
               image->colors=WPG_Palette.NumOfEntries;
-              if (!AcquireImageColormap(image,image->colors))
+              if (AcquireImageColormap(image,image->colors) == MagickFalse)
                 goto NoMemory;
               for (i=WPG_Palette.StartIndex;
                    i < (int)WPG_Palette.NumOfEntries; i++)
@@ -1255,10 +1262,28 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               else
                 {
                   if (bpp < 24)
-                    if ( (image->colors < (one << bpp)) && (bpp != 24) )
-                      image->colormap=(PixelPacket *) ResizeQuantumMemory(
-                        image->colormap,(size_t) (one << bpp),
-                        sizeof(*image->colormap));
+                  if ( (image->colors < (one << bpp)) && (bpp != 24) )
+                    {
+                      PixelPacket
+                        *colormap;
+
+                      size_t
+                        colors;
+
+                      colormap=image->colormap;
+                      colors=image->colors;
+                      image->colormap=(PixelPacket *) NULL;
+                      if (AcquireImageColormap(image,one << bpp) == MagickFalse)
+                        {
+                          colormap=(PixelPacket *)
+                            RelinquishMagickMemory(colormap);
+                          goto NoMemory;
+                        }
+                      (void) memcpy(image->colormap,colormap,MagickMin(
+                        image->colors,colors)*sizeof(*image->colormap));
+                      colormap=(PixelPacket *)
+                        RelinquishMagickMemory(colormap);
+                    }
                 }
 
               if ((bpp == 1) && (image->colors > 1))
@@ -1327,6 +1352,10 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                 }
 
               /* Allocate next image structure. */
+              if ((image_info->ping != MagickFalse) &&
+                  (image_info->number_scenes != 0))
+                if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+                  goto Finish;
               AcquireNextImage(image_info,image);
               image->depth=8;
               if (image->next == (Image *) NULL)
@@ -1534,6 +1563,10 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 
 
               /* Allocate next image structure. */
+              if ((image_info->ping != MagickFalse) &&
+                  (image_info->number_scenes != 0))
+                if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+                  goto Finish;
               AcquireNextImage(image_info,image);
               image->depth=8;
               if (image->next == (Image *) NULL)
@@ -1641,7 +1674,7 @@ ModuleExport size_t RegisterWPGImage(void)
   entry->decoder=(DecodeImageHandler *) ReadWPGImage;
   entry->magick=(IsImageFormatHandler *) IsWPG;
   entry->description=AcquireString("Word Perfect Graphics");
-  entry->module=ConstantString("WPG");
+  entry->magick_module=ConstantString("WPG");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
