@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -158,7 +158,7 @@ static int XBMInteger(Image *image,short int *hex_digits)
       {
         value*=16;
         c&=0xff;
-        if (value <= (unsigned int) (INT_MAX-hex_digits[c]))
+        if (value <= (unsigned int) ((INT_MAX-1)-hex_digits[c]))
           value+=hex_digits[c];
       }
     c=ReadBlobByte(image);
@@ -183,17 +183,17 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   long
@@ -452,7 +452,7 @@ ModuleExport size_t RegisterXBMImage(void)
   entry->adjoin=MagickFalse;
   entry->description=ConstantString(
     "X Windows system bitmap (black and white)");
-  entry->module=ConstantString("XBM");
+  entry->magick_module=ConstantString("XBM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -515,10 +515,10 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   size_t
@@ -597,8 +597,8 @@ static MagickBooleanType WriteXBMImage(const ImageInfo *image_info,Image *image)
           bit=0;
           byte=0;
         }
-        p++;
-      }
+      p++;
+    }
     if (bit != 0)
       {
         /*
