@@ -17,7 +17,7 @@
 %                               November 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -304,7 +304,7 @@ static char *ReadBlobStringWithLongSize(Image *image,char *string,size_t max,
   MagickOffsetType
     offset;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -338,10 +338,10 @@ static MagickBooleanType load_tile(Image *image,Image *tile_image,
   ssize_t
     y;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   size_t
@@ -430,7 +430,7 @@ static MagickBooleanType load_tile_rle(Image *image,Image *tile_image,
     alpha,
     data;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   size_t
@@ -618,7 +618,7 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     offset,
     offset2;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -639,8 +639,14 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     and we can simply return.
   */
   offset=(MagickOffsetType) ReadBlobMSBLong(image);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowBinaryException(CorruptImageError,"UnexpectedEndOfFile", 
+      image->filename);
   if (offset == 0)
-    return(MagickTrue);
+    {
+      (void) SetImageBackgroundColor(image);
+      return(MagickTrue);
+    }
   /*
     Initialize the reference for the in-memory tile-compression.
   */
@@ -667,7 +673,7 @@ static MagickBooleanType load_level(Image *image,XCFDocInfo *inDocInfo,
     if (offset2 == 0)
       offset2=(MagickOffsetType) (offset + TILE_WIDTH * TILE_WIDTH * 4* 1.5);
     /* seek to the tile offset */
-    if (SeekBlob(image, offset, SEEK_SET) != offset)
+    if ((offset > offset2) || ( SeekBlob(image, offset, SEEK_SET) != offset))
       ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
         image->filename);
 
@@ -955,7 +961,10 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   outLayer->image->background_color.opacity=
     ScaleCharToQuantum((unsigned char) (255-outLayer->alpha));
   if (outLayer->alpha != 255U)
-    outLayer->image->matte=MagickTrue;
+    {
+      outLayer->image->matte=MagickTrue;
+      (void) SetImageBackgroundColor(outLayer->image);
+    }
 
   InitXCFImage(outLayer);
 
@@ -974,8 +983,8 @@ static MagickBooleanType ReadOneLayer(const ImageInfo *image_info,Image* image,
   /* read in the hierarchy */
   offset=SeekBlob(image, (MagickOffsetType) hierarchy_offset, SEEK_SET);
   if (offset != (MagickOffsetType) hierarchy_offset)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageError,"InvalidImageHeader","`%s'",image->filename);
+    ThrowBinaryImageException(CorruptImageError,"InvalidImageHeader",
+      image->filename);
   if (load_hierarchy (image, inDocInfo, outLayer) == 0)
     return(MagickFalse);
 
@@ -1058,7 +1067,7 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickOffsetType
     offset;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1114,6 +1123,8 @@ static Image *ReadXCFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
   if (image_type == GIMP_INDEXED)
     ThrowReaderException(CoderError,"ColormapTypeNotSupported");
   if (image_type == GIMP_RGB)
@@ -1511,7 +1522,7 @@ ModuleExport size_t RegisterXCFImage(void)
   entry->decoder=(DecodeImageHandler *) ReadXCFImage;
   entry->magick=(IsImageFormatHandler *) IsXCF;
   entry->description=ConstantString("GIMP image");
-  entry->module=ConstantString("XCF");
+  entry->magick_module=ConstantString("XCF");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
