@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -72,6 +72,12 @@
 #include "magick/threshold.h"
 #include "magick/utility.h"
 
+/*
+  Global declarations.
+*/
+static SplayTreeInfo
+  *xpm_symbolic = (SplayTreeInfo *) NULL;
+
 /*
   Forward declarations.
 */
@@ -154,7 +160,7 @@ static int CompareXPMColor(const void *target,const void *source)
 
 static ssize_t CopyXPMColor(char *destination,const char *source,size_t length)
 {
-  register const char
+  const char
     *p;
 
   p=source;
@@ -181,19 +187,21 @@ static char *ParseXPMColor(char *color,MagickBooleanType search_start)
 {
 #define NumberTargets  6
 
-  register char
+  char
     *p,
     *r;
 
-  register const char
+  const char
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   static const char
     *const targets[NumberTargets] = { "c ", "g ", "g4 ", "m ", "b ", "s " };
 
+  if (*color == '\0')
+    return((char *) NULL);
   if (search_start != MagickFalse)
     {
       for (i=0; i < NumberTargets; i++)
@@ -252,20 +260,20 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     active,
     status;
 
-  register char
+  char
     *next,
     *p,
     *q;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register ssize_t
-    x;
-
-  register PixelPacket
+  PixelPacket
     *r;
 
+  ssize_t
+    x;
+
   size_t
     length;
 
@@ -366,6 +374,11 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       *q++=(*p);
   }
   *q='\0';
+  if (active != MagickFalse)
+    {
+      xpm_buffer=DestroyString(xpm_buffer);
+      ThrowReaderException(CorruptImageError,"UnexpectedEndOfFile");
+    }
   /*
     Initialize image structure.
   */
@@ -384,6 +397,9 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   next=NextXPMLine(xpm_buffer);
   for (j=0; (j < (ssize_t) image->colors) && (next != (char *) NULL); j++)
   {
+    char
+      symbolic[MagickPathExtent];
+
     MagickPixelPacket
       pixel;
 
@@ -398,10 +414,13 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Parse color.
     */
+    (void) memset(target,0,sizeof(target));
     (void) CopyMagickString(target,"gray",MaxTextExtent);
     q=(char *) NULL;
     if (strlen(p) > width)
       q=ParseXPMColor(p+width,MagickTrue);
+    (void) memset(symbolic,0,sizeof(symbolic));
+    *symbolic='\0';
     if (q != (char *) NULL)
       {
         while ((isspace((int) ((unsigned char) *q)) == 0) && (*q != '\0'))
@@ -414,10 +433,15 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         else
           (void) CopyMagickString(target,q,MaxTextExtent);
         q=ParseXPMColor(target,MagickFalse);
+        (void) CopyXPMColor(symbolic,q,MagickMin((size_t) (next-q),
+          MagickPathExtent-1));
         if (q != (char *) NULL)
           *q='\0';
       }
     StripString(target);
+    if (*symbolic != '\0')
+      (void) AddValueToSplayTree(xpm_symbolic,ConstantString(target),
+        ConstantString(symbolic));
     grey=strstr(target,"grey");
     if (grey != (char *) NULL)
       grey[2]='a';
@@ -465,7 +489,10 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          ssize_t count=CopyXPMColor(key,p,MagickMin(width,MaxTextExtent-1));
+          ssize_t
+            count;
+
+          count=CopyXPMColor(key,p,MagickMin(width,MaxTextExtent-1));
           if (count != (ssize_t) width)
             break;
           j=(ssize_t) GetValueFromSplayTree(xpm_colors,key);
@@ -524,12 +551,15 @@ ModuleExport size_t RegisterXPMImage(void)
   MagickInfo
     *entry;
 
+  if (xpm_symbolic == (SplayTreeInfo *) NULL)
+    xpm_symbolic=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+      RelinquishMagickMemory);
   entry=SetMagickInfo("PICON");
   entry->decoder=(DecodeImageHandler *) ReadXPMImage;
   entry->encoder=(EncodeImageHandler *) WritePICONImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Personal Icon");
-  entry->module=ConstantString("XPM");
+  entry->magick_module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PM");
   entry->decoder=(DecodeImageHandler *) ReadXPMImage;
@@ -537,7 +567,7 @@ ModuleExport size_t RegisterXPMImage(void)
   entry->adjoin=MagickFalse;
   entry->stealth=MagickTrue;
   entry->description=ConstantString("X Windows system pixmap (color)");
-  entry->module=ConstantString("XPM");
+  entry->magick_module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XPM");
   entry->decoder=(DecodeImageHandler *) ReadXPMImage;
@@ -545,7 +575,7 @@ ModuleExport size_t RegisterXPMImage(void)
   entry->magick=(IsImageFormatHandler *) IsXPM;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("X Windows system pixmap (color)");
-  entry->module=ConstantString("XPM");
+  entry->magick_module=ConstantString("XPM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -574,6 +604,8 @@ ModuleExport void UnregisterXPMImage(void)
   (void) UnregisterMagickInfo("PICON");
   (void) UnregisterMagickInfo("PM");
   (void) UnregisterMagickInfo("XPM");
+  if (xpm_symbolic != (SplayTreeInfo *) NULL)
+    xpm_symbolic=DestroySplayTree(xpm_symbolic);
 }
 
 /*
@@ -672,17 +704,17 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   RectangleInfo
     geometry;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   size_t
@@ -775,7 +807,7 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   colors=picon->colors;
   if (transparent != MagickFalse)
     {
-      register IndexPacket
+      IndexPacket
         *indexes;
 
       colors++;
@@ -825,6 +857,9 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
   GetMagickPixelPacket(image,&pixel);
   for (i=0; i < (ssize_t) colors; i++)
   {
+    const char
+      *symbolic;
+
     /*
       Define XPM color.
     */
@@ -850,8 +885,13 @@ static MagickBooleanType WritePICONImage(const ImageInfo *image_info,
       symbol[j]=Cixel[k];
     }
     symbol[j]='\0';
-    (void) FormatLocaleString(buffer,MaxTextExtent,"\"%.1024s c %.1024s\",\n",
-      symbol,name);
+    symbolic=(const char *) GetValueFromSplayTree(xpm_symbolic,name);
+    if (symbolic == (const char *) NULL)
+      (void) FormatLocaleString(buffer,MaxTextExtent,"\"%.1024s c %.1024s\",\n",
+        symbol,name);
+    else
+      (void) FormatLocaleString(buffer,MaxTextExtent,
+        "\"%.1024s c %.1024s %.1024s\",\n",symbol,name,symbolic);
     (void) WriteBlobString(image,buffer);
   }
   /*
@@ -939,13 +979,13 @@ static MagickBooleanType WriteXPMImage(const ImageInfo *image_info,Image *image)
   MagickPixelPacket
     pixel;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
