@@ -17,7 +17,7 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -89,6 +89,7 @@
 #include "magick/segment.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/thread-private.h"
 #include "magick/threshold.h"
 #include "magick/transform.h"
@@ -122,12 +123,280 @@
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+typedef struct _EdgeInfo
+{
+  double
+    left,
+    right,
+    top,
+    bottom;
+} EdgeInfo;
+
+static double GetEdgeBackgroundFactor(const Image *image,
+  const CacheView *image_view,const GravityType gravity,const size_t width,
+  const size_t height,const ssize_t x_offset,const ssize_t y_offset,
+  ExceptionInfo *exception)
+{
+  CacheView
+    *edge_view;
+
+  const char
+    *artifact;
+
+  double
+    factor;
+
+  Image
+    *edge_image;
+
+  MagickPixelPacket
+    background,
+    pixel;
+
+  RectangleInfo
+    edge_geometry;
+
+  const PixelPacket
+    *p;
+
+  ssize_t
+    y;
+
+  /*
+    Determine the percent of image background for this edge.
+  */
+  switch (gravity)
+  {
+    case NorthWestGravity:
+    case NorthGravity:
+    default:
+    {
+      p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
+      break;
+    }
+    case NorthEastGravity:
+    case EastGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,0,1,1,
+        exception);
+      break;
+    }
+    case SouthEastGravity:
+    case SouthGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,
+        (ssize_t) image->rows-1,1,1,exception);
+      break;
+    }
+    case SouthWestGravity:
+    case WestGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,0,(ssize_t) image->rows-1,1,1,
+        exception);
+      break;
+    }
+  }
+  GetMagickPixelPacket(image,&background);
+  SetMagickPixelPacket(image,p,(IndexPacket *) NULL,&background);
+  artifact=GetImageArtifact(image,"background");
+  if (artifact != (const char *) NULL)
+    (void) QueryMagickColor(artifact,&background,exception);
+  artifact=GetImageArtifact(image,"trim:background-color");
+  if (artifact != (const char *) NULL)
+    (void) QueryMagickColor(artifact,&background,exception);
+  edge_geometry.width=width;
+  edge_geometry.height=height;
+  edge_geometry.x=x_offset;
+  edge_geometry.y=y_offset;
+  GravityAdjustGeometry(image->columns,image->rows,gravity,&edge_geometry);
+  edge_image=CropImage(image,&edge_geometry,exception);
+  if (edge_image == (Image *) NULL)
+    return(0.0);
+  factor=0.0;
+  GetMagickPixelPacket(edge_image,&pixel);
+  edge_view=AcquireVirtualCacheView(edge_image,exception);
+  for (y=0; y < (ssize_t) edge_image->rows; y++)
+  {
+    ssize_t
+      x;
+
+    p=GetCacheViewVirtualPixels(edge_view,0,y,edge_image->columns,1,exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    for (x=0; x < (ssize_t) edge_image->columns; x++)
+    {
+      SetMagickPixelPacket(edge_image,p,(IndexPacket *) NULL,&pixel);
+      if (IsMagickColorSimilar(&pixel,&background) == MagickFalse)
+        factor++;
+      p++;
+    }
+  }
+  factor/=((double) edge_image->columns*edge_image->rows);
+  edge_view=DestroyCacheView(edge_view);
+  edge_image=DestroyImage(edge_image);
+  return(factor);
+}
+
+static inline double GetMinEdgeBackgroundFactor(const EdgeInfo *edge)
+{
+  double
+    factor;
+
+  factor=MagickMin(MagickMin(MagickMin(edge->left,edge->right),edge->top),
+    edge->bottom);
+  return(factor);
+}
+
+static RectangleInfo GetEdgeBoundingBox(const Image *image,
+  ExceptionInfo *exception)
+{
+  CacheView
+    *edge_view;
+
+  const char
+    *artifact;
+
+  double
+    background_factor,
+    percent_background;
+
+  EdgeInfo
+    edge,
+    vertex;
+
+  Image
+    *edge_image;
+
+  RectangleInfo
+    bounds;
+
+  /*
+    Get the image bounding box.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickCoreSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  SetGeometry(image,&bounds);
+  edge_image=CloneImage(image,0,0,MagickTrue,exception);
+  if (edge_image == (Image *) NULL)
+    return(bounds);
+  (void) ParseAbsoluteGeometry("0x0+0+0",&edge_image->page);
+  memset(&vertex,0,sizeof(vertex));
+  edge_view=AcquireVirtualCacheView(edge_image,exception);
+  edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,WestGravity,
+    1,0,0,0,exception);
+  edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,EastGravity,
+    1,0,0,0,exception);
+  edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,NorthGravity,
+    0,1,0,0,exception);
+  edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,SouthGravity,
+    0,1,0,0,exception);
+  percent_background=1.0;
+  artifact=GetImageArtifact(edge_image,"trim:percent-background");
+  if (artifact != (const char *) NULL)
+    percent_background=StringToDouble(artifact,(char **) NULL)/100.0;
+  percent_background=MagickMin(MagickMax(1.0-percent_background,MagickEpsilon),
+    1.0);
+  background_factor=GetMinEdgeBackgroundFactor(&edge);
+  for ( ; background_factor < percent_background;
+          background_factor=GetMinEdgeBackgroundFactor(&edge))
+  {
+    if ((bounds.width == 0) || (bounds.height == 0))
+      break;
+    if (fabs(edge.left-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim left edge.
+        */
+        vertex.left++;
+        bounds.width--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+    if (fabs(edge.right-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim right edge.
+        */
+        vertex.right++;
+        bounds.width--;
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+    if (fabs(edge.top-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim top edge.
+        */
+        vertex.top++;
+        bounds.height--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        continue;
+      }
+    if (fabs(edge.bottom-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim bottom edge.
+        */
+        vertex.bottom++;
+        bounds.height--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+  }
+  edge_view=DestroyCacheView(edge_view);
+  edge_image=DestroyImage(edge_image);
+  bounds.x=(ssize_t) vertex.left;
+  bounds.y=(ssize_t) vertex.top;
+  if ((bounds.width == 0) || (bounds.height == 0))
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
+      "GeometryDoesNotContainImage","`%s'",image->filename);
+  return(bounds);
+}
+
 MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   ExceptionInfo *exception)
 {
   CacheView
     *image_view;
 
+  const char
+    *artifact;
+
   MagickBooleanType
     status;
 
@@ -138,7 +407,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   RectangleInfo
     bounds;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   ssize_t
@@ -148,6 +417,9 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  artifact=GetImageArtifact(image,"trim:percent-background");
+  if (artifact != (const char *) NULL)
+    return(GetEdgeBoundingBox(image,exception));
   bounds.width=0;
   bounds.height=0;
   bounds.x=(ssize_t) image->columns;
@@ -188,13 +460,13 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     RectangleInfo
       bounding_box;
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -243,7 +515,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     }
   }
   image_view=DestroyCacheView(image_view);
-  if ((bounds.width == 0) && (bounds.height == 0))
+  if ((bounds.width == 0) || (bounds.height == 0))
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
       "GeometryDoesNotContainImage","`%s'",image->filename);
   else
@@ -296,7 +568,7 @@ MagickExport size_t GetImageChannelDepth(const Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -409,13 +681,13 @@ RestoreMSCWarning
         const int
           id = GetOpenMPThreadId();
 
-        register const IndexPacket
+        const IndexPacket
           *magick_restrict indexes;
 
-        register const PixelPacket
+        const PixelPacket
           *magick_restrict p;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -485,13 +757,13 @@ RestoreMSCWarning
     const int
       id = GetOpenMPThreadId();
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -693,10 +965,10 @@ MagickExport ImageType IdentifyImageGray(const Image *image,
   ImageType
     type;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -774,10 +1046,10 @@ MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
   ImageType
     type;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   ssize_t
@@ -911,6 +1183,7 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
+  magick_unreferenced(exception);
   if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
       (image->type == GrayscaleMatteType))
     return(MagickTrue);
@@ -947,6 +1220,7 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
+  magick_unreferenced(exception);
   if (image->type == BilevelType)
     return(MagickTrue);
   return(MagickFalse);
@@ -984,10 +1258,10 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
   CacheView
     *image_view;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -1086,7 +1360,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   range=GetQuantumRange(depth);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -1121,7 +1395,7 @@ RestoreMSCWarning
       Quantum
         *depth_map;
 
-      register ssize_t
+      ssize_t
         i;
 
       /*
@@ -1139,12 +1413,12 @@ RestoreMSCWarning
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register ssize_t
-          x;
-
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
+        ssize_t
+          x;
+
         if (status == MagickFalse)
           continue;
         q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
@@ -1189,12 +1463,12 @@ RestoreMSCWarning
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
-      x;
-
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
+    ssize_t
+      x;
+
     if (status == MagickFalse)
       continue;
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
