@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
   
     https://imagemagick.org/script/license.php
@@ -124,6 +124,9 @@ typedef struct _NexusInfo
 
   size_t
     signature;
+
+  struct _NexusInfo
+    *virtual_nexus;
 } NexusInfo;
 
 typedef struct _OpenCLCacheInfo
@@ -240,6 +243,10 @@ typedef struct _CacheInfo
 
   OpenCLCacheInfo
     *opencl;
+
+  MagickSizeType
+    width_limit,
+    height_limit;
 } CacheInfo;
 
 extern MagickExport Cache
