@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -163,13 +163,13 @@ MagickExport Image *CombineImages(const Image *image,const ChannelType channel,
     PixelPacket
       *pixels;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -272,7 +272,11 @@ MagickExport Image *CombineImages(const Image *image,const ChannelType channel,
         MagickBooleanType
           proceed;
 
-        proceed=SetImageProgress(image,CombineImageTag,progress++,
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp atomic
+#endif
+        progress++;
+        proceed=SetImageProgress(image,CombineImageTag,progress,
           combine_image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -417,13 +421,13 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -523,9 +527,10 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_SeparateImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SeparateImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SeparateImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -687,10 +692,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -758,13 +763,13 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register IndexPacket
+        IndexPacket
           *magick_restrict indexes;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -840,10 +845,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -915,13 +920,13 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        register IndexPacket
+        IndexPacket
           *magick_restrict indexes;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
