@@ -16,7 +16,7 @@
 %                               March  2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -281,7 +281,7 @@ static AESInfo *DestroyAESInfo(AESInfo *aes_info)
 static inline void AddRoundKey(const unsigned int *ciphertext,
   const unsigned int *key,unsigned int *plaintext)
 {
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -291,7 +291,7 @@ static inline void AddRoundKey(const unsigned int *ciphertext,
     plaintext[i]=key[i] ^ ciphertext[i];
 }
 
-static inline unsigned char ByteMultiply(const unsigned char alpha,
+static inline unsigned int ByteMultiply(const unsigned char alpha,
   const unsigned char beta)
 {
   /*
@@ -299,7 +299,7 @@ static inline unsigned char ByteMultiply(const unsigned char alpha,
   */
   if ((alpha == 0) || (beta == 0))
     return(0);
-  return(InverseLog[(Log[alpha]+Log[beta]) % 0xff]);
+  return((unsigned int) InverseLog[(Log[alpha]+Log[beta]) % 0xff]);
 }
 
 static inline unsigned int ByteSubTransform(unsigned int x,
@@ -311,18 +311,20 @@ static inline unsigned int ByteSubTransform(unsigned int x,
   /*
     Non-linear layer resists differential and linear cryptoanalysis attacks.
   */
-  key=(s_box[x & 0xff]) | (s_box[(x >> 8) & 0xff] << 8) |
-    (s_box[(x >> 16) & 0xff] << 16) | (s_box[(x >> 24) & 0xff] << 24);
+  key=((unsigned int) s_box[x & 0xff]) |
+    ((unsigned int) s_box[(x >> 8) & 0xff] << 8) |
+    ((unsigned int) s_box[(x >> 16) & 0xff] << 16) |
+    ((unsigned int) s_box[(x >> 24) & 0xff] << 24);
   return(key);
 }
 
 static void FinalizeRoundKey(const unsigned int *ciphertext,
   const unsigned int *key,unsigned char *plaintext)
 {
-  register unsigned char
+  unsigned char
     *p;
 
-  register unsigned int
+  unsigned int
     i,
     j;
 
@@ -348,10 +350,10 @@ static void FinalizeRoundKey(const unsigned int *ciphertext,
 static void InitializeRoundKey(const unsigned char *ciphertext,
   const unsigned int *key,unsigned int *plaintext)
 {
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register unsigned int
+  unsigned int
     i,
     j;
 
@@ -363,7 +365,7 @@ static void InitializeRoundKey(const unsigned char *ciphertext,
   {
     value=0;
     for (j=0; j < 4; j++)
-      value|=(*p++ << (8*j));
+      value|=((unsigned int) *p++ << (8*j));
     plaintext[i]=key[i] ^ value;
   }
   /*
@@ -380,7 +382,7 @@ static inline unsigned int RotateLeft(const unsigned int x)
 static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
   unsigned char *ciphertext)
 {
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -522,7 +524,7 @@ static void EncipherAESBlock(AESInfo *aes_info,const unsigned char *plaintext,
 static inline void IncrementCipherNonce(const size_t length,
   unsigned char *nonce)
 {
-  register ssize_t
+  ssize_t
     i;
 
   for (i=(ssize_t) (length-1); i >= 0; i--)
@@ -582,7 +584,7 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   SignatureInfo
     *signature_info;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -657,11 +659,11 @@ MagickExport MagickBooleanType PasskeyDecipherImage(Image *image,
   image_view=AcquireAuthenticCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       i,
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
@@ -791,7 +793,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   QuantumType
     quantum_type;
 
-  register unsigned char
+  unsigned char
     *p;
 
   SignatureInfo
@@ -876,11 +878,11 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
   image_view=AcquireAuthenticCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register ssize_t
+    ssize_t
       i,
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
@@ -958,7 +960,7 @@ MagickExport MagickBooleanType PasskeyEncipherImage(Image *image,
 static inline void InverseAddRoundKey(const unsigned int *alpha,
   unsigned int *beta)
 {
-  register unsigned int
+  unsigned int
     i,
     j;
 
@@ -991,7 +993,7 @@ static inline unsigned int RotateRight(const unsigned int x)
 
 static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
 {
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1033,8 +1035,10 @@ static void SetAESKey(AESInfo *aes_info,const StringInfo *key)
   (void) memcpy(datum,GetStringInfoDatum(key),MagickMin(
     GetStringInfoLength(key),GetStringInfoLength(aes_info->key)));
   for (i=0; i < n; i++)
-    aes_info->encipher_key[i]=datum[4*i] | (datum[4*i+1] << 8) |
-      (datum[4*i+2] << 16) | (datum[4*i+3] << 24);
+    aes_info->encipher_key[i]=(unsigned int) datum[4*i] |
+      ((unsigned int) datum[4*i+1] << 8) |
+      ((unsigned int) datum[4*i+2] << 16) |
+      ((unsigned int) datum[4*i+3] << 24);
   beta=1;
   bytes=(AESBlocksize/4)*(aes_info->rounds+1);
   for (i=n; i < bytes; i++)
