@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -81,18 +81,20 @@ static const CoderMapInfo
   CoderMap[] =
   {
     { "3FR", "DNG" },
-    { "3GP", "MPEG" },
-    { "3G2", "MPEG" },
+    { "3GP", "VIDEO" },
+    { "3G2", "VIDEO" },
     { "8BIM", "META" },
     { "8BIMTEXT", "META" },
     { "8BIMWTEXT", "META" },
     { "AFM", "TTF" },
     { "A", "RAW" },
     { "AI", "PDF" },
+    { "APNG", "VIDEO" },
     { "APP1JPEG", "META" },
     { "APP1", "META" },
     { "ARW", "DNG" },
-    { "AVI", "MPEG" },
+    { "AVI", "VIDEO" },
+    { "AVIF", "HEIC" },
     { "BIE", "JBIG" },
     { "BMP2", "BMP" },
     { "BMP3", "BMP" },
@@ -106,6 +108,7 @@ static const CoderMapInfo
     { "CANVAS", "XC" },
     { "CMYKA", "CMYK" },
     { "CR2", "DNG" },
+    { "CR3", "DNG" },
     { "CRW", "DNG" },
     { "CUR", "ICON" },
     { "DATA", "INLINE" },
@@ -177,14 +180,15 @@ static const CoderMapInfo
     { "KDC", "DNG" },
     { "LOGO", "MAGICK" },
     { "M", "RAW" },
-    { "M2V", "MPEG" },
-    { "M4V", "MPEG" },
+    { "M2V", "VIDEO" },
+    { "M4V", "VIDEO" },
     { "MEF", "DNG" },
-    { "MKV", "MPEG" },
+    { "MKV", "VIDEO" },
     { "MNG", "PNG" },
-    { "MOV", "MPEG" },
-    { "MP4", "MPEG" },
-    { "MPG", "MPEG" },
+    { "MOV", "VIDEO" },
+    { "MP4", "VIDEO" },
+    { "MPEG", "VIDEO" },
+    { "MPG", "VIDEO" },
     { "MPRI", "MPR" },
     { "MEF", "DNG" },
     { "MRW", "DNG" },
@@ -244,11 +248,12 @@ static const CoderMapInfo
     { "UBRL6", "BRAILLE" },
     { "VDA", "TGA" },
     { "VST", "TGA" },
+    { "WEBM", "VIDEO" },
     { "WIZARD", "MAGICK" },
 #if defined(MAGICKCORE_WINGDI32_DELEGATE)
     { "WMF", "EMF" },
 #endif
-    { "WMV", "MPEG" },
+    { "WMV", "VIDEO" },
     { "WMZ", "WMF" },
     { "X3f", "DNG" },
     { "XMP", "META" },
@@ -268,9 +273,13 @@ static SplayTreeInfo
   Forward declarations.
 */
 static MagickBooleanType
-  IsCoderTreeInstantiated(ExceptionInfo *),
+  IsCoderTreeInstantiated(ExceptionInfo *);
+
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
+static MagickBooleanType
   LoadCoderCache(SplayTreeInfo *,const char *,const char *,const size_t,
     ExceptionInfo *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -301,7 +310,7 @@ static MagickBooleanType
 
 static void *DestroyCoderNode(void *coder_info)
 {
-  register CoderInfo
+  CoderInfo
     *p;
 
   p=(CoderInfo *) coder_info;
@@ -323,7 +332,7 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
   MagickStatusType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   SplayTreeInfo
@@ -337,7 +346,7 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
   if (cache == (SplayTreeInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
       *option;
@@ -364,7 +373,7 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
     CoderInfo
       *coder_info;
 
-    register const CoderMapInfo
+    const CoderMapInfo
       *p;
 
     p=CoderMap+i;
@@ -526,10 +535,10 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
   const CoderInfo
     **coder_map;
 
-  register const CoderInfo
+  const CoderInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -596,7 +605,7 @@ MagickExport const CoderInfo **GetCoderInfoList(const char *pattern,
 
 static int CoderCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -611,10 +620,10 @@ MagickExport char **GetCoderList(const char *pattern,
   char
     **coder_map;
 
-  register const CoderInfo
+  const CoderInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -721,7 +730,7 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
   const CoderInfo
     **coder_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -763,6 +772,7 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
   return(MagickTrue);
 }
 
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -828,7 +838,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -838,7 +848,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -847,7 +857,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -858,10 +868,10 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > MagickMaxRecursionDepth)
@@ -921,11 +931,11 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
         coder_info=(CoderInfo *) NULL;
         continue;
       }
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetNextToken(q,&q,extent,token);
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'M':
@@ -965,3 +975,4 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
   token=(char *) RelinquishMagickMemory(token);
   return(status != 0 ? MagickTrue : MagickFalse);
 }
+#endif
