@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,7 @@
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colormap.h"
+#include "magick/colormap-private.h"
 #include "magick/client.h"
 #include "magick/configure.h"
 #include "magick/exception.h"
@@ -102,7 +103,7 @@
 MagickExport MagickBooleanType AcquireImageColormap(Image *image,
   const size_t colors)
 {
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -112,13 +113,20 @@ MagickExport MagickBooleanType AcquireImageColormap(Image *image,
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (colors > MaxColormapSize)
+    {
+      image->colors=0;
+      image->storage_class=DirectClass;
+      ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
+        image->filename);
+    }
   image->colors=MagickMax(colors,1);
   if (image->colormap == (PixelPacket *) NULL)
-    image->colormap=(PixelPacket *) AcquireQuantumMemory(image->colors+1,
+    image->colormap=(PixelPacket *) AcquireQuantumMemory(image->colors+256,
       sizeof(*image->colormap));
   else
     image->colormap=(PixelPacket *) ResizeQuantumMemory(image->colormap,
-      image->colors+1,sizeof(*image->colormap));
+      image->colors+256,sizeof(*image->colormap));
   if (image->colormap == (PixelPacket *) NULL)
     {
       image->colors=0;
@@ -196,13 +204,13 @@ MagickExport MagickBooleanType CycleColormapImage(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     ssize_t
@@ -292,7 +300,7 @@ MagickExport MagickBooleanType SortColormapByIntensity(Image *image)
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -338,13 +346,13 @@ MagickExport MagickBooleanType SortColormapByIntensity(Image *image)
     IndexPacket
       index;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
@@ -356,7 +364,8 @@ MagickExport MagickBooleanType SortColormapByIntensity(Image *image)
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      index=(IndexPacket) pixels[(ssize_t) GetPixelIndex(indexes+x)];
+      i=ConstrainColormapIndex(image,GetPixelIndex(indexes+x));
+      index=(IndexPacket) pixels[i];
       SetPixelIndex(indexes+x,index);
       SetPixelRGBO(q,image->colormap+(ssize_t) index);
       q++;
