@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -285,7 +285,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
   PrimaryInfo
     primary_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -339,13 +339,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickPixelPacket
           pixel;
 
-        register IndexPacket
+        IndexPacket
           *magick_restrict indexes;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -381,6 +381,62 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       return(status);
     }
     case LinearGRAYColorspace:
+    {
+      /*
+        Transform image from sRGB to GRAY.
+      */
+      if (image->storage_class == PseudoClass)
+        {
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
+        }
+      image_view=AcquireAuthenticCacheView(image,exception);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+      #pragma omp parallel for schedule(static) shared(status) \
+        magick_number_threads(image,image,image->rows,1)
+#endif
+      for (y=0; y < (ssize_t) image->rows; y++)
+      {
+        MagickBooleanType
+          sync;
+
+        ssize_t
+          x;
+
+        PixelPacket
+          *magick_restrict q;
+
+        if (status == MagickFalse)
+          continue;
+        q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+          exception);
+        if (q == (PixelPacket *) NULL)
+          {
+            status=MagickFalse;
+            continue;
+          }
+        for (x=0; x < (ssize_t) image->columns; x++)
+        {
+          MagickRealType
+            gray;
+
+          gray=0.212656*GetPixelRed(q)+0.715158*GetPixelGreen(q)+
+            0.072186*GetPixelBlue(q);
+          SetPixelGray(q,ClampToQuantum(DecodePixelGamma(gray)));
+          q++;
+        }
+        sync=SyncCacheViewAuthenticPixels(image_view,exception);
+        if (sync == MagickFalse)
+          status=MagickFalse;
+      }
+      image_view=DestroyCacheView(image_view);
+      if (SetImageColorspace(image,colorspace) == MagickFalse)
+        return(MagickFalse);
+      image->type=GrayscaleType;
+      return(status);
+    }
     case GRAYColorspace:
     {
       /*
@@ -403,10 +459,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -420,7 +476,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           }
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          SetPixelGray(q,ClampToQuantum(GetPixelIntensity(image,q)));
+          MagickRealType
+            gray;
+
+          gray=0.212656*GetPixelRed(q)+0.715158*GetPixelGreen(q)+
+            0.072186*GetPixelBlue(q);
+          SetPixelGray(q,ClampToQuantum(gray));
           q++;
         }
         sync=SyncCacheViewAuthenticPixels(image_view,exception);
@@ -475,10 +536,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -695,10 +756,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -761,10 +822,10 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -851,13 +912,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=(MagickRealType) (0.33333*(double) i);
-        y_map[i].x=(MagickRealType) (0.33334*(double) i);
-        z_map[i].x=(MagickRealType) (0.33333*(double) i);
         x_map[i].y=(MagickRealType) (0.50000*(double) i);
-        y_map[i].y=(MagickRealType) (0.00000*(double) i);
-        z_map[i].y=(MagickRealType) (-0.50000*(double) i);
         x_map[i].z=(MagickRealType) (-0.25000*(double) i);
+        y_map[i].x=(MagickRealType) (0.33334*(double) i);
+        y_map[i].y=(MagickRealType) (0.00000*(double) i);
         y_map[i].z=(MagickRealType) (0.50000*(double) i);
+        z_map[i].x=(MagickRealType) (0.33333*(double) i);
+        z_map[i].y=(MagickRealType) (-0.50000*(double) i);
         z_map[i].z=(MagickRealType) (-0.25000*(double) i);
       }
       break;
@@ -875,13 +936,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=(MagickRealType) (0.298839*(double) i);
-        y_map[i].x=(MagickRealType) (0.586811*(double) i);
-        z_map[i].x=(MagickRealType) (0.114350*(double) i);
         x_map[i].y=(MagickRealType) (0.298839*(double) i);
-        y_map[i].y=(MagickRealType) (0.586811*(double) i);
-        z_map[i].y=(MagickRealType) (0.114350*(double) i);
         x_map[i].z=(MagickRealType) (0.298839*(double) i);
+        y_map[i].x=(MagickRealType) (0.586811*(double) i);
+        y_map[i].y=(MagickRealType) (0.586811*(double) i);
         y_map[i].z=(MagickRealType) (0.586811*(double) i);
+        z_map[i].x=(MagickRealType) (0.114350*(double) i);
+        z_map[i].y=(MagickRealType) (0.114350*(double) i);
         z_map[i].z=(MagickRealType) (0.114350*(double) i);
       }
       break;
@@ -906,13 +967,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=(MagickRealType) (0.298839*(double) i);
-        y_map[i].x=(MagickRealType) (0.586811*(double) i);
-        z_map[i].x=(MagickRealType) (0.114350*(double) i);
         x_map[i].y=(MagickRealType) (-0.1687367*(double) i);
-        y_map[i].y=(MagickRealType) (-0.331264*(double) i);
-        z_map[i].y=(MagickRealType) (0.500000*(double) i);
         x_map[i].z=(MagickRealType) (0.500000*(double) i);
+        y_map[i].x=(MagickRealType) (0.586811*(double) i);
+        y_map[i].y=(MagickRealType) (-0.331264*(double) i);
         y_map[i].z=(MagickRealType) (-0.418688*(double) i);
+        z_map[i].x=(MagickRealType) (0.114350*(double) i);
+        z_map[i].y=(MagickRealType) (0.500000*(double) i);
         z_map[i].z=(MagickRealType) (-0.081312*(double) i);
       }
       break;
@@ -930,13 +991,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=(MagickRealType) (0.212656*(double) i);
-        y_map[i].x=(MagickRealType) (0.715158*(double) i);
-        z_map[i].x=(MagickRealType) (0.072186*(double) i);
         x_map[i].y=(MagickRealType) (0.212656*(double) i);
-        y_map[i].y=(MagickRealType) (0.715158*(double) i);
-        z_map[i].y=(MagickRealType) (0.072186*(double) i);
         x_map[i].z=(MagickRealType) (0.212656*(double) i);
+        y_map[i].x=(MagickRealType) (0.715158*(double) i);
+        y_map[i].y=(MagickRealType) (0.715158*(double) i);
         y_map[i].z=(MagickRealType) (0.715158*(double) i);
+        z_map[i].x=(MagickRealType) (0.072186*(double) i);
+        z_map[i].y=(MagickRealType) (0.072186*(double) i);
         z_map[i].z=(MagickRealType) (0.072186*(double) i);
       }
       break;
@@ -961,13 +1022,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=(MagickRealType) (0.212656*(double) i);
-        y_map[i].x=(MagickRealType) (0.715158*(double) i);
-        z_map[i].x=(MagickRealType) (0.072186*(double) i);
         x_map[i].y=(MagickRealType) (-0.114572*(double) i);
-        y_map[i].y=(MagickRealType) (-0.385428*(double) i);
-        z_map[i].y=(MagickRealType) (0.500000*(double) i);
         x_map[i].z=(MagickRealType) (0.500000*(double) i);
+        y_map[i].x=(MagickRealType) (0.715158*(double) i);
+        y_map[i].y=(MagickRealType) (-0.385428*(double) i);
         y_map[i].z=(MagickRealType) (-0.454153*(double) i);
+        z_map[i].x=(MagickRealType) (0.072186*(double) i);
+        z_map[i].y=(MagickRealType) (0.500000*(double) i);
         z_map[i].z=(MagickRealType) (-0.045847*(double) i);
       }
       break;
@@ -988,25 +1049,25 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       for (i=0; i <= (ssize_t) (0.018*MaxMap); i++)
       {
         x_map[i].x=0.005382*i;
-        y_map[i].x=0.010566*i;
-        z_map[i].x=0.002052*i;
         x_map[i].y=(-0.003296)*i;
-        y_map[i].y=(-0.006471)*i;
-        z_map[i].y=0.009768*i;
         x_map[i].z=0.009410*i;
+        y_map[i].x=0.010566*i;
+        y_map[i].y=(-0.006471)*i;
         y_map[i].z=(-0.007880)*i;
+        z_map[i].x=0.002052*i;
+        z_map[i].y=0.009768*i;
         z_map[i].z=(-0.001530)*i;
       }
       for ( ; i <= (ssize_t) MaxMap; i++)
       {
         x_map[i].x=0.298839*(1.099*i-0.099);
-        y_map[i].x=0.586811*(1.099*i-0.099);
-        z_map[i].x=0.114350*(1.099*i-0.099);
         x_map[i].y=(-0.298839)*(1.099*i-0.099);
-        y_map[i].y=(-0.586811)*(1.099*i-0.099);
-        z_map[i].y=0.88600*(1.099*i-0.099);
         x_map[i].z=0.70100*(1.099*i-0.099);
+        y_map[i].x=0.586811*(1.099*i-0.099);
+        y_map[i].y=(-0.586811)*(1.099*i-0.099);
         y_map[i].z=(-0.586811)*(1.099*i-0.099);
+        z_map[i].x=0.114350*(1.099*i-0.099);
+        z_map[i].y=0.88600*(1.099*i-0.099);
         z_map[i].z=(-0.114350)*(1.099*i-0.099);
       }
       break;
@@ -1058,13 +1119,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         MagickPixelPacket
           pixel;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register size_t
+        size_t
           blue,
           green,
           red;
@@ -1120,7 +1181,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     }
     case PseudoClass:
     {
-      register size_t
+      size_t
         blue,
         green,
         red;
@@ -1276,10 +1337,10 @@ MagickExport MagickBooleanType SetImageGray(Image *image,
   ImageType
     type;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -1370,10 +1431,10 @@ MagickExport MagickBooleanType SetImageMonochrome(Image *image,
   ImageType
     type;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
   ssize_t
@@ -1889,7 +1950,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
   MagickOffsetType
     progress;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1938,13 +1999,13 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickPixelPacket
           pixel;
 
-        register IndexPacket
+        IndexPacket
           *magick_restrict indexes;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -1974,8 +2035,69 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         return(MagickFalse);
       return(status);
     }
-    case GRAYColorspace:
+    case LinearGRAYColorspace:
     case Rec601LumaColorspace:
+    {
+      /*
+        Transform linear RGB to sRGB colorspace.
+      */
+      if (image->storage_class == PseudoClass)
+        {
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
+        }
+      if (SetImageColorspace(image,sRGBColorspace) == MagickFalse)
+        return(MagickFalse);
+      image_view=AcquireAuthenticCacheView(image,exception);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+      #pragma omp parallel for schedule(static) shared(status) \
+        magick_number_threads(image,image,image->rows,1)
+#endif
+      for (y=0; y < (ssize_t) image->rows; y++)
+      {
+        MagickBooleanType
+          sync;
+
+        ssize_t
+          x;
+
+        PixelPacket
+          *magick_restrict q;
+
+        if (status == MagickFalse)
+          continue;
+        q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+          exception);
+        if (q == (PixelPacket *) NULL)
+          {
+            status=MagickFalse;
+            continue;
+          }
+        for (x=(ssize_t) image->columns; x != 0; x--)
+        {
+          MagickRealType
+            gray;
+
+          gray=0.212656*GetPixelRed(q)+0.715158*GetPixelGreen(q)+
+            0.072186*GetPixelBlue(q);
+          gray=EncodePixelGamma(gray);
+          SetPixelRed(q,ClampToQuantum(gray));
+          SetPixelGreen(q,ClampToQuantum(gray));
+          SetPixelBlue(q,ClampToQuantum(gray));
+          q++;
+        }
+        sync=SyncCacheViewAuthenticPixels(image_view,exception);
+        if (sync == MagickFalse)
+          status=MagickFalse;
+      }
+      image_view=DestroyCacheView(image_view);
+      if (SetImageColorspace(image,sRGBColorspace) == MagickFalse)
+        return(MagickFalse);
+      return(status);
+    }
+    case GRAYColorspace:
     case Rec709LumaColorspace:
     {
       /*
@@ -2000,10 +2122,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2020,10 +2142,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           MagickRealType
             gray;
 
-          gray=(MagickRealType) GetPixelGray(q);
-          if ((image->intensity == Rec601LuminancePixelIntensityMethod) ||
-              (image->intensity == Rec709LuminancePixelIntensityMethod))
-            gray=EncodePixelGamma(gray);
+          gray=0.212656*GetPixelRed(q)+0.715158*GetPixelGreen(q)+
+            0.072186*GetPixelBlue(q);
           SetPixelRed(q,ClampToQuantum(gray));
           SetPixelGreen(q,ClampToQuantum(gray));
           SetPixelBlue(q,ClampToQuantum(gray));
@@ -2080,10 +2200,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2303,10 +2423,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2369,10 +2489,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickBooleanType
           sync;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2603,10 +2723,10 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickPixelPacket
           pixel;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2620,7 +2740,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           }
         for (x=0; x < (ssize_t) image->columns; x++)
         {
-          register size_t
+          size_t
             blue,
             green,
             red;
@@ -2686,7 +2806,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
         MagickPixelPacket
           pixel;
 
-        register size_t
+        size_t
           blue,
           green,
           red;
