@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -264,21 +264,21 @@ MagickExport Image *CompareImageChannels(Image *image,
       pixel,
       reconstruct_pixel;
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict highlight_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict r;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -316,6 +316,7 @@ MagickExport Image *CompareImageChannels(Image *image,
           double
             Da,
             distance,
+            pixel,
             Sa;
 
           Sa=QuantumScale*(image->matte != MagickFalse ? GetPixelAlpha(p) :
@@ -324,34 +325,39 @@ MagickExport Image *CompareImageChannels(Image *image,
             (QuantumRange-OpaqueOpacity));
           if ((channel & RedChannel) != 0)
             {
-              distance=Sa*GetPixelRed(p)-Da*GetPixelRed(q);
-              if ((distance*distance) > fuzz)
+              pixel=Sa*GetPixelRed(p)-Da*GetPixelRed(q);
+              distance=pixel*pixel;
+              if (distance >= fuzz)
                 difference=MagickTrue;
             }
           if ((channel & GreenChannel) != 0)
             {
-              distance=Sa*GetPixelGreen(p)-Da*GetPixelGreen(q);
-              if ((distance*distance) > fuzz)
+              pixel=Sa*GetPixelGreen(p)-Da*GetPixelGreen(q);
+              distance=pixel*pixel;
+              if (distance >= fuzz)
                 difference=MagickTrue;
             }
           if ((channel & BlueChannel) != 0)
             {
-              distance=Sa*GetPixelBlue(p)-Da*GetPixelBlue(q);
-              if ((distance*distance) > fuzz)
+              pixel=Sa*GetPixelBlue(p)-Da*GetPixelBlue(q);
+              distance=pixel*pixel;
+              if (distance >= fuzz)
                 difference=MagickTrue;
             }
           if (((channel & OpacityChannel) != 0) &&
               (image->matte != MagickFalse))
             {
-              distance=(double) GetPixelOpacity(p)-GetPixelOpacity(q);
-              if ((distance*distance) > fuzz)
+              pixel=(double) GetPixelOpacity(p)-GetPixelOpacity(q);
+              distance=pixel*pixel;
+              if (distance >= fuzz)
                 difference=MagickTrue;
             }
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
             {
-              distance=Sa*indexes[x]-Da*reconstruct_indexes[x];
-              if ((distance*distance) > fuzz)
+              pixel=Sa*indexes[x]-Da*reconstruct_indexes[x];
+              distance=pixel*pixel;
+              if (distance >= fuzz)
                 difference=MagickTrue;
             }
         }
@@ -450,9 +456,7 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
     Compute the absolute difference in pixels between two images.
   */
   status=MagickTrue;
-  fuzz=MagickMin(GetNumberChannels(image,channel),
-    GetNumberChannels(reconstruct_image,channel))*
-    GetFuzzyColorDistance(image,reconstruct_image);
+  fuzz=GetFuzzyColorDistance(image,reconstruct_image);
   rows=MagickMax(image->rows,reconstruct_image->rows);
   columns=MagickMax(image->columns,reconstruct_image->columns);
   image_view=AcquireVirtualCacheView(image,exception);
@@ -466,15 +470,15 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
     double
       channel_distortion[CompositeChannels+1];
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -506,12 +510,11 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
         (QuantumRange-OpaqueOpacity));
       Da=QuantumScale*(image->matte != MagickFalse ? GetPixelAlpha(q) :
         (QuantumRange-OpaqueOpacity));
-      distance=0.0;
       if ((channel & RedChannel) != 0)
         {
           pixel=Sa*GetPixelRed(p)-Da*GetPixelRed(q);
-          distance+=pixel*pixel;
-          if (distance > fuzz)
+          distance=pixel*pixel;
+          if (distance >= fuzz)
             {
               channel_distortion[RedChannel]++;
               difference=MagickTrue;
@@ -520,8 +523,8 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
       if ((channel & GreenChannel) != 0)
         {
           pixel=Sa*GetPixelGreen(p)-Da*GetPixelGreen(q);
-          distance+=pixel*pixel;
-          if (distance > fuzz)
+          distance=pixel*pixel;
+          if (distance >= fuzz)
             {
               channel_distortion[GreenChannel]++;
               difference=MagickTrue;
@@ -530,8 +533,8 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
       if ((channel & BlueChannel) != 0)
         {
           pixel=Sa*GetPixelBlue(p)-Da*GetPixelBlue(q);
-          distance+=pixel*pixel;
-          if (distance > fuzz)
+          distance=pixel*pixel;
+          if (distance >= fuzz)
             {
               channel_distortion[BlueChannel]++;
               difference=MagickTrue;
@@ -541,8 +544,8 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
           (image->matte != MagickFalse))
         {
           pixel=(double) GetPixelOpacity(p)-GetPixelOpacity(q);
-          distance+=pixel*pixel;
-          if (distance > fuzz)
+          distance=pixel*pixel;
+          if (distance >= fuzz)
             {
               channel_distortion[OpacityChannel]++;
               difference=MagickTrue;
@@ -552,8 +555,8 @@ static MagickBooleanType GetAbsoluteDistortion(const Image *image,
           (image->colorspace == CMYKColorspace))
         {
           pixel=Sa*indexes[x]-Da*reconstruct_indexes[x];
-          distance+=pixel*pixel;
-          if (distance > fuzz)
+          distance=pixel*pixel;
+          if (distance >= fuzz)
             {
               channel_distortion[BlackChannel]++;
               difference=MagickTrue;
@@ -586,7 +589,7 @@ static MagickBooleanType GetFuzzDistortion(const Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -610,15 +613,15 @@ static MagickBooleanType GetFuzzDistortion(const Image *image,
     double
       channel_distortion[CompositeChannels+1];
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -711,7 +714,7 @@ static MagickBooleanType GetMeanAbsoluteDistortion(const Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -735,15 +738,15 @@ static MagickBooleanType GetMeanAbsoluteDistortion(const Image *image,
     double
       channel_distortion[CompositeChannels+1];
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -772,35 +775,38 @@ static MagickBooleanType GetMeanAbsoluteDistortion(const Image *image,
         GetPixelAlpha(q) : (QuantumRange-OpaqueOpacity));
       if ((channel & RedChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelRed(p)-Da*GetPixelRed(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelRed(p)-Da*
+            GetPixelRed(q)));
           channel_distortion[RedChannel]+=distance;
           channel_distortion[CompositeChannels]+=distance;
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelGreen(p)-Da*GetPixelGreen(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelGreen(p)-Da*
+            GetPixelGreen(q)));
           channel_distortion[GreenChannel]+=distance;
           channel_distortion[CompositeChannels]+=distance;
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelBlue(p)-Da*GetPixelBlue(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelBlue(p)-Da*
+            GetPixelBlue(q)));
           channel_distortion[BlueChannel]+=distance;
           channel_distortion[CompositeChannels]+=distance;
         }
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=QuantumScale*fabs(GetPixelOpacity(p)-(double)
-            GetPixelOpacity(q));
+          distance=QuantumScale*fabs((double) (GetPixelOpacity(p)-(double)
+            GetPixelOpacity(q)));
           channel_distortion[OpacityChannel]+=distance;
           channel_distortion[CompositeChannels]+=distance;
         }
       if (((channel & IndexChannel) != 0) &&
           (image->colorspace == CMYKColorspace))
         {
-          distance=QuantumScale*fabs(Sa*GetPixelIndex(indexes+x)-Da*
-            GetPixelIndex(reconstruct_indexes+x));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelIndex(indexes+x)-Da*
+            GetPixelIndex(reconstruct_indexes+x)));
           channel_distortion[BlackChannel]+=distance;
           channel_distortion[CompositeChannels]+=distance;
         }
@@ -855,15 +861,15 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
   reconstruct_view=AcquireVirtualCacheView(reconstruct_image,exception);
   for (y=0; y < (ssize_t) rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,columns,1,exception);
@@ -888,7 +894,7 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
         GetPixelAlpha(q) : (QuantumRange-OpaqueOpacity));
       if ((channel & RedChannel) != 0)
         {
-          distance=fabs(Sa*GetPixelRed(p)-Da*GetPixelRed(q));
+          distance=fabs((double) (Sa*GetPixelRed(p)-Da*GetPixelRed(q)));
           distortion[RedChannel]+=distance;
           distortion[CompositeChannels]+=distance;
           mean_error+=distance*distance;
@@ -898,7 +904,7 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=fabs(Sa*GetPixelGreen(p)-Da*GetPixelGreen(q));
+          distance=fabs((double) (Sa*GetPixelGreen(p)-Da*GetPixelGreen(q)));
           distortion[GreenChannel]+=distance;
           distortion[CompositeChannels]+=distance;
           mean_error+=distance*distance;
@@ -908,7 +914,7 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=fabs(Sa*GetPixelBlue(p)-Da*GetPixelBlue(q));
+          distance=fabs((double) (Sa*GetPixelBlue(p)-Da*GetPixelBlue(q)));
           distortion[BlueChannel]+=distance;
           distortion[CompositeChannels]+=distance;
           mean_error+=distance*distance;
@@ -919,8 +925,8 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=fabs((double) GetPixelOpacity(p)-
-            GetPixelOpacity(q));
+          distance=fabs((double) (GetPixelOpacity(p)-
+            (double) GetPixelOpacity(q)));
           distortion[OpacityChannel]+=distance;
           distortion[CompositeChannels]+=distance;
           mean_error+=distance*distance;
@@ -932,8 +938,8 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
           (image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
-          distance=fabs(Sa*GetPixelIndex(indexes+x)-Da*
-            GetPixelIndex(reconstruct_indexes+x));
+          distance=fabs((double) (Sa*GetPixelIndex(indexes+x)-Da*
+            GetPixelIndex(reconstruct_indexes+x)));
           distortion[BlackChannel]+=distance;
           distortion[CompositeChannels]+=distance;
           mean_error+=distance*distance;
@@ -965,7 +971,7 @@ static MagickBooleanType GetMeanSquaredDistortion(const Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -989,15 +995,15 @@ static MagickBooleanType GetMeanSquaredDistortion(const Image *image,
     double
       channel_distortion[CompositeChannels+1];
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -1099,7 +1105,7 @@ static MagickBooleanType GetNormalizedCrossCorrelationDistortion(
   MagickRealType
     area;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1136,15 +1142,15 @@ static MagickBooleanType GetNormalizedCrossCorrelationDistortion(
   reconstruct_view=AcquireVirtualCacheView(reconstruct_image,exception);
   for (y=0; y < (ssize_t) rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -1283,15 +1289,15 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
     double
       channel_distortion[CompositeChannels+1];
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -1320,7 +1326,8 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
         GetPixelAlpha(q) : (QuantumRange-OpaqueOpacity));
       if ((channel & RedChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelRed(p)-Da*GetPixelRed(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelRed(p)-Da*
+            GetPixelRed(q)));
           if (distance > channel_distortion[RedChannel])
             channel_distortion[RedChannel]=distance;
           if (distance > channel_distortion[CompositeChannels])
@@ -1328,7 +1335,8 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelGreen(p)-Da*GetPixelGreen(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelGreen(p)-Da*
+            GetPixelGreen(q)));
           if (distance > channel_distortion[GreenChannel])
             channel_distortion[GreenChannel]=distance;
           if (distance > channel_distortion[CompositeChannels])
@@ -1336,7 +1344,8 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=QuantumScale*fabs(Sa*GetPixelBlue(p)-Da*GetPixelBlue(q));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelBlue(p)-Da*
+            GetPixelBlue(q)));
           if (distance > channel_distortion[BlueChannel])
             channel_distortion[BlueChannel]=distance;
           if (distance > channel_distortion[CompositeChannels])
@@ -1345,8 +1354,8 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=QuantumScale*fabs(GetPixelOpacity(p)-(double)
-            GetPixelOpacity(q));
+          distance=QuantumScale*fabs((double) (GetPixelOpacity(p)-(double)
+            GetPixelOpacity(q)));
           if (distance > channel_distortion[OpacityChannel])
             channel_distortion[OpacityChannel]=distance;
           if (distance > channel_distortion[CompositeChannels])
@@ -1356,8 +1365,8 @@ static MagickBooleanType GetPeakAbsoluteDistortion(const Image *image,
           (image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
-          distance=QuantumScale*fabs(Sa*GetPixelIndex(indexes+x)-Da*
-            GetPixelIndex(reconstruct_indexes+x));
+          distance=QuantumScale*fabs((double) (Sa*GetPixelIndex(indexes+x)-Da*
+            GetPixelIndex(reconstruct_indexes+x)));
           if (distance > channel_distortion[BlackChannel])
             channel_distortion[BlackChannel]=distance;
           if (distance > channel_distortion[CompositeChannels])
@@ -1455,7 +1464,7 @@ static MagickBooleanType GetPerceptualHashDistortion(const Image *image,
   double
     difference;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1935,15 +1944,15 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
   reconstruct_view=AcquireVirtualCacheView(reconstruct_image,exception);
   for (y=0; y < (ssize_t) rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes,
       *magick_restrict reconstruct_indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,columns,1,exception);
@@ -1957,19 +1966,19 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
       MagickRealType
         distance;
 
-      distance=fabs(GetPixelRed(p)-(double) GetPixelRed(q));
+      distance=fabs((double) (GetPixelRed(p)-(double) GetPixelRed(q)));
       mean_error_per_pixel+=distance;
       mean_error+=distance*distance;
       if (distance > maximum_error)
         maximum_error=distance;
       area++;
-      distance=fabs(GetPixelGreen(p)-(double) GetPixelGreen(q));
+      distance=fabs((double) (GetPixelGreen(p)-(double) GetPixelGreen(q)));
       mean_error_per_pixel+=distance;
       mean_error+=distance*distance;
       if (distance > maximum_error)
         maximum_error=distance;
       area++;
-      distance=fabs(GetPixelBlue(p)-(double) GetPixelBlue(q));
+      distance=fabs((double) (GetPixelBlue(p)-(double) GetPixelBlue(q)));
       mean_error_per_pixel+=distance;
       mean_error+=distance*distance;
       if (distance > maximum_error)
@@ -1977,7 +1986,8 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
       area++;
       if (image->matte != MagickFalse)
         {
-          distance=fabs(GetPixelOpacity(p)-(double) GetPixelOpacity(q));
+          distance=fabs((double) (GetPixelOpacity(p)-(double)
+            GetPixelOpacity(q)));
           mean_error_per_pixel+=distance;
           mean_error+=distance*distance;
           if (distance > maximum_error)
@@ -1987,8 +1997,8 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
       if ((image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
-          distance=fabs(GetPixelIndex(indexes+x)-(double)
-            GetPixelIndex(reconstruct_indexes+x));
+          distance=fabs((double) (GetPixelIndex(indexes+x)-(double)
+            GetPixelIndex(reconstruct_indexes+x)));
           mean_error_per_pixel+=distance;
           mean_error+=distance*distance;
           if (distance > maximum_error)
@@ -2154,10 +2164,10 @@ MagickExport Image *SimilarityMetricImage(Image *image,const Image *reference,
     double
       similarity;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
