@@ -17,7 +17,7 @@
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -53,6 +53,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/version.h"
 #include "magick/xml-tree.h"
 #include "magick/xml-tree-private.h"
 
@@ -60,29 +61,6 @@
   Define declarations.
 */
 #define ConfigureFilename  "configure.xml"
-
-#ifdef _OPENMP
-#define MAGICKCORE_FEATURE_OPENMP_STR "OpenMP "
-#else
-#define MAGICKCORE_FEATURE_OPENMP_STR ""
-#endif
-#ifdef _OPENCL
-#define MAGICKCORE_FEATURE_OPENCL_STR "OpenCL "
-#else
-#define MAGICKCORE_FEATURE_OPENCL_STR ""
-#endif
-#ifdef MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
-#define MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR "Zero-Configuration "
-#else
-#define MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR ""
-#endif
-#ifdef HDRI_SUPPORT
-#define MAGICKCORE_FEATURE_HDRI_STR "HDRI"
-#else
-#define MAGICKCORE_FEATURE_HDRI_STR ""
-#endif
-
-#define MAGICKCORE_FEATURES_STR MAGICKCORE_FEATURE_OPENMP_STR MAGICKCORE_FEATURE_OPENCL_STR MAGICKCORE_FEATURE_ZERO_CONFIGURATION_STR MAGICKCORE_FEATURE_HDRI_STR
 
 /*
   Typedef declarations.
@@ -97,14 +75,6 @@ typedef struct _ConfigureMapInfo
 /*
   Static declarations.
 */
-static const ConfigureMapInfo
-  ConfigureMap[] =
-  {
-    { "NAME", "ImageMagick" },
-    { "QuantumDepth", MAGICKCORE_STRING_XQUOTE(MAGICKCORE_QUANTUM_DEPTH) },
-    { "FEATURES", MAGICKCORE_FEATURES_STR }
-  };
-
 static LinkedListInfo
   *configure_cache = (LinkedListInfo *) NULL;
 
@@ -115,9 +85,13 @@ static SemaphoreInfo
   Forward declarations.
 */
 static MagickBooleanType
-  IsConfigureCacheInstantiated(ExceptionInfo *),
+  IsConfigureCacheInstantiated(ExceptionInfo *);
+
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
+static MagickBooleanType
   LoadConfigureCache(LinkedListInfo *,const char *,const char *,const size_t,
     ExceptionInfo *);
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -145,26 +119,37 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
+static inline void AddConfigureKey(LinkedListInfo *cache,const char *path,
+  const char *name,const char *value)
+{
+  ConfigureInfo
+    *configure_info;
+
+  configure_info=(ConfigureInfo *) AcquireMagickMemory(sizeof(*configure_info));
+  if (configure_info == (ConfigureInfo *) NULL)
+    return;
+  (void) memset(configure_info,0,sizeof(*configure_info));
+  configure_info->path=(char *) path;
+  configure_info->name=(char *) name;
+  configure_info->value=(char *) value;
+  configure_info->exempt=MagickTrue;
+  configure_info->signature=MagickCoreSignature;
+  (void) AppendValueToLinkedList(cache,configure_info);
+}
+
 static LinkedListInfo *AcquireConfigureCache(const char *filename,
   ExceptionInfo *exception)
 {
   LinkedListInfo
     *cache;
 
-  MagickStatusType
-    status;
-
-  register ssize_t
-    i;
-
   /*
     Load external configure map.
   */
   cache=NewLinkedList(0);
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
       *option;
@@ -172,49 +157,33 @@ static LinkedListInfo *AcquireConfigureCache(const char *filename,
     LinkedListInfo
       *options;
 
+    MagickBooleanType
+      status;
+
     options=GetConfigureOptions(filename,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
     while (option != (const StringInfo *) NULL)
     {
-      status&=LoadConfigureCache(cache,(const char *)
+      status=LoadConfigureCache(cache,(const char *)
         GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+      if (status == MagickTrue)
+        break;
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
   }
 #endif
   /*
-    Load built-in configure map.
+    Load built-in configure.
   */
-  for (i=0; i < (ssize_t) (sizeof(ConfigureMap)/sizeof(*ConfigureMap)); i++)
-  {
-    ConfigureInfo
-      *configure_info;
-
-    register const ConfigureMapInfo
-      *p;
-
-    p=ConfigureMap+i;
-    configure_info=(ConfigureInfo *) AcquireMagickMemory(
-      sizeof(*configure_info));
-    if (configure_info == (ConfigureInfo *) NULL)
-      {
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
-        continue;
-      }
-    (void) memset(configure_info,0,sizeof(*configure_info));
-    configure_info->path=(char *) "[built-in]";
-    configure_info->name=(char *) p->name;
-    configure_info->value=(char *) p->value;
-    configure_info->exempt=MagickTrue;
-    configure_info->signature=MagickCoreSignature;
-    status&=AppendValueToLinkedList(cache,configure_info);
-    if (status == MagickFalse)
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",
-        configure_info->name);
-  }
+  AddConfigureKey(cache,"[built-in]","NAME","ImageMagick");
+  /*
+    Load runtime configuration.
+  */
+  AddConfigureKey(cache,"[built-in]","QuantumDepth",GetMagickQuantumDepth(
+    (size_t *)NULL));
+  AddConfigureKey(cache,"[built-in]","FEATURES",GetMagickFeatures());
+  AddConfigureKey(cache,"[built-in]","DELEGATES",GetMagickDelegates());
   return(cache);
 }
 
@@ -264,7 +233,7 @@ MagickExport MagickBooleanType ConfigureComponentGenesis(void)
 
 static void *DestroyConfigureElement(void *configure_info)
 {
-  register ConfigureInfo
+  ConfigureInfo
     *p;
 
   p=(ConfigureInfo *) configure_info;
@@ -360,7 +329,7 @@ MagickExport LinkedListInfo *DestroyConfigureOptions(LinkedListInfo *options)
 MagickExport const ConfigureInfo *GetConfigureInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -447,10 +416,10 @@ MagickExport const ConfigureInfo **GetConfigureInfoList(const char *pattern,
   const ConfigureInfo
     **options;
 
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -522,7 +491,7 @@ extern "C" {
 
 static int ConfigureCompare(const void *x,const void *y)
 {
-  register char
+  char
     **p,
     **q;
 
@@ -541,10 +510,10 @@ MagickExport char **GetConfigureList(const char *pattern,
   char
     **options;
 
-  register const ConfigureInfo
+  const ConfigureInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -770,7 +739,7 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
     configure_path=GetEnvironmentValue("MAGICK_CONFIGURE_PATH");
     if (configure_path != (char *) NULL)
       {
-        register char
+        char
           *p,
           *q;
 
@@ -889,12 +858,14 @@ MagickExport LinkedListInfo *GetConfigurePaths(const char *filename,
       *home;
 
     home=GetEnvironmentValue("XDG_CONFIG_HOME");
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__MINGW32__)
     if (home == (char *) NULL)
       home=GetEnvironmentValue("LOCALAPPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("APPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("USERPROFILE");
+#endif
     if (home != (char *) NULL)
       {
         /*
@@ -1068,7 +1039,7 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
   const ConfigureInfo
     **configure_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1118,12 +1089,13 @@ MagickExport MagickBooleanType ListConfigureInfo(FILE *file,
   return(MagickTrue);
 }
 
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   L o a d C o n f i g u r e L i s t                                         %
++   L o a d C o n f i g u r e C a c h e                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1183,7 +1155,7 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -1193,7 +1165,7 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -1202,7 +1174,7 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1213,10 +1185,10 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > MagickMaxRecursionDepth)
@@ -1253,7 +1225,7 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
         /*
           Configure element.
         */
-        configure_info=(ConfigureInfo *) AcquireMagickMemory(
+        configure_info=(ConfigureInfo *) AcquireQuantumMemory(1,
           sizeof(*configure_info));
         if (configure_info == (ConfigureInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
@@ -1279,11 +1251,11 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
     /*
       Parse configure element.
     */
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetNextToken(q,&q,extent,token);
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'N':
@@ -1323,3 +1295,4 @@ static MagickBooleanType LoadConfigureCache(LinkedListInfo *cache,
   token=(char *) RelinquishMagickMemory(token);
   return(status != 0 ? MagickTrue : MagickFalse);
 }
+#endif
