@@ -16,7 +16,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -186,7 +186,7 @@ static LinkedListInfo *AcquireDelegateCache(const char *filename,
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
       *option;
@@ -257,7 +257,7 @@ MagickExport MagickBooleanType DelegateComponentGenesis(void)
 
 static void *DestroyDelegate(void *delegate_info)
 {
-  register DelegateInfo
+  DelegateInfo
     *p;
 
   p=(DelegateInfo *) delegate_info;
@@ -340,7 +340,7 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
   PolicyRights
     rights;
 
-  register ssize_t
+  ssize_t
     i;
 
   status=(-1);
@@ -377,55 +377,92 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
   if (message != (char *) NULL)
     *message='\0';
 #if defined(MAGICKCORE_POSIX_SUPPORT)
-#if !defined(MAGICKCORE_HAVE_EXECVP)
-  status=system(sanitize_command);
-#else
-  if ((asynchronous != MagickFalse) ||
-      (strpbrk(sanitize_command,"&;<>|") != (char *) NULL))
-    status=system(sanitize_command);
-  else
+#if defined(MAGICKCORE_HAVE_POPEN)
+  if ((asynchronous == MagickFalse) && (message !=  (char *) NULL))
     {
-      pid_t
-        child_pid;
+      char
+        buffer[MagickPathExtent];
 
-      /*
-        Call application directly rather than from a shell.
-      */
-      child_pid=(pid_t) fork();
-      if (child_pid == (pid_t) -1)
+      FILE
+        *file;
+
+      size_t
+        offset;
+
+      offset=0;
+      file=popen_utf8(sanitize_command,"r");
+      if (file == (FILE *) NULL)
         status=system(sanitize_command);
       else
-        if (child_pid == 0)
+        {
+          while (fgets(buffer,(int) sizeof(buffer),file) != NULL)
           {
-            status=execvp(arguments[1],arguments+1);
-            _exit(1);
-          }
-        else
-          {
-            int
-              child_status;
+            size_t
+              length;
 
-            pid_t
-              pid;
-
-            child_status=0;
-            pid=(pid_t) waitpid(child_pid,&child_status,0);
-            if (pid == -1)
-              status=(-1);
-            else
+            length=MagickMin(MagickPathExtent-offset,strlen(buffer)+1);
+            if (length > 0)
               {
-                if (WIFEXITED(child_status) != 0)
-                  status=WEXITSTATUS(child_status);
-                else
-                  if (WIFSIGNALED(child_status))
-                    status=(-1);
+                (void) CopyMagickString(message+offset,buffer,length);
+                offset+=length-1;
               }
           }
+          status=pclose(file);
+        }
     }
+  else
 #endif
+    {
+#if !defined(MAGICKCORE_HAVE_EXECVP)
+      status=system(sanitize_command);
+#else
+      if ((asynchronous != MagickFalse) ||
+          (strpbrk(sanitize_command,"&;<>|") != (char *) NULL))
+        status=system(sanitize_command);
+      else
+        {
+          pid_t
+            child_pid;
+
+          /*
+            Call application directly rather than from a shell.
+          */
+          child_pid=(pid_t) fork();
+          if (child_pid == (pid_t) -1)
+            status=system(sanitize_command);
+          else
+            if (child_pid == 0)
+              {
+                status=execvp(arguments[1],arguments+1);
+                _exit(1);
+              }
+            else
+              {
+                int
+                  child_status;
+
+                pid_t
+                  pid;
+
+                child_status=0;
+                pid=(pid_t) waitpid(child_pid,&child_status,0);
+                if (pid == -1)
+                  status=(-1);
+                else
+                  {
+                    if (WIFEXITED(child_status) != 0)
+                      status=WEXITSTATUS(child_status);
+                    else
+                      if (WIFSIGNALED(child_status))
+                        status=(-1);
+                  }
+              }
+        }
+#endif
+    }
 #elif defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
-    register char
+    char
       *p;
 
     /*
@@ -436,18 +473,16 @@ MagickExport int ExternalDelegateCommand(const MagickBooleanType asynchronous,
       TODO: This won't work if one of the delegate parameters has a forward
             slash as aparameter.
     */
-    p=strstr(sanitize_command, "cmd.exe /c");
+    p=strstr(sanitize_command,"cmd.exe /c");
     if (p != (char*) NULL)
       {
         p+=10;
-        for (; *p != '\0'; p++)
+        for ( ; *p != '\0'; p++)
           if (*p == '/')
-            *p=*DirectorySeparator;
+            *p=(*DirectorySeparator);
       }
   }
   status=NTSystemCommand(sanitize_command,message);
-#elif defined(macintosh)
-  status=MACSystemCommand(sanitize_command);
 #elif defined(vms)
   status=system(sanitize_command);
 #else
@@ -774,6 +809,15 @@ static char *GetMagickPropertyLetter(const ImageInfo *image_info,Image *image,
         image->page.height);
       break;
     }
+    case 'I':
+    {
+      /*
+        Image iterations for animations.
+      */
+      (void) FormatLocaleString(value,MaxTextExtent,"%.20g",(double)
+        image->iterations);
+      break;
+    }
     case 'M':
     {
       /*
@@ -918,14 +962,13 @@ static char *GetMagickPropertyLetter(const ImageInfo *image_info,Image *image,
       break;
     }
   }
-  return(SanitizeString(string));
+  return(SanitizeDelegateString(string));
 }
 
 static char *InterpretDelegateProperties(const ImageInfo *image_info,
   Image *image,const char *embed_text)
 {
 #define ExtendInterpretText(string_length) \
-DisableMSCWarning(4127) \
 { \
   size_t length=(string_length); \
   if ((size_t) (q-interpret_text+length+1) >= extent) \
@@ -937,11 +980,9 @@ DisableMSCWarning(4127) \
         return((char *) NULL); \
       q=interpret_text+strlen(interpret_text); \
    } \
-} \
-RestoreMSCWarning
+}
 
 #define AppendKeyValue2Text(key,value)\
-DisableMSCWarning(4127) \
 { \
   size_t length=strlen(key)+strlen(value)+2; \
   if ((size_t) (q-interpret_text+length+1) >= extent) \
@@ -954,11 +995,9 @@ DisableMSCWarning(4127) \
       q=interpret_text+strlen(interpret_text); \
      } \
    q+=FormatLocaleString(q,extent,"%s=%s\n",(key),(value)); \
-} \
-RestoreMSCWarning
+}
 
 #define AppendString2Text(string) \
-DisableMSCWarning(4127) \
 { \
   size_t length=strlen((string)); \
   if ((size_t) (q-interpret_text+length+1) >= extent) \
@@ -972,17 +1011,16 @@ DisableMSCWarning(4127) \
     } \
   (void) CopyMagickString(q,(string),extent); \
   q+=length; \
-} \
-RestoreMSCWarning
+}
 
   char
     *interpret_text,
     *property;
 
-  register char
+  char
     *q;  /* current position in interpret_text */
 
-  register const char
+  const char
     *p;  /* position in embed_text string being expanded */
 
   size_t
@@ -1131,7 +1169,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
   const DelegateInfo
     *delegate_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(image_info != (ImageInfo *) NULL);
@@ -1230,7 +1268,7 @@ MagickExport const char *GetDelegateCommands(const DelegateInfo *delegate_info)
 MagickExport const DelegateInfo *GetDelegateInfo(const char *decode,
   const char *encode,ExceptionInfo *exception)
 {
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -1349,10 +1387,10 @@ MagickExport const DelegateInfo **GetDelegateInfoList(const char *pattern,
   const DelegateInfo
     **delegates;
 
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1426,7 +1464,7 @@ extern "C" {
 
 static int DelegateCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -1445,10 +1483,10 @@ MagickExport char **GetDelegateList(const char *pattern,
   char
     **delegates;
 
-  register const DelegateInfo
+  const DelegateInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1625,7 +1663,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
   MagickBooleanType
     status;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -1707,7 +1745,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   PolicyRights
     rights;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1774,7 +1812,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       ImageInfo
         *clone_info;
 
-      register Image
+      Image
         *p;
 
       /*
@@ -1981,7 +2019,7 @@ MagickExport MagickBooleanType ListDelegateInfo(FILE *file,
   const char
     *path;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2106,7 +2144,7 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -2116,7 +2154,7 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -2125,7 +2163,7 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -2136,10 +2174,10 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > MagickMaxRecursionDepth)
@@ -2176,7 +2214,7 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
         /*
           Delegate element.
         */
-        delegate_info=(DelegateInfo *) AcquireMagickMemory(
+        delegate_info=(DelegateInfo *) AcquireQuantumMemory(1,
           sizeof(*delegate_info));
         if (delegate_info == (DelegateInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
@@ -2199,11 +2237,11 @@ static MagickBooleanType LoadDelegateCache(LinkedListInfo *cache,
         delegate_info=(DelegateInfo *) NULL;
         continue;
       }
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetNextToken(q,&q,extent,token);
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'C':
