@@ -22,7 +22,7 @@
 %                                January 2013                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -147,7 +147,7 @@
 static inline MagickOffsetType dpc_read(int file,const MagickSizeType length,
   unsigned char *magick_restrict message)
 {
-  register MagickOffsetType
+  MagickOffsetType
     i;
 
   ssize_t
@@ -162,7 +162,7 @@ static inline MagickOffsetType dpc_read(int file,const MagickSizeType length,
   for (i=0; i < (MagickOffsetType) length; i+=count)
   {
     count=recv(file,CHAR_TYPE_CAST message+i,(LENGTH_TYPE) MagickMin(length-i,
-      (MagickSizeType) SSIZE_MAX),0);
+      (MagickSizeType) MAGICK_SSIZE_MAX),0);
     if (count <= 0)
       {
         count=0;
@@ -206,7 +206,6 @@ static int ConnectPixelCacheServer(const char *hostname,const int port,
   shared_secret=GetPolicyValue("cache:shared-secret");
   if (shared_secret == (char *) NULL)
     {
-      shared_secret=DestroyString(shared_secret);
       (void) ThrowMagickException(exception,GetMagickModule(),CacheError,
         "DistributedPixelCache","'%s'","shared secret expected");
       return(-1);
@@ -284,7 +283,7 @@ static char *GetHostname(int *port,ExceptionInfo *exception)
   int
     argc;
 
-  register ssize_t
+  ssize_t
     i;
 
   static size_t
@@ -443,7 +442,7 @@ static inline MagickOffsetType dpc_send(int file,const MagickSizeType length,
   MagickOffsetType
     count;
 
-  register MagickOffsetType
+  MagickOffsetType
     i;
 
 #if !defined(MAGICKCORE_HAVE_DISTRIBUTE_CACHE)
@@ -458,7 +457,7 @@ static inline MagickOffsetType dpc_send(int file,const MagickSizeType length,
   for (i=0; i < (MagickOffsetType) length; i+=count)
   {
     count=(MagickOffsetType) send(file,CHAR_TYPE_CAST message+i,(LENGTH_TYPE)
-      MagickMin(length-i,(MagickSizeType) SSIZE_MAX),MSG_NOSIGNAL);
+      MagickMin(length-i,(MagickSizeType) MAGICK_SSIZE_MAX),MSG_NOSIGNAL);
     if (count <= 0)
       {
         count=0;
@@ -484,7 +483,7 @@ static MagickBooleanType OpenDistributeCache(SplayTreeInfo *registry,int file,
   MagickSizeType
     length;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -539,10 +538,10 @@ static MagickBooleanType ReadDistributeCacheIndexes(SplayTreeInfo *registry,
   RectangleInfo
     region;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register unsigned char
+  unsigned char
     *q;
 
   unsigned char
@@ -596,10 +595,10 @@ static MagickBooleanType ReadDistributeCachePixels(SplayTreeInfo *registry,
   RectangleInfo
     region;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register unsigned char
+  unsigned char
     *q;
 
   unsigned char
@@ -660,10 +659,10 @@ static MagickBooleanType WriteDistributeCacheIndexes(SplayTreeInfo *registry,
   RectangleInfo
     region;
 
-  register unsigned char
+  unsigned char
     *p;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
   unsigned char
@@ -717,10 +716,10 @@ static MagickBooleanType WriteDistributeCachePixels(SplayTreeInfo *registry,
   RectangleInfo
     region;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -775,7 +774,7 @@ static HANDLER_RETURN_TYPE DistributePixelCacheClient(void *socket)
   RandomInfo
     *random_info;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -901,7 +900,7 @@ MagickExport void DistributePixelCacheServer(const int port,
   Not implemented!
 #endif
 
-  register struct addrinfo
+  struct addrinfo
     *p;
 
   SOCKET_TYPE
@@ -1127,7 +1126,7 @@ MagickPrivate MagickBooleanType OpenDistributePixelCache(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1207,7 +1206,7 @@ MagickPrivate MagickOffsetType ReadDistributePixelCacheIndexes(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1220,7 +1219,7 @@ MagickPrivate MagickOffsetType ReadDistributePixelCacheIndexes(
   assert(server_info->signature == MagickCoreSignature);
   assert(region != (RectangleInfo *) NULL);
   assert(indexes != (unsigned char *) NULL);
-  if (length > (MagickSizeType) SSIZE_MAX)
+  if (length > (MagickSizeType) MAGICK_SSIZE_MAX)
     return(-1);
   p=message;
   *p++='R';
@@ -1282,7 +1281,7 @@ MagickPrivate MagickOffsetType ReadDistributePixelCachePixels(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1295,7 +1294,7 @@ MagickPrivate MagickOffsetType ReadDistributePixelCachePixels(
   assert(server_info->signature == MagickCoreSignature);
   assert(region != (RectangleInfo *) NULL);
   assert(pixels != (unsigned char *) NULL);
-  if (length > (MagickSizeType) SSIZE_MAX)
+  if (length > (MagickSizeType) MAGICK_SSIZE_MAX)
     return(-1);
   p=message;
   *p++='r';
@@ -1354,7 +1353,7 @@ MagickPrivate MagickBooleanType RelinquishDistributePixelCache(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1418,7 +1417,7 @@ MagickPrivate MagickOffsetType WriteDistributePixelCacheIndexes(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1431,7 +1430,7 @@ MagickPrivate MagickOffsetType WriteDistributePixelCacheIndexes(
   assert(server_info->signature == MagickCoreSignature);
   assert(region != (RectangleInfo *) NULL);
   assert(indexes != (unsigned char *) NULL);
-  if (length > (MagickSizeType) SSIZE_MAX)
+  if (length > (MagickSizeType) MAGICK_SSIZE_MAX)
     return(-1);
   p=message;
   *p++='W';
@@ -1494,7 +1493,7 @@ MagickPrivate MagickOffsetType WriteDistributePixelCachePixels(
   MagickOffsetType
     count;
 
-  register unsigned char
+  unsigned char
     *p;
 
   unsigned char
@@ -1507,7 +1506,7 @@ MagickPrivate MagickOffsetType WriteDistributePixelCachePixels(
   assert(server_info->signature == MagickCoreSignature);
   assert(region != (RectangleInfo *) NULL);
   assert(pixels != (const unsigned char *) NULL);
-  if (length > (MagickSizeType) SSIZE_MAX)
+  if (length > (MagickSizeType) MAGICK_SSIZE_MAX)
     return(-1);
   p=message;
   *p++='w';
