@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
   
     https://imagemagick.org/script/license.php
@@ -96,9 +96,11 @@ extern "C" {
 }
 
 extern MagickPrivate MagickBooleanType
-  ClearExceptionInfo(ExceptionInfo *,MagickBooleanType);
+  ClearExceptionInfo(ExceptionInfo *,MagickBooleanType),
+  ExceptionComponentGenesis(void);
 
 extern MagickPrivate void
+  ExceptionComponentTerminus(void),
   InitializeExceptionInfo(ExceptionInfo *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
