@@ -17,7 +17,7 @@
 %                                July 1993                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -49,6 +49,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
 
@@ -85,6 +86,12 @@ static FatalErrorHandler
 static WarningHandler
   warning_handler = DefaultWarningHandler;
 
+/*
+  Static declarations.
+*/
+static SemaphoreInfo
+  *exception_semaphore = (SemaphoreInfo *) NULL;
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -142,7 +149,7 @@ MagickExport ExceptionInfo *AcquireExceptionInfo(void)
 
 static void *DestroyExceptionElement(void *exception)
 {
-  register ExceptionInfo
+  ExceptionInfo
     *p;
 
   p=(ExceptionInfo *) exception;
@@ -198,7 +205,7 @@ MagickExport void CatchException(ExceptionInfo *exception)
   LinkedListInfo
     *exceptions;
 
-  register const ExceptionInfo
+  const ExceptionInfo
     *p;
 
   ssize_t
@@ -450,6 +457,58 @@ MagickExport ExceptionInfo *DestroyExceptionInfo(ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   E x e c e p t i o n C o m p o n e n t G e n e s i s                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ExceptionComponentGenesis() instantiates the exception component.
+%
+%  The format of the ExceptionComponentGenesis method is:
+%
+%      MagickBooleanType ExceptionComponentGenesis(void)
+%
+*/
+MagickPrivate MagickBooleanType ExceptionComponentGenesis(void)
+{
+  if (exception_semaphore == (SemaphoreInfo *) NULL)
+    exception_semaphore=AllocateSemaphoreInfo();
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   E x c e p t i o n C o m p o n e n t T e r m i n u s                       %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ExceptionComponentTerminus() destroys the exception component.
+%
+%  The format of the ExceptionComponentTerminus method is:
+%
+%      void ExceptionComponentTerminus(void)
+%
+*/
+MagickPrivate void ExceptionComponentTerminus(void)
+{
+  if (exception_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&exception_semaphore);
+  LockSemaphoreInfo(exception_semaphore);
+  UnlockSemaphoreInfo(exception_semaphore);
+  DestroySemaphoreInfo(&exception_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t E x c e p t i o n M e s s a g e                                     %
 %                                                                             %
 %                                                                             %
@@ -633,7 +692,7 @@ MagickExport const char *GetLocaleExceptionMessage(const ExceptionType severity,
 MagickExport void InheritException(ExceptionInfo *exception,
   const ExceptionInfo *relative)
 {
-  register const ExceptionInfo
+  const ExceptionInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -821,8 +880,12 @@ MagickExport ErrorHandler SetErrorHandler(ErrorHandler handler)
   ErrorHandler
     previous_handler;
 
+  if (exception_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&exception_semaphore);
+  LockSemaphoreInfo(exception_semaphore);
   previous_handler=error_handler;
   error_handler=handler;
+  UnlockSemaphoreInfo(exception_semaphore);
   return(previous_handler);
 }
 
@@ -854,8 +917,12 @@ MagickExport FatalErrorHandler SetFatalErrorHandler(FatalErrorHandler handler)
   FatalErrorHandler
     previous_handler;
 
+  if (exception_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&exception_semaphore);
+  LockSemaphoreInfo(exception_semaphore);
   previous_handler=fatal_error_handler;
   fatal_error_handler=handler;
+  UnlockSemaphoreInfo(exception_semaphore);
   return(previous_handler);
 }
 
@@ -887,8 +954,12 @@ MagickExport WarningHandler SetWarningHandler(WarningHandler handler)
   WarningHandler
     previous_handler;
 
+  if (exception_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&exception_semaphore);
+  LockSemaphoreInfo(exception_semaphore);
   previous_handler=warning_handler;
   warning_handler=handler;
+  UnlockSemaphoreInfo(exception_semaphore);
   return(previous_handler);
 }
 
@@ -929,7 +1000,7 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
   LinkedListInfo
     *exceptions;
 
-  register ExceptionInfo
+  ExceptionInfo
     *p;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -938,8 +1009,17 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
   exceptions=(LinkedListInfo *) exception->exceptions;
   if (GetNumberOfElementsInLinkedList(exceptions) > MaxExceptionList)
     {
-      UnlockSemaphoreInfo(exception->semaphore);
-      return(MagickTrue);
+      if (severity < ErrorException)
+        {
+          UnlockSemaphoreInfo(exception->semaphore);
+          return(MagickTrue);
+        }
+      p=(ExceptionInfo *) GetLastValueInLinkedList(exceptions);
+      if (p->severity >= ErrorException)
+        {
+          UnlockSemaphoreInfo(exception->semaphore);
+          return(MagickTrue);
+        }
     }
   p=(ExceptionInfo *) GetLastValueInLinkedList(exceptions);
   if ((p != (ExceptionInfo *) NULL) && (p->severity == severity) &&
@@ -971,8 +1051,9 @@ MagickExport MagickBooleanType ThrowException(ExceptionInfo *exception,
     }
   UnlockSemaphoreInfo(exception->semaphore);
   if (GetNumberOfElementsInLinkedList(exceptions) == MaxExceptionList)
-    (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
-      "TooManyExceptions","(exception processing is suspended)");
+    (void) ThrowMagickException(exception,GetMagickModule(),
+      ResourceLimitWarning,"TooManyExceptions",
+      "(exception processing is suspended)");
   return(MagickTrue);
 }
 
