@@ -19,7 +19,7 @@
 %                                 July 2009                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -164,6 +164,10 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
   MagickOffsetType
     progress;
 
+  size_t
+    columns,
+    rows;
+
   ssize_t
     y;
 
@@ -190,12 +194,18 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
   image->depth=32UL;
   complex_images=NewImageList();
   AppendImageToList(&complex_images,image);
-  image=CloneImage(images,0,0,MagickTrue,exception);
+  image=CloneImage(images->next,0,0,MagickTrue,exception);
   if (image == (Image *) NULL)
     {
       complex_images=DestroyImageList(complex_images);
       return(complex_images);
     }
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return(image);
+    }
+  image->depth=32UL;
   AppendImageToList(&complex_images,image);
   /*
     Apply complex mathematics to image pixels.
@@ -224,33 +234,35 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
   Ci_view=AcquireAuthenticCacheView(Ci_image,exception);
   status=MagickTrue;
   progress=0;
+  columns=MagickMin(Cr_image->columns,Ci_image->columns);
+  rows=MagickMin(Cr_image->rows,Ci_image->rows);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(static) shared(progress,status) \
-    magick_number_threads(images,complex_images,images->rows,1L)
+    magick_number_threads(Cr_image,complex_images,rows,1L)
 #endif
-  for (y=0; y < (ssize_t) images->rows; y++)
+  for (y=0; y < (ssize_t) rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict Ai,
       *magick_restrict Ar,
       *magick_restrict Bi,
       *magick_restrict Br;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict Ci,
       *magick_restrict Cr;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
       continue;
-    Ar=GetCacheViewVirtualPixels(Ar_view,0,y,Ar_image->columns,1,exception);
-    Ai=GetCacheViewVirtualPixels(Ai_view,0,y,Ai_image->columns,1,exception);
-    Br=GetCacheViewVirtualPixels(Br_view,0,y,Br_image->columns,1,exception);
-    Bi=GetCacheViewVirtualPixels(Bi_view,0,y,Bi_image->columns,1,exception);
-    Cr=QueueCacheViewAuthenticPixels(Cr_view,0,y,Cr_image->columns,1,exception);
-    Ci=QueueCacheViewAuthenticPixels(Ci_view,0,y,Ci_image->columns,1,exception);
+    Ar=GetCacheViewVirtualPixels(Ar_view,0,y,columns,1,exception);
+    Ai=GetCacheViewVirtualPixels(Ai_view,0,y,columns,1,exception);
+    Br=GetCacheViewVirtualPixels(Br_view,0,y,columns,1,exception);
+    Bi=GetCacheViewVirtualPixels(Bi_view,0,y,columns,1,exception);
+    Cr=QueueCacheViewAuthenticPixels(Cr_view,0,y,columns,1,exception);
+    Ci=QueueCacheViewAuthenticPixels(Ci_view,0,y,columns,1,exception);
     if ((Ar == (const PixelPacket *) NULL) ||
         (Ai == (const PixelPacket *) NULL) ||
         (Br == (const PixelPacket *) NULL) ||
@@ -260,7 +272,7 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
         status=MagickFalse;
         continue;
       }
-    for (x=0; x < (ssize_t) images->columns; x++)
+    for (x=0; x < (ssize_t) columns; x++)
     {
       switch (op)
       {
@@ -300,56 +312,76 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
           double
             gamma;
 
-          gamma=PerceptibleReciprocal(Br->red*Br->red+Bi->red*Bi->red+snr);
-          Cr->red=gamma*(Ar->red*Br->red+Ai->red*Bi->red);
-          Ci->red=gamma*(Ai->red*Br->red-Ar->red*Bi->red);
-          gamma=PerceptibleReciprocal(Br->green*Br->green+Bi->green*Bi->green+
-            snr);
-          Cr->green=gamma*(Ar->green*Br->green+Ai->green*Bi->green);
-          Ci->green=gamma*(Ai->green*Br->green-Ar->green*Bi->green);
-          gamma=PerceptibleReciprocal(Br->blue*Br->blue+Bi->blue*Bi->blue+snr);
-          Cr->blue=gamma*(Ar->blue*Br->blue+Ai->blue*Bi->blue);
-          Ci->blue=gamma*(Ai->blue*Br->blue-Ar->blue*Bi->blue);
+          gamma=QuantumRange*PerceptibleReciprocal(QuantumScale*Br->red*Br->red+
+            QuantumScale*Bi->red*Bi->red+snr);
+          Cr->red=gamma*(QuantumScale*Ar->red*Br->red+QuantumScale*Ai->red*
+            Bi->red);
+          Ci->red=gamma*(QuantumScale*Ai->red*Br->red-QuantumScale*Ar->red*
+            Bi->red);
+          gamma=QuantumRange*PerceptibleReciprocal(QuantumScale*Br->green*
+            Br->green+QuantumScale*Bi->green*Bi->green+snr);
+          Cr->green=gamma*(QuantumScale*Ar->green*Br->green+QuantumScale*
+            Ai->green*Bi->green);
+          Ci->green=gamma*(QuantumScale*Ai->green*Br->green-QuantumScale*
+            Ar->green*Bi->green);
+          gamma=QuantumRange*PerceptibleReciprocal(QuantumScale*Br->blue*
+            Br->blue+QuantumScale*Bi->blue*Bi->blue+snr);
+          Cr->blue=gamma*(QuantumScale*Ar->blue*Br->blue+QuantumScale*
+            Ai->blue*Bi->blue);
+          Ci->blue=gamma*(QuantumScale*Ai->blue*Br->blue-QuantumScale*
+            Ar->blue*Bi->blue);
           if (images->matte != MagickFalse)
             {
-              gamma=PerceptibleReciprocal(Br->opacity*Br->opacity+Bi->opacity*
-                Bi->opacity+snr);
-              Cr->opacity=gamma*(Ar->opacity*Br->opacity+Ai->opacity*
-                Bi->opacity);
-              Ci->opacity=gamma*(Ai->opacity*Br->opacity-Ar->opacity*
-                Bi->opacity);
+              gamma=QuantumRange*PerceptibleReciprocal(QuantumScale*Br->opacity*
+                Br->opacity+QuantumScale*Bi->opacity*Bi->opacity+snr);
+              Cr->opacity=gamma*(QuantumScale*Ar->opacity*Br->opacity+
+                QuantumScale*Ai->opacity*Bi->opacity);
+              Ci->opacity=gamma*(QuantumScale*Ai->opacity*Br->opacity-
+                QuantumScale*Ar->opacity*Bi->opacity);
             }
           break;
         }
         case MagnitudePhaseComplexOperator:
         {
-          Cr->red=sqrt(Ar->red*Ar->red+Ai->red*Ai->red);
-          Ci->red=atan2(Ai->red,Ar->red)/(2.0*MagickPI)+0.5;
-          Cr->green=sqrt(Ar->green*Ar->green+Ai->green*Ai->green);
-          Ci->green=atan2(Ai->green,Ar->green)/(2.0*MagickPI)+0.5;
-          Cr->blue=sqrt(Ar->blue*Ar->blue+Ai->blue*Ai->blue);
+          Cr->red=sqrt(QuantumScale*Ar->red*Ar->red+QuantumScale*
+            Ai->red*Ai->red);
+          Ci->red=atan2((double) Ai->red,(double) Ar->red)/(2.0*MagickPI)+0.5;
+          Cr->green=sqrt(QuantumScale*Ar->green*Ar->green+QuantumScale*
+            Ai->green*Ai->green);
+          Ci->green=atan2((double) Ai->green,(double) Ar->green)/
+            (2.0*MagickPI)+0.5;
+          Cr->blue=sqrt(QuantumScale*Ar->blue*Ar->blue+QuantumScale*
+            Ai->blue*Ai->blue);
           Ci->blue=atan2(Ai->blue,Ar->blue)/(2.0*MagickPI)+0.5;
           if (images->matte != MagickFalse)
             {
-              Cr->opacity=sqrt(Ar->opacity*Ar->opacity+Ai->opacity*Ai->opacity);
-              Ci->opacity=atan2(Ai->opacity,Ar->opacity)/(2.0*MagickPI)+0.5;
+              Cr->opacity=sqrt(QuantumScale*Ar->opacity*Ar->opacity+
+                QuantumScale*Ai->opacity*Ai->opacity);
+              Ci->opacity=atan2((double) Ai->opacity,(double) Ar->opacity)/
+                (2.0*MagickPI)+0.5;
             }
           break;
         }
         case MultiplyComplexOperator:
         {
-          Cr->red=QuantumScale*(Ar->red*Br->red-Ai->red*Bi->red);
-          Ci->red=QuantumScale*(Ai->red*Br->red+Ar->red*Bi->red);
-          Cr->green=QuantumScale*(Ar->green*Br->green-Ai->green*Bi->green);
-          Ci->green=QuantumScale*(Ai->green*Br->green+Ar->green*Bi->green);
-          Cr->blue=QuantumScale*(Ar->blue*Br->blue-Ai->blue*Bi->blue);
-          Ci->blue=QuantumScale*(Ai->blue*Br->blue+Ar->blue*Bi->blue);
+          Cr->red=(QuantumScale*Ar->red*Br->red-(double)
+            Ai->red*Bi->red);
+          Ci->red=(QuantumScale*Ai->red*Br->red+(double)
+            Ar->red*Bi->red);
+          Cr->green=(QuantumScale*Ar->green*Br->green-(double)
+            Ai->green*Bi->green);
+          Ci->green=(QuantumScale*Ai->green*Br->green+(double)
+            Ar->green*Bi->green);
+          Cr->blue=(QuantumScale*Ar->blue*Br->blue-(double)
+            Ai->blue*Bi->blue);
+          Ci->blue=(QuantumScale*Ai->blue*Br->blue+(double)
+            Ar->blue*Bi->blue);
           if (images->matte != MagickFalse)
             {
-              Cr->opacity=QuantumScale*(Ar->opacity*Br->opacity-Ai->opacity*
-                Bi->opacity);
-              Ci->opacity=QuantumScale*(Ai->opacity*Br->opacity+Ar->opacity*
-                Bi->opacity);
+              Cr->opacity=(QuantumScale*Ar->opacity*Br->opacity-
+                QuantumScale*Ai->opacity*Bi->opacity);
+              Ci->opacity=(QuantumScale*Ai->opacity*Br->opacity+
+                QuantumScale*Ar->opacity*Bi->opacity);
             }
           break;
         }
@@ -376,7 +408,7 @@ MagickExport Image *ComplexImages(const Image *images,const ComplexOperator op,
           Ci->green=Ai->green-Bi->green;
           Cr->blue=Ar->blue-Br->blue;
           Ci->blue=Ai->blue-Bi->blue;
-          if (images->matte != MagickFalse)
+          if (Cr_image->matte != MagickFalse)
             {
               Cr->opacity=Ar->opacity-Br->opacity;
               Ci->opacity=Ai->opacity-Bi->opacity;
@@ -462,7 +494,7 @@ static MagickBooleanType RollFourier(const size_t width,const size_t height,
   MemoryInfo
     *source_info;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -508,7 +540,7 @@ static MagickBooleanType ForwardQuadrantSwap(const size_t width,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -538,7 +570,7 @@ static MagickBooleanType ForwardQuadrantSwap(const size_t width,
 static void CorrectPhaseLHS(const size_t width,const size_t height,
   double *fourier_pixels)
 {
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -571,13 +603,13 @@ static MagickBooleanType ForwardFourier(const FourierInfo *fourier_info,
     *magnitude_info,
     *phase_info;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -768,13 +800,13 @@ static MagickBooleanType ForwardFourierTransform(FourierInfo *fourier_info,
     *forward_info,
     *source_info;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -864,13 +896,13 @@ static MagickBooleanType ForwardFourierTransform(FourierInfo *fourier_info,
   fftw_destroy_plan(fftw_r2c_plan);
   source_info=(MemoryInfo *) RelinquishVirtualMemory(source_info);
   value=GetImageArtifact(image,"fourier:normalize");
-  if ((value == (const char *) NULL) || (LocaleCompare(value,"forward") == 0))
+  if (LocaleCompare(value,"forward") == 0)
     {
       double
         gamma;
 
       /*
-        Normalize Fourier transform.
+        Normalize forward transform.
       */
       i=0L;
       gamma=PerceptibleReciprocal((double) fourier_info->width*
@@ -1148,7 +1180,7 @@ MagickExport Image *ForwardFourierTransformImage(const Image *image,
 static MagickBooleanType InverseQuadrantSwap(const size_t width,
   const size_t height,const double *source,double *destination)
 {
-  register ssize_t
+  ssize_t
     x;
 
   ssize_t
@@ -1190,13 +1222,13 @@ static MagickBooleanType InverseFourier(FourierInfo *fourier_info,
     *magnitude_info,
     *phase_info;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1404,13 +1436,13 @@ static MagickBooleanType InverseFourierTransform(FourierInfo *fourier_info,
   MemoryInfo
     *source_info;
 
-  register IndexPacket
+  IndexPacket
     *indexes;
 
-  register PixelPacket
+  PixelPacket
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
