@@ -17,13 +17,13 @@
 %                                September 1996                               %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -1349,13 +1349,13 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   */
   for (y=0; y < image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *restrict q;
 
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
@@ -1428,7 +1428,7 @@ static Boolean SearchForFile(OSType creator_type,OSType file_type,FSSpec *file,
     serial_number;
 
   ssize_t
-    buffer_size = 16384;
+    buffer_size = MagickMinBufferExtent;
 
   serial_number.lowLongOfPSN=kCurrentProcess;
   serial_number.highLongOfPSN=0;
