@@ -17,7 +17,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,9 +33,36 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Segregate our memory requirements from any program that calls our API.  This
-%  should help reduce the risk of others changing our program state or causing
-%  memory corruption.
+%  We provide these memory allocators:
+%
+%    AcquireCriticalMemory(): allocate a small memory request with
+%      AcquireMagickMemory(), however, on fail throw a fatal exception and exit.
+%      Free the memory reserve with RelinquishMagickMemory().
+%    AcquireAlignedMemory(): allocate a small memory request that is aligned
+%      on a cache line.  On fail, return NULL for possible recovery.
+%      Free the memory reserve with RelinquishMagickMemory().
+%    AcquireMagickMemory()/ResizeMagickMemory(): allocate a small to medium
+%      memory request, typically with malloc()/realloc(). On fail, return NULL
+%      for possible recovery.  Free the memory reserve with
+%      RelinquishMagickMemory().
+%    AcquireQuantumMemory()/ResizeQuantumMemory(): allocate a small to medium
+%      memory request.  This is a secure memory allocator as it accepts two
+%      parameters, count and quantum, to ensure the request does not overflow.
+%      It also check to ensure the request does not exceed the maximum memory
+%      per the security policy.  Free the memory reserve with
+%      RelinquishMagickMemory().
+%    AcquireVirtualMemory(): allocate a large memory request either in heap,
+%      memory-mapped, or memory-mapped on disk depending on whether heap
+%      allocation fails or if the request exceeds the maximum memory policy.
+%      Free the memory reserve with RelinquishVirtualMemory().
+%    ResetMagickMemory(): fills the bytes of the memory area with a constant
+%      byte.
+%    
+%  In addition, we provide hooks for your own memory constructor/destructors.
+%  You can also utilize our internal custom allocator as follows: Segregate
+%  our memory requirements from any program that calls our API.  This should
+%  help reduce the risk of others changing our program state or causing memory
+%  corruption.
 %
 %  Our custom memory allocation manager implements a best-fit allocation policy
 %  using segregated free lists.  It uses a linear distribution of size classes
@@ -43,7 +70,7 @@
 %  sizes.  It is based on the paper, "Fast Memory Allocation using Lazy Fits."
 %  written by Yoo C. Chung.
 %
-%  By default, ANSI memory methods are called (e.g. malloc).  Use the
+%  By default, C's standard library is used (e.g. malloc);  use the
 %  custom memory allocator by defining MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT
 %  to allocate memory with private anonymous mapping rather than from the
 %  heap.
@@ -58,6 +85,7 @@
 #include "magick/blob-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/memory-private.h"
 #include "magick/policy.h"
@@ -65,7 +93,7 @@
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/string-private.h"
-#include "magick/utility.h"
+#include "magick/utility-private.h"
 
 /*
   Define declarations.
@@ -73,12 +101,10 @@
 #define BlockFooter(block,size) \
   ((size_t *) ((char *) (block)+(size)-2*sizeof(size_t)))
 #define BlockHeader(block)  ((size_t *) (block)-1)
-#define BlockSize  4096
 #define BlockThreshold  1024
 #define MaxBlockExponent  16
 #define MaxBlocks ((BlockThreshold/(4*sizeof(size_t)))+MaxBlockExponent+1)
 #define MaxSegments  1024
-#define MemoryGuard  ((0xdeadbeef << 31)+0xdeafdeed)
 #define NextBlock(block)  ((char *) (block)+SizeOfBlock(block))
 #define NextBlockInList(block)  (*(void **) (block))
 #define PreviousBlock(block)  ((char *) (block)-(*((size_t *) (block)-2)))
@@ -126,12 +152,18 @@ typedef struct _MagickMemoryMethods
 
   DestroyMemoryHandler
     destroy_memory_handler;
+
+  AcquireAlignedMemoryHandler
+    acquire_aligned_memory_handler;
+
+  RelinquishAlignedMemoryHandler
+    relinquish_aligned_memory_handler;
 } MagickMemoryMethods;
 
 struct _MemoryInfo
 {
   char
-    filename[MaxTextExtent];
+    filename[MagickPathExtent];
 
   VirtualMemoryType
     type;
@@ -170,14 +202,16 @@ static size_t
   virtual_anonymous_memory = 0;
 
 #if defined _MSC_VER
-static void* MSCMalloc(size_t size)
+static void *MSCMalloc(size_t size)
 {
-  return malloc(size);
+  return(malloc(size));
 }
-static void* MSCRealloc(void* ptr, size_t size)
+
+static void *MSCRealloc(void* ptr, size_t size)
 {
-  return realloc(ptr, size);
+  return(realloc(ptr,size));
 }
+
 static void MSCFree(void* ptr)
 {
   free(ptr);
@@ -190,14 +224,15 @@ static MagickMemoryMethods
 #if defined _MSC_VER
     (AcquireMemoryHandler) MSCMalloc,
     (ResizeMemoryHandler) MSCRealloc,
-    (DestroyMemoryHandler) MSCFree
+    (DestroyMemoryHandler) MSCFree,
 #else
     (AcquireMemoryHandler) malloc,
     (ResizeMemoryHandler) realloc,
-    (DestroyMemoryHandler) free
+    (DestroyMemoryHandler) free,
 #endif
+    (AcquireAlignedMemoryHandler) NULL,
+    (RelinquishAlignedMemoryHandler) NULL
   };
-
 #if defined(MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT)
 static MemoryPool
   memory_pool;
@@ -226,8 +261,8 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AcquireAlignedMemory() returns a pointer to a block of memory at least size
-%  bytes whose address is aligned on a cache line or page boundary.
+%  AcquireAlignedMemory() returns a pointer to a block of memory whose size is
+%  at least (count*quantum) bytes, and whose address is aligned on a cache line.
 %
 %  The format of the AcquireAlignedMemory method is:
 %
@@ -235,68 +270,112 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o count: the number of quantum elements to allocate.
+%    o count: the number of objects to allocate contiguously.
 %
-%    o quantum: the number of bytes in each quantum.
+%    o quantum: the size (in bytes) of each object.
 %
 */
-MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
+#if defined(MAGICKCORE_HAVE_ALIGNED_MALLOC)
+#define AcquireAlignedMemory_Actual AcquireAlignedMemory_STDC
+static inline void *AcquireAlignedMemory_STDC(const size_t size)
 {
-#define AlignedExtent(size,alignment) \
-  (((size)+((alignment)-1)) & ~((alignment)-1))
-#define AlignedPowerOf2(x)  ((((x) - 1) & (x)) == 0)
-
   size_t
-    alignment,
-    extent,
-    size;
+    extent = CACHE_ALIGNED(size);
 
+  if (extent < size)
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
+  return(aligned_alloc(CACHE_LINE_SIZE,extent));
+}
+#elif defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
+#define AcquireAlignedMemory_Actual AcquireAlignedMemory_POSIX
+static inline void *AcquireAlignedMemory_POSIX(const size_t size)
+{
   void
     *memory;
 
-  if (HeapOverflowSanityCheck(count,quantum) != MagickFalse)
-    return((void *) NULL);
-  memory=NULL;
-  size=count*quantum;
-  alignment=CACHE_LINE_SIZE;
-  extent=AlignedExtent(size,alignment);
-  if ((size == 0) || (extent < size))
-    return((void *) NULL);
-#if defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
-  if (posix_memalign(&memory,alignment,extent) != 0)
-    memory=NULL;
-#elif defined(MAGICKCORE_HAVE__ALIGNED_MALLOC)
-  memory=_aligned_malloc(extent,alignment);
-#else
-  {
-    void
-      *p;
-
-    if ((alignment == 0) || (alignment % sizeof(void *) != 0) ||
-        (AlignedPowerOf2(alignment/sizeof (void *)) == 0))
-      {
-        errno=EINVAL;
-        return((void *) NULL);
-      }
-    if (size > (SIZE_MAX-alignment-sizeof(void *)-1))
-      {
-        errno=ENOMEM;
-        return((void *) NULL);
-      }
-    extent=(size+alignment-1)+sizeof(void *);
-    if (extent > size)
-      {
-        p=malloc(extent);
-        if (p != NULL)
-          {
-            memory=(void *) AlignedExtent((size_t) p+sizeof(void *),alignment);
-            *((void **) memory-1)=p;
-          }
-      }
-  }
-#endif
+  if (posix_memalign(&memory,CACHE_LINE_SIZE,size))
+    return(NULL);
   return(memory);
 }
+#elif defined(MAGICKCORE_HAVE__ALIGNED_MALLOC)
+#define AcquireAlignedMemory_Actual AcquireAlignedMemory_WinAPI
+static inline void *AcquireAlignedMemory_WinAPI(const size_t size)
+{
+  return(_aligned_malloc(size,CACHE_LINE_SIZE));
+}
+#else
+#define ALIGNMENT_OVERHEAD \
+  (MAGICKCORE_MAX_ALIGNMENT_PADDING(CACHE_LINE_SIZE) + MAGICKCORE_SIZEOF_VOID_P)
+static inline void *reserve_space_for_actual_base_address(void *const p)
+{
+  return((void **) p+1);
+}
+
+static inline void **pointer_to_space_for_actual_base_address(void *const p)
+{
+  return((void **) p-1);
+}
+
+static inline void *actual_base_address(void *const p)
+{
+  return(*pointer_to_space_for_actual_base_address(p));
+}
+
+static inline void *align_to_cache(void *const p)
+{
+  return((void *) CACHE_ALIGNED((MagickAddressType) p));
+}
+
+static inline void *adjust(void *const p)
+{
+  return(align_to_cache(reserve_space_for_actual_base_address(p)));
+}
+
+#define AcquireAlignedMemory_Actual AcquireAlignedMemory_Generic
+static inline void *AcquireAlignedMemory_Generic(const size_t size)
+{
+  size_t
+    extent;
+
+  void
+    *memory,
+    *p;
+
+  #if SIZE_MAX < ALIGNMENT_OVERHEAD
+    #error "CACHE_LINE_SIZE is way too big."
+  #endif
+  extent=(size+ALIGNMENT_OVERHEAD);
+  if (extent <= size)
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
+  p=AcquireMagickMemory(extent);
+  if (p == NULL)
+    return(NULL);
+  memory=adjust(p);
+  *pointer_to_space_for_actual_base_address(memory)=p;
+  return(memory);
+}
+#endif
+
+MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
+{
+  size_t
+    size;
+
+  if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
+  if (memory_methods.acquire_aligned_memory_handler != (AcquireAlignedMemoryHandler) NULL)
+    return(memory_methods.acquire_aligned_memory_handler(size,CACHE_LINE_SIZE));
+  return(AcquireAlignedMemory_Actual(size));
+}
 
 #if defined(MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT)
 /*
@@ -325,7 +404,7 @@ MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
 
 static inline size_t AllocationPolicy(size_t size)
 {
-  register size_t
+  size_t
     blocksize;
 
   /*
@@ -353,7 +432,7 @@ static inline size_t AllocationPolicy(size_t size)
 
 static inline void InsertFreeBlock(void *block,const size_t i)
 {
-  register void
+  void
     *next,
     *previous;
 
@@ -380,7 +459,7 @@ static inline void InsertFreeBlock(void *block,const size_t i)
 
 static inline void RemoveFreeBlock(void *block,const size_t i)
 {
-  register void
+  void
     *next,
     *previous;
 
@@ -396,10 +475,10 @@ static inline void RemoveFreeBlock(void *block,const size_t i)
 
 static void *AcquireBlock(size_t size)
 {
-  register size_t
+  size_t
     i;
 
-  register void
+  void
     *block;
 
   /*
@@ -472,7 +551,7 @@ static void *AcquireBlock(size_t size)
 */
 MagickExport void *AcquireMagickMemory(const size_t size)
 {
-  register void
+  void
     *memory;
 
 #if !defined(MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT)
@@ -485,7 +564,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
       LockSemaphoreInfo(memory_semaphore);
       if (free_segments == (DataSegmentInfo *) NULL)
         {
-          register ssize_t
+          ssize_t
             i;
 
           assert(2*sizeof(size_t) > (size_t) (~SizeMask));
@@ -521,6 +600,48 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e C r i t i c a l M e m o r y                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireCriticalMemory() is just like AcquireMagickMemory(), throws a fatal
+%  exception if the memory cannot be acquired.
+%
+%  That is, AcquireCriticalMemory() returns a pointer to a block of memory that
+%  is at least size bytes, and that is suitably aligned for any use; however,
+%  if this is not possible, it throws an exception and terminates the program
+%  as unceremoniously as possible.
+%
+%  The format of the AcquireCriticalMemory method is:
+%
+%      void *AcquireCriticalMemory(const size_t size)
+%
+%  A description of each parameter follows:
+%
+%    o size: the size (in bytes) of the memory to allocate.
+%
+*/
+MagickExport void *AcquireCriticalMemory(const size_t size)
+{
+  void
+    *memory;
+
+  /*
+    Fail if memory request cannot be fulfilled.
+  */
+  memory=AcquireMagickMemory(size);
+  if (memory == (void *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  return(memory);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   A c q u i r e Q u a n t u m M e m o r y                                   %
 %                                                                             %
 %                                                                             %
@@ -536,20 +657,23 @@ MagickExport void *AcquireMagickMemory(const size_t size)
 %
 %  A description of each parameter follows:
 %
-%    o count: the number of quantum elements to allocate.
+%    o count: the number of objects to allocate contiguously.
 %
-%    o quantum: the number of bytes in each quantum.
+%    o quantum: the size (in bytes) of each object.
 %
 */
 MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
 {
   size_t
-    extent;
+    size;
 
-  if (HeapOverflowSanityCheck(count,quantum) != MagickFalse)
-    return((void *) NULL);
-  extent=count*quantum;
-  return(AcquireMagickMemory(extent));
+  if ((HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse) ||
+      (size > GetMaxMemoryRequest()))
+    {
+      errno=ENOMEM;
+      return(NULL);
+    }
+  return(AcquireMagickMemory(size));
 }
 
 /*
@@ -573,23 +697,11 @@ MagickExport void *AcquireQuantumMemory(const size_t count,const size_t quantum)
 %
 %  A description of each parameter follows:
 %
-%    o count: the number of quantum elements to allocate.
+%    o count: the number of objects to allocate contiguously.
 %
-%    o quantum: the number of bytes in each quantum.
+%    o quantum: the size (in bytes) of each object.
 %
 */
-
-static inline size_t StringToSizeType(const char *string,const double interval)
-{
-  double
-    value;
-
-  value=SiPrefixToDoubleInterval(string,interval);
-  if (value >= (double) MagickULLConstant(~0))
-    return((size_t) MagickULLConstant(~0));
-  return((size_t) value);
-}
-
 MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
   const size_t quantum)
 {
@@ -600,10 +712,13 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
     *memory_info;
 
   size_t
-    extent;
+    size;
 
-  if (HeapOverflowSanityCheck(count,quantum) != MagickFalse)
-    return((MemoryInfo *) NULL);
+  if (HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse)
+    {
+      errno=ENOMEM;
+      return((MemoryInfo *) NULL);
+    }
   if (virtual_anonymous_memory == 0)
     {
       virtual_anonymous_memory=1;
@@ -624,13 +739,11 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
   if (memory_info == (MemoryInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) memset(memory_info,0,sizeof(*memory_info));
-  extent=count*quantum;
-  memory_info->length=extent;
+  memory_info->length=size;
   memory_info->signature=MagickCoreSignature;
-  if ((virtual_anonymous_memory == 1) &&
-      ((count*quantum) <= GetMaxMemoryRequest()))
+  if ((virtual_anonymous_memory == 1) && (size <= GetMaxMemoryRequest()))
     {
-      memory_info->blob=AcquireAlignedMemory(1,extent);
+      memory_info->blob=AcquireAlignedMemory(1,size);
       if (memory_info->blob != NULL)
         memory_info->type=AlignedVirtualMemory;
     }
@@ -640,8 +753,8 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
         Acquire anonymous memory map.
       */
       memory_info->blob=NULL;
-      if ((count*quantum) <= GetMaxMemoryRequest())
-        memory_info->blob=MapBlob(-1,IOMode,0,extent);
+      if (size <= GetMaxMemoryRequest())
+        memory_info->blob=MapBlob(-1,IOMode,0,size);
       if (memory_info->blob != NULL)
         memory_info->type=MapVirtualMemory;
       else
@@ -658,11 +771,16 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
               MagickOffsetType
                 offset;
 
-              offset=(MagickOffsetType) lseek(file,extent-1,SEEK_SET);
-              if ((offset == (MagickOffsetType) (extent-1)) &&
+              offset=(MagickOffsetType) lseek(file,size-1,SEEK_SET);
+              if ((offset == (MagickOffsetType) (size-1)) &&
                   (write(file,"",1) == 1))
                 {
-                  memory_info->blob=MapBlob(file,IOMode,0,extent);
+#if !defined(MAGICKCORE_HAVE_POSIX_FALLOCATE)
+                  memory_info->blob=MapBlob(file,IOMode,0,size);
+#else
+                  if (posix_fallocate(file,0,(MagickOffsetType) size) == 0)
+                    memory_info->blob=MapBlob(file,IOMode,0,size);
+#endif
                   if (memory_info->blob != NULL)
                     memory_info->type=MapVirtualMemory;
                   else
@@ -678,7 +796,7 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
     }
   if (memory_info->blob == NULL)
     {
-      memory_info->blob=AcquireQuantumMemory(1,extent);
+      memory_info->blob=AcquireQuantumMemory(1,size);
       if (memory_info->blob != NULL)
         memory_info->type=UnalignedVirtualMemory;
     }
@@ -704,8 +822,8 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
 %
 %  The format of the CopyMagickMemory method is:
 %
-%      void *CopyMagickMemory(void *destination,const void *source,
-%        const size_t size)
+%      void *CopyMagickMemory(void *magick_restrict destination,
+%        const void *magick_restrict source,const size_t size)
 %
 %  A description of each parameter follows:
 %
@@ -716,13 +834,13 @@ MagickExport MemoryInfo *AcquireVirtualMemory(const size_t count,
 %    o size: the size of the memory in bytes to allocate.
 %
 */
-MagickExport void *CopyMagickMemory(void *destination,const void *source,
-  const size_t size)
+MagickExport void *CopyMagickMemory(void *magick_restrict destination,
+  const void *magick_restrict source,const size_t size)
 {
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register unsigned char
+  unsigned char
     *q;
 
   assert(destination != (void *) NULL);
@@ -767,7 +885,7 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 MagickExport void DestroyMagickMemory(void)
 {
 #if defined(MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT)
-  register ssize_t
+  ssize_t
     i;
 
   if (memory_semaphore == (SemaphoreInfo *) NULL)
@@ -819,10 +937,10 @@ static MagickBooleanType ExpandHeap(size_t size)
   MagickBooleanType
     mapped;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register void
+  void
     *block;
 
   size_t
@@ -904,8 +1022,7 @@ MagickExport void GetMagickMemoryMethods(
   *resize_memory_handler=memory_methods.resize_memory_handler;
   *destroy_memory_handler=memory_methods.destroy_memory_handler;
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -917,7 +1034,7 @@ MagickExport void GetMagickMemoryMethods(
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMaxMemoryRequest() returns the max_memory_request value.
+%  GetMaxMemoryRequest() returns the max memory request value.
 %
 %  The format of the GetMaxMemoryRequest method is:
 %
@@ -926,24 +1043,26 @@ MagickExport void GetMagickMemoryMethods(
 */
 MagickExport size_t GetMaxMemoryRequest(void)
 {
+#define MinMemoryRequest "16MiB"
+
   if (max_memory_request == 0)
     {
       char
         *value;
 
+      max_memory_request=(size_t) MagickULLConstant(~0);
       value=GetPolicyValue("system:max-memory-request");
       if (value != (char *) NULL)
         {
           /*
             The security policy sets a max memory request limit.
           */
-          max_memory_request=StringToSizeType(value,100.0);
+          max_memory_request=MagickMax(StringToSizeType(value,100.0),
+            StringToSizeType(MinMemoryRequest,100.0));
           value=DestroyString(value);
         }
-      else
-        max_memory_request=(size_t) MagickULLConstant(~0);
     }
-  return(max_memory_request);
+  return(MagickMin(max_memory_request,MAGICK_SSIZE_MAX));
 }
 
 /*
@@ -980,45 +1099,6 @@ MagickExport void *GetVirtualMemoryBlob(const MemoryInfo *memory_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   H e a p O v e r f l o w S a n i t y C h e c k                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  HeapOverflowSanityCheck() returns MagickTrue if the heap allocation request
-%  does not exceed the maximum limits of a size_t otherwise MagickFalse.
-%
-%  The format of the HeapOverflowSanityCheck method is:
-%
-%      MagickBooleanType HeapOverflowSanityCheck(const size_t count,
-%        const size_t quantum)
-%
-%  A description of each parameter follows:
-%
-%    o size: the size of the memory in bytes we require.
-%
-*/
-MagickExport MagickBooleanType HeapOverflowSanityCheck(const size_t count,
-  const size_t quantum)
-{
-  size_t
-    size;
-
-  size=count*quantum;
-  if ((count == 0) || (quantum != (size/count)))
-    {
-      errno=ENOMEM;
-      return(MagickTrue);
-    }
-  return(MagickFalse);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   R e l i n q u i s h A l i g n e d M e m o r y                             %
 %                                                                             %
 %                                                                             %
@@ -1041,12 +1121,17 @@ MagickExport void *RelinquishAlignedMemory(void *memory)
 {
   if (memory == (void *) NULL)
     return((void *) NULL);
-#if defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
+  if (memory_methods.relinquish_aligned_memory_handler != (RelinquishAlignedMemoryHandler) NULL)
+    {
+      memory_methods.relinquish_aligned_memory_handler(memory);
+      return(NULL);
+    }
+#if defined(MAGICKCORE_HAVE_ALIGNED_MALLOC) || defined(MAGICKCORE_HAVE_POSIX_MEMALIGN)
   free(memory);
 #elif defined(MAGICKCORE_HAVE__ALIGNED_MALLOC)
   _aligned_free(memory);
 #else
-  free(*((void **) memory-1));
+  RelinquishMagickMemory(actual_base_address(memory));
 #endif
   return(NULL);
 }
@@ -1211,6 +1296,52 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 %                                                                             %
 %                                                                             %
 %                                                                             %
++   R e s e t M a x M e m o r y R e q u e s t                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResetMaxMemoryRequest() resets the max_memory_request value.
+%
+%  The format of the ResetMaxMemoryRequest method is:
+%
+%      void ResetMaxMemoryRequest(void)
+%
+*/
+MagickPrivate void ResetMaxMemoryRequest(void)
+{
+  max_memory_request=0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   R e s e t V i r t u a l A n o n y m o u s M e m o r y                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResetVirtualAnonymousMemory() resets the virtual_anonymous_memory value.
+%
+%  The format of the ResetVirtualAnonymousMemory method is:
+%
+%      void ResetVirtualAnonymousMemory(void)
+%
+*/
+MagickPrivate void ResetVirtualAnonymousMemory(void)
+{
+  virtual_anonymous_memory=0;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e s i z e M a g i c k M e m o r y                                       %
 %                                                                             %
 %                                                                             %
@@ -1236,7 +1367,7 @@ MagickExport void *ResetMagickMemory(void *memory,int byte,const size_t size)
 #if defined(MAGICKCORE_ANONYMOUS_MEMORY_SUPPORT)
 static inline void *ResizeBlock(void *block,size_t size)
 {
-  register void
+  void
     *memory;
 
   if (block == (void *) NULL)
@@ -1255,7 +1386,7 @@ static inline void *ResizeBlock(void *block,size_t size)
 
 MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 {
-  register void
+  void
     *block;
 
   if (memory == (void *) NULL)
@@ -1308,24 +1439,63 @@ MagickExport void *ResizeMagickMemory(void *memory,const size_t size)
 %
 %    o memory: A pointer to a memory allocation.
 %
-%    o count: the number of quantum elements to allocate.
+%    o count: the number of objects to allocate contiguously.
 %
-%    o quantum: the number of bytes in each quantum.
+%    o quantum: the size (in bytes) of each object.
 %
 */
 MagickExport void *ResizeQuantumMemory(void *memory,const size_t count,
   const size_t quantum)
 {
   size_t
-    extent;
+    size;
 
-  if (HeapOverflowSanityCheck(count,quantum) != MagickFalse)
+  if ((HeapOverflowSanityCheckGetSize(count,quantum,&size) != MagickFalse) ||
+      (size > GetMaxMemoryRequest()))
     {
+      errno=ENOMEM;
       memory=RelinquishMagickMemory(memory);
-      return((void *) NULL);
+      return(NULL);
     }
-  extent=count*quantum;
-  return(ResizeMagickMemory(memory,extent));
+  if (size > GetMaxMemoryRequest())
+    return(NULL);
+  return(ResizeMagickMemory(memory,size));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t M a g i c k A l i g n e d M e m o r y M e t h o d s                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetMagickAlignedMemoryMethods() sets the methods to acquire and relinquish
+%  aligned memory.
+%
+%  The format of the SetMagickAlignedMemoryMethods() method is:
+%
+%      SetMagickAlignedMemoryMethods(
+%        AcquireAlignedMemoryHandler acquire_aligned_memory_handler,
+%        RelinquishAlignedMemoryHandler relinquish_aligned_memory_handler)
+%
+%  A description of each parameter follows:
+%
+%    o acquire_memory_handler: method to acquire aligned memory.
+%
+%    o relinquish_aligned_memory_handler: method to relinquish aligned memory.
+%
+*/
+MagickExport void SetMagickAlignedMemoryMethods(
+  AcquireAlignedMemoryHandler acquire_aligned_memory_handler,
+  RelinquishAlignedMemoryHandler relinquish_aligned_memory_handler)
+{
+  memory_methods.acquire_aligned_memory_handler=acquire_aligned_memory_handler;
+  memory_methods.relinquish_aligned_memory_handler=
+      relinquish_aligned_memory_handler;
 }
 
 /*
