@@ -15,13 +15,13 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://imagemagick.org/MagicksToolkit/script/license.php             %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -167,7 +167,7 @@ static LinkedListInfo *AcquireMimeCache(const char *filename,
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     const StringInfo
       *option;
@@ -240,13 +240,13 @@ MagickExport const MimeInfo *GetMimeInfo(const char *filename,
   EndianType
     endian;
 
-  register const MimeInfo
+  const MimeInfo
     *p;
 
-  register const unsigned char
+  const unsigned char
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -453,10 +453,10 @@ MagickExport const MimeInfo **GetMimeInfoList(const char *pattern,
   const MimeInfo
     **aliases;
 
-  register const MimeInfo
+  const MimeInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -529,7 +529,7 @@ extern "C" {
 
 static int MimeCompare(const void *x,const void *y)
 {
-  register char
+  char
     *p,
     *q;
 
@@ -548,10 +548,10 @@ MagickExport char **GetMimeList(const char *pattern,
   char
     **aliases;
 
-  register const MimeInfo
+  const MimeInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -714,7 +714,7 @@ MagickExport MagickBooleanType ListMimeInfo(FILE *file,ExceptionInfo *exception)
   const MimeInfo
     **mime_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -895,7 +895,7 @@ static MagickBooleanType LoadMimeCache(LinkedListInfo *cache,const char *xml,
         const char
           *p;
 
-        register unsigned char
+        unsigned char
           *q;
 
         token=AcquireString(attribute);
@@ -1074,7 +1074,7 @@ MagickExport MagickBooleanType MimeComponentGenesis(void)
 
 static void *DestroyMimeElement(void *mime_info)
 {
-  register MimeInfo
+  MimeInfo
     *p;
 
   p=(MimeInfo *) mime_info;
