@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
 
     https://imagemagick.org/script/license.php
@@ -52,11 +52,15 @@ extern "C" {
 #define R_OK 4
 #define W_OK 2
 #define RW_OK 6
-#define _SC_PAGESIZE 1
+#define _SC_PAGE_SIZE 1
 #define _SC_PHYS_PAGES 2
 #define _SC_OPEN_MAX 3
 #if !defined(SSIZE_MAX)
-#define SSIZE_MAX  0x7fffffffL
+# ifdef _WIN64
+#   define SSIZE_MAX LLONG_MAX
+# else
+#   define SSIZE_MAX LONG_MAX
+# endif
 #endif
 
 /*
@@ -101,15 +105,18 @@ extern "C" {
 #if !defined(fileno)
 #  define fileno  _fileno
 #endif
+#if !defined(freelocale)
+#  define freelocale  _free_locale
+#endif
 #if !defined(fseek) && !defined(__MINGW32__)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define fseek  _fseeki64
 #endif
 #endif
 #if !defined(fstat) && !defined(__BORLANDC__)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define fstat  _fstati64
@@ -121,7 +128,7 @@ extern "C" {
 #  define fsync  _commit
 #endif
 #if !defined(ftell) && !defined(__MINGW32__)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define ftell  _ftelli64
@@ -148,7 +155,7 @@ extern "C" {
 #if !defined(locale_t)
 #define locale_t _locale_t
 #endif
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #if !defined(lseek)
@@ -189,9 +196,6 @@ extern "C" {
 #  define mmap(address,length,protection,access,file,offset) \
   NTMapMemory(address,length,protection,access,file,offset)
 #endif
-#if !defined(msync)
-#  define msync(address,length,flags)  NTSyncMemory(address,length,flags)
-#endif
 #if !defined(munmap)
 #  define munmap(address,length)  NTUnmapMemory(address,length)
 #endif
@@ -216,9 +220,6 @@ extern "C" {
 #if !defined(readdir)
 #  define readdir(directory)  NTReadDirectory(directory)
 #endif
-#if !defined(seekdir)
-#  define seekdir(directory,offset)  NTSeekDirectory(directory,offset)
-#endif
 #if !defined(setmode)
 #  define setmode  _setmode
 #endif
@@ -229,7 +230,7 @@ extern "C" {
 #define strtod_l  _strtod_l
 #endif
 #if !defined(stat) && !defined(__BORLANDC__)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define stat  _stati64
@@ -246,19 +247,22 @@ extern "C" {
 #if !defined(sysconf)
 #  define sysconf(name)  NTSystemConfiguration(name)
 #endif
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define tell  _telli64
 #else
 #  define tell  _tell
 #endif
-#if !defined(telldir)
-#  define telldir(directory)  NTTellDirectory(directory)
-#endif
 #if !defined(tempnam)
 #  define tempnam  _tempnam_s
 #endif
+#if !defined(tolower_l)
+#define tolower_l  _tolower_l
+#endif
+#if !defined(toupper_l)
+#define toupper_l  _toupper_l
+#endif
 #if !defined(umask)
 #  define umask  _umask
 #endif
@@ -283,7 +287,7 @@ extern "C" {
 #  define write(fd,buffer,count)  _write(fd,buffer,(unsigned int) count)
 #endif
 #if !defined(wstat) && !defined(__BORLANDC__)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) && \
   !(defined(__MSVCRT_VERSION__) && (__MSVCRT_VERSION__ < 0x800))
 #  define wstat  _wstati64
@@ -320,6 +324,7 @@ extern MagickExport int
 extern MagickExport void
   NTErrorHandler(const ExceptionType,const char *,const char *),
   NTWarningHandler(const ExceptionType,const char *,const char *);
+
 #endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
