@@ -18,7 +18,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -95,8 +95,8 @@ MagickExport MagickBooleanType NTIsMagickConflict(const char *magick)
   assert(magick != (char *) NULL);
   if (strlen(magick) > 1)
     return(MagickFalse);
-  status=(GetLogicalDrives() & (1 << ((toupper((int) (*magick)))-'A'))) != 0 ?
-    MagickTrue : MagickFalse;
+  status=(GetLogicalDrives() & (1 <<
+    ((LocaleUppercase((int) (*magick)))-'A'))) != 0 ? MagickTrue : MagickFalse;
   return(status);
 }
 
@@ -254,7 +254,7 @@ MagickExport MagickBooleanType NTAcquireTypeCache(SplayTreeInfo *type_cache,
         family_extent=value_name;
         for (q=value_name; *q != '\0'; )
         {
-            GetNextToken(q,(const char **) &q,MaxTextExtent,token);
+            (void) GetNextToken(q,(const char **) &q,MaxTextExtent,token);
             if (*token == '\0')
               break;
 
