@@ -16,7 +16,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -169,7 +169,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
   MagickStatusType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -179,7 +179,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
   if (cache == (LinkedListInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   status=LoadPolicyCache(cache,ZeroConfigurationPolicy,"[zero-configuration]",0,
     exception);
 #else
@@ -209,7 +209,7 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
     PolicyInfo
       *policy_info;
 
-    register const PolicyMapInfo
+    const PolicyMapInfo
       *p;
 
     p=PolicyMap+i;
@@ -217,7 +217,8 @@ static LinkedListInfo *AcquirePolicyCache(const char *filename,
     if (policy_info == (PolicyInfo *) NULL)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),
-          ResourceLimitError,"MemoryAllocationFailed","`%s'",p->name);
+          ResourceLimitError,"MemoryAllocationFailed","`%s'",
+          p->name == (char *) NULL ? "" : p->name);
         continue;
       }
     (void) memset(policy_info,0,sizeof(*policy_info));
@@ -270,10 +271,10 @@ static PolicyInfo *GetPolicyInfo(const char *name,ExceptionInfo *exception)
   PolicyDomain
     domain;
 
-  register PolicyInfo
+  PolicyInfo
     *p;
 
-  register char
+  char
     *q;
 
   assert(exception != (ExceptionInfo *) NULL);
@@ -364,10 +365,10 @@ MagickExport const PolicyInfo **GetPolicyInfoList(const char *pattern,
   const PolicyInfo
     **policies;
 
-  register const PolicyInfo
+  const PolicyInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -430,16 +431,39 @@ MagickExport const PolicyInfo **GetPolicyInfoList(const char *pattern,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport char **GetPolicyList(const char *pattern,
-  size_t *number_policies,ExceptionInfo *exception)
+
+static char *AcquirePolicyString(const char *source,const size_t pad)
+{
+  char
+    *destination;
+
+  size_t
+    length;
+
+  length=0;
+  if (source != (char *) NULL)
+    length+=strlen(source);
+  destination=(char *) NULL;
+  if (~length >= pad) 
+    destination=(char *) AcquireQuantumMemory(length+pad,sizeof(*destination));
+  if (destination == (char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
+  if (source != (char *) NULL)
+    (void) memcpy(destination,source,length*sizeof(*destination));
+  destination[length]='\0';
+  return(destination);
+}
+
+MagickExport char **GetPolicyList(const char *pattern,size_t *number_policies,
+  ExceptionInfo *exception)
 {
   char
     **policies;
 
-  register const PolicyInfo
+  const PolicyInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -466,7 +490,7 @@ MagickExport char **GetPolicyList(const char *pattern,
   {
     if ((p->stealth == MagickFalse) &&
         (GlobExpression(p->name,pattern,MagickFalse) != MagickFalse))
-      policies[i++]=ConstantString(p->name);
+      policies[i++]=AcquirePolicyString(p->name,1);
     p=(const PolicyInfo *) GetNextValueInLinkedList(policy_cache);
   }
   UnlockSemaphoreInfo(policy_semaphore);
@@ -518,7 +542,7 @@ MagickExport char *GetPolicyValue(const char *name)
   value=policy_info->value;
   if ((value == (const char *) NULL) || (*value == '\0'))
     return((char *) NULL);
-  return(ConstantString(value));
+  return(AcquirePolicyString(value,1));
 }
 
 /*
@@ -598,7 +622,7 @@ MagickExport MagickBooleanType IsRightsAuthorized(const PolicyDomain domain,
   MagickBooleanType
     authorized;
 
-  register PolicyInfo
+  PolicyInfo
     *p;
 
   (void) LogMagickEvent(PolicyEvent,GetMagickModule(),
@@ -669,7 +693,7 @@ MagickExport MagickBooleanType ListPolicyInfo(FILE *file,
   const PolicyInfo
     **policy_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -762,7 +786,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
   const char *filename,const size_t depth,ExceptionInfo *exception)
 {
   char
-    keyword[MaxTextExtent],
+    keyword[MagickPathExtent],
     *token;
 
   const char
@@ -786,24 +810,24 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
     return(MagickFalse);
   status=MagickTrue;
   policy_info=(PolicyInfo *) NULL;
-  token=AcquireString(xml);
-  extent=strlen(token)+MaxTextExtent;
+  token=AcquirePolicyString(xml,MagickPathExtent);
+  extent=strlen(token)+MagickPathExtent;
   for (q=(const char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
-    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    (void) CopyMagickString(keyword,token,MagickPathExtent);
     if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
         /*
           Docdomain element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -812,7 +836,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -822,11 +846,11 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
         */
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
-          (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) CopyMagickString(keyword,token,MagickPathExtent);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > MagickMaxRecursionDepth)
@@ -835,17 +859,17 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
               else
                 {
                   char
-                    path[MaxTextExtent],
+                    path[MagickPathExtent],
                     *xml;
 
                   GetPathComponent(filename,HeadPath,path);
                   if (*path != '\0')
                     (void) ConcatenateMagickString(path,DirectorySeparator,
-                      MaxTextExtent);
+                      MagickPathExtent);
                   if (*token == *DirectorySeparator)
-                    (void) CopyMagickString(path,token,MaxTextExtent);
+                    (void) CopyMagickString(path,token,MagickPathExtent);
                   else
-                    (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                    (void) ConcatenateMagickString(path,token,MagickPathExtent);
                   xml=FileToXML(path,~0UL);
                   if (xml != (char *) NULL)
                     {
@@ -867,7 +891,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
         if (policy_info == (PolicyInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
         (void) memset(policy_info,0,sizeof(*policy_info));
-        policy_info->path=ConstantString(filename);
+        policy_info->path=AcquirePolicyString(filename,1);
         policy_info->exempt=MagickFalse;
         policy_info->signature=MagickCoreSignature;
         continue;
@@ -885,11 +909,11 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
         policy_info=(PolicyInfo *) NULL;
         continue;
       }
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetNextToken(q,&q,extent,token);
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'D':
@@ -908,7 +932,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"name") == 0)
           {
-            policy_info->name=ConstantString(token);
+            policy_info->name=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -918,7 +942,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"pattern") == 0)
           {
-            policy_info->pattern=ConstantString(token);
+            policy_info->pattern=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -949,7 +973,7 @@ static MagickBooleanType LoadPolicyCache(LinkedListInfo *cache,const char *xml,
       {
         if (LocaleCompare((char *) keyword,"value") == 0)
           {
-            policy_info->value=ConstantString(token);
+            policy_info->value=AcquirePolicyString(token,1);
             break;
           }
         break;
@@ -1008,7 +1032,7 @@ MagickExport MagickBooleanType PolicyComponentGenesis(void)
 
 static void *DestroyPolicyElement(void *policy_info)
 {
-  register PolicyInfo
+  PolicyInfo
     *p;
 
   p=(PolicyInfo *) policy_info;
