@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
   
     https://imagemagick.org/script/license.php
@@ -27,7 +27,7 @@ extern "C" {
 static inline RandomInfo **DestroyRandomInfoThreadSet(
   RandomInfo **random_info)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(random_info != (RandomInfo **) NULL);
@@ -39,7 +39,7 @@ static inline RandomInfo **DestroyRandomInfoThreadSet(
 
 static inline RandomInfo **AcquireRandomInfoThreadSet(void)
 {
-  register ssize_t
+  ssize_t
     i;
 
   RandomInfo
