@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -110,7 +110,7 @@ MagickExport MagickBooleanType DefineImageRegistry(const RegistryType type,
     key[MaxTextExtent],
     value[MaxTextExtent];
 
-  register char
+  char
     *p;
 
   assert(option != (const char *) NULL);
@@ -422,7 +422,7 @@ MagickExport void ResetImageRegistryIterator(void)
 
 static void *DestroyRegistryNode(void *registry_info)
 {
-  register RegistryInfo
+  RegistryInfo
     *p;
 
   p=(RegistryInfo *) registry_info;
