@@ -18,7 +18,7 @@
 %                                August 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -206,7 +206,7 @@ struct _ResampleFilter
 MagickExport ResampleFilter *AcquireResampleFilter(const Image *image,
   ExceptionInfo *exception)
 {
-  register ResampleFilter
+  ResampleFilter
     *resample_filter;
 
   assert(image != (Image *) NULL);
@@ -216,7 +216,7 @@ MagickExport ResampleFilter *AcquireResampleFilter(const Image *image,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
 
-  resample_filter=(ResampleFilter *) AcquireMagickMemory(
+  resample_filter=(ResampleFilter *) AcquireQuantumMemory(1,
     sizeof(*resample_filter));
   if (resample_filter == (ResampleFilter *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
@@ -327,9 +327,9 @@ MagickExport MagickBooleanType ResamplePixelColor(
   double u1;
   double U,V,Q,DQ,DDQ;
   double divisor_c,divisor_m;
-  register double weight;
-  register const PixelPacket *pixels;
-  register const IndexPacket *indexes;
+  double weight;
+  const PixelPacket *pixels;
+  const IndexPacket *indexes;
   assert(resample_filter != (ResampleFilter *) NULL);
   assert(resample_filter->signature == MagickCoreSignature);
 
@@ -1209,7 +1209,7 @@ MagickExport void ScaleResampleFilter(ResampleFilter *resample_filter,
   }
 
   /* Scale ellipse formula to directly index the Filter Lookup Table */
-  { register double scale;
+  { double scale;
 #if FILTER_LUT
     /* scale so that F = WLUT_WIDTH; -- hardcoded */
     scale = (double)WLUT_WIDTH/F;
@@ -1295,7 +1295,7 @@ MagickExport void SetResampleFilter(ResampleFilter *resample_filter,
 
 #if FILTER_LUT
   /* Fill the LUT with the weights from the selected filter function */
-  { register int
+  { int
        Q;
     double
        r_scale;
@@ -1350,7 +1350,7 @@ MagickExport void SetResampleFilter(ResampleFilter *resample_filter,
     if (IsMagickTrue(GetImageArtifact(resample_filter->image,
              "resample:verbose")) )
       {
-        register int
+        int
           Q;
         double
           r_scale;
