@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -69,7 +69,9 @@
 /*
   Define declarations.
 */
-#define MagickPathTemplate "XXXXXXXXXXXX"
+#define MagickPathTemplate "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"  /* min 6 X's */
+#define NumberOfResourceTypes  \
+  (sizeof(resource_semaphore)/sizeof(*resource_semaphore))
 
 /*
   Typedef declarations.
@@ -112,32 +114,45 @@ static RandomInfo
 static ResourceInfo
   resource_info =
   {
-    MagickULLConstant(0),              /* initial width */
-    MagickULLConstant(0),              /* initial height */
-    MagickULLConstant(0),              /* initial list length */
-    MagickULLConstant(0),              /* initial area */
-    MagickULLConstant(0),              /* initial memory */
-    MagickULLConstant(0),              /* initial map */
-    MagickULLConstant(0),              /* initial disk */
-    MagickULLConstant(0),              /* initial file */
-    MagickULLConstant(0),              /* initial thread */
-    MagickULLConstant(0),              /* initial throttle */
-    MagickULLConstant(0),              /* initial time */
-    (INT_MAX/(5*sizeof(Quantum))),     /* width limit */
-    (INT_MAX/(5*sizeof(Quantum))),     /* height limit */
-    MagickResourceInfinity,            /* list length limit */
-    MagickULLConstant(3072)*1024*1024, /* area limit */
-    MagickULLConstant(1536)*1024*1024, /* memory limit */
-    MagickULLConstant(3072)*1024*1024, /* map limit */
-    MagickResourceInfinity,            /* disk limit */
-    MagickULLConstant(768),            /* file limit */
-    MagickULLConstant(1),              /* thread limit */
-    MagickULLConstant(0),              /* throttle limit */
-    MagickResourceInfinity             /* time limit */
+    MagickULLConstant(0),                 /* initial width */
+    MagickULLConstant(0),                 /* initial height */
+    MagickULLConstant(0),                 /* initial list length */
+    MagickULLConstant(0),                 /* initial area */
+    MagickULLConstant(0),                 /* initial memory */
+    MagickULLConstant(0),                 /* initial map */
+    MagickULLConstant(0),                 /* initial disk */
+    MagickULLConstant(0),                 /* initial file */
+    MagickULLConstant(0),                 /* initial thread */
+    MagickULLConstant(0),                 /* initial throttle */
+    MagickULLConstant(0),                 /* initial time */
+    MAGICK_SSIZE_MAX/sizeof(PixelPacket), /* width limit */
+    MAGICK_SSIZE_MAX/sizeof(PixelPacket), /* height limit */
+    MagickResourceInfinity,               /* list length limit */
+    MagickULLConstant(3072)*1024*1024,    /* area limit */
+    MagickULLConstant(1536)*1024*1024,    /* memory limit */
+    MagickULLConstant(3072)*1024*1024,    /* map limit */
+    MagickResourceInfinity,               /* disk limit */
+    MagickULLConstant(768),               /* file limit */
+    MagickULLConstant(1),                 /* thread limit */
+    MagickULLConstant(0),                 /* throttle limit */
+    MagickResourceInfinity                /* time limit */
   };
 
 static SemaphoreInfo
-  *resource_semaphore = (SemaphoreInfo *) NULL;
+  *resource_semaphore[] = {
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL,
+     (SemaphoreInfo *) NULL
+  };
 
 static SplayTreeInfo
   *temporary_resources = (SplayTreeInfo *) NULL;
@@ -192,9 +207,21 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     return(MagickFalse);
   status=MagickFalse;
   logging=IsEventLogging();
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      if (resource_semaphore[type] == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&resource_semaphore[type]);
+      LockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   switch (type)
   {
     case AreaResource:
@@ -214,7 +241,8 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case DiskResource:
     {
       limit=resource_info.disk_limit;
-      if ((resource_info.disk+request) > resource_info.disk)
+      if (((MagickSizeType) resource_info.disk+request) >
+          (MagickSizeType) resource_info.disk)
         {
           resource_info.disk+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -235,7 +263,8 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case FileResource:
     {
       limit=resource_info.file_limit;
-      if ((resource_info.file+request) > resource_info.file)
+      if (((MagickSizeType) resource_info.file+request) >
+          (MagickSizeType) resource_info.file)
         {
           resource_info.file+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -282,7 +311,8 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case MapResource:
     {
       limit=resource_info.map_limit;
-      if ((resource_info.map+request) > resource_info.map)
+      if (((MagickSizeType) resource_info.map+request) >
+          (MagickSizeType) resource_info.map)
         {
           resource_info.map+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -303,7 +333,8 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case MemoryResource:
     {
       limit=resource_info.memory_limit;
-      if ((resource_info.memory+request) > resource_info.memory)
+      if (((MagickSizeType) resource_info.memory+request) >
+          (MagickSizeType) resource_info.memory)
         {
           resource_info.memory+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -354,11 +385,12 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case TimeResource:
     {
       limit=resource_info.time_limit;
-      if ((resource_info.time+request) > resource_info.time)
+      if (((MagickSizeType) resource_info.time+request) >
+          (MagickSizeType) resource_info.time)
         {
           resource_info.time+=request;
           if ((limit == MagickResourceInfinity) ||
-              (resource_info.time < (MagickOffsetType) limit))
+              ((MagickSizeType) resource_info.time < limit))
             status=MagickTrue;
           else
             resource_info.time-=request;
@@ -389,7 +421,19 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     default:
       break;
   }
-  UnlockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      UnlockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   if (logging != MagickFalse)
     {
       (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
@@ -485,8 +529,7 @@ MagickExport MagickBooleanType GetPathTemplate(char *path)
   struct stat
     attributes;
 
-  (void) FormatLocaleString(path,MaxTextExtent,"magick-%.20g"
-    MagickPathTemplate,(double) getpid());
+  (void) FormatLocaleString(path,MaxTextExtent,"magick-" MagickPathTemplate);
   exception=AcquireExceptionInfo();
   directory=(char *) GetImageRegistry(StringRegistryType,"temporary-path",
     exception);
@@ -531,16 +574,15 @@ MagickExport MagickBooleanType GetPathTemplate(char *path)
       return(MagickFalse);
     }
   if (directory[strlen(directory)-1] == *DirectorySeparator)
-    (void) FormatLocaleString(path,MaxTextExtent,"%smagick-%.20g"
-      MagickPathTemplate,directory,(double) getpid());
+    (void) FormatLocaleString(path,MaxTextExtent,"%smagick-" MagickPathTemplate,
+      directory);
   else
-    (void) FormatLocaleString(path,MaxTextExtent,
-      "%s%smagick-%.20g" MagickPathTemplate,directory,DirectorySeparator,
-      (double) getpid());
+    (void) FormatLocaleString(path,MaxTextExtent,"%s%smagick-"
+      MagickPathTemplate,directory,DirectorySeparator);
   directory=DestroyString(directory);
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   {
-    register char
+    char
       *p;
 
     /*
@@ -568,10 +610,10 @@ MagickExport int AcquireUniqueFileResource(char *path)
     c,
     file;
 
-  register char
+  char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   static const char
@@ -588,21 +630,24 @@ MagickExport int AcquireUniqueFileResource(char *path)
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"...");
   if (random_info == (RandomInfo *) NULL)
     {
-      LockSemaphoreInfo(resource_semaphore);
+      if (resource_semaphore[FileResource] == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&resource_semaphore[FileResource]);
+      LockSemaphoreInfo(resource_semaphore[FileResource]);
       if (random_info == (RandomInfo *) NULL)
         random_info=AcquireRandomInfo();
-      UnlockSemaphoreInfo(resource_semaphore);
+      UnlockSemaphoreInfo(resource_semaphore[FileResource]);
     }
   file=(-1);
   for (i=0; i < (ssize_t) TMP_MAX; i++)
   {
-    register ssize_t
+    ssize_t
       j;
+
     /*
       Get temporary pathname.
     */
     (void) GetPathTemplate(path);
-    key=GetRandomKey(random_info,6);
+    key=GetRandomKey(random_info,strlen(MagickPathTemplate)-6);
     p=path+strlen(path)-strlen(MagickPathTemplate);
     datum=GetStringInfoDatum(key);
     for (j=0; j < (ssize_t) GetStringInfoLength(key); j++)
@@ -641,13 +686,13 @@ MagickExport int AcquireUniqueFileResource(char *path)
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"Acquire %s",path);
   if (file == -1)
     return(file);
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  if (resource_semaphore[FileResource] == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&resource_semaphore[FileResource]);
+  LockSemaphoreInfo(resource_semaphore[FileResource]);
   if (temporary_resources == (SplayTreeInfo *) NULL)
     temporary_resources=NewSplayTree(CompareSplayTreeString,
       DestroyTemporaryResources,(void *(*)(void *)) NULL);
-  UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[FileResource]);
   (void) AddValueToSplayTree(temporary_resources,ConstantString(path),
     (const void *) NULL);
   return(file);
@@ -681,7 +726,21 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
     resource;
 
   resource=0;
-  LockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      if (resource_semaphore[type] == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&resource_semaphore[type]);
+      LockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   switch (type)
   {
     case AreaResource:
@@ -742,7 +801,19 @@ MagickExport MagickSizeType GetMagickResource(const ResourceType type)
     default:
       break;
   }
-  UnlockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      UnlockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   return(resource);
 }
 
@@ -774,9 +845,9 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
     resource;
 
   resource=0;
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  if (resource_semaphore[type] == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&resource_semaphore[type]);
+  LockSemaphoreInfo(resource_semaphore[type]);
   switch (type)
   {
     case AreaResource:
@@ -837,7 +908,7 @@ MagickExport MagickSizeType GetMagickResourceLimit(const ResourceType type)
     default:
       break;
   }
-  UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[type]);
   return(resource);
 }
 
@@ -877,7 +948,7 @@ static ssize_t FormatPixelSize(const MagickSizeType size,
     bytes,
     length;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -936,14 +1007,16 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
 
   if (file == (const FILE *) NULL)
     file=stdout;
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  if (resource_semaphore[FileResource] == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&resource_semaphore[FileResource]);
+  LockSemaphoreInfo(resource_semaphore[FileResource]);
   (void) FormatPixelSize(resource_info.width_limit,MagickFalse,width_limit);
   (void) FormatPixelSize(resource_info.height_limit,MagickFalse,height_limit);
-  (void) FormatPixelSize(resource_info.list_length_limit,MagickFalse,
-    list_length_limit);
   (void) FormatPixelSize(resource_info.area_limit,MagickFalse,area_limit);
+  (void) CopyMagickString(list_length_limit,"unlimited",MaxTextExtent);
+  if (resource_info.list_length_limit != MagickResourceInfinity)
+    (void) FormatMagickSize(resource_info.list_length_limit,MagickTrue,
+      list_length_limit);
   (void) FormatMagickSize(resource_info.memory_limit,MagickTrue,memory_limit);
   (void) FormatMagickSize(resource_info.map_limit,MagickTrue,map_limit);
   (void) CopyMagickString(disk_limit,"unlimited",MaxTextExtent);
@@ -969,7 +1042,7 @@ MagickExport MagickBooleanType ListMagickResourceInfo(FILE *file,
     ((MagickOffsetType) resource_info.throttle_limit));
   (void) FormatLocaleFile(file,"  Time: %s\n",time_limit);
   (void) fflush(file);
-  UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[FileResource]);
   return(MagickTrue);
 }
 
@@ -1012,9 +1085,21 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
   logging=IsEventLogging();
   if (logging != MagickFalse)
     (void) FormatMagickSize(size,MagickFalse,resource_request);
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      if (resource_semaphore[type] == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&resource_semaphore[type]);
+      LockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   switch (type)
   {
     case AreaResource:
@@ -1156,7 +1241,19 @@ MagickExport void RelinquishMagickResource(const ResourceType type,
     default:
       break;
   }
-  UnlockSemaphoreInfo(resource_semaphore);
+  switch (type)
+  {
+    case DiskResource:
+    case FileResource:
+    case MapResource:
+    case MemoryResource:
+    case TimeResource:
+    {
+      UnlockSemaphoreInfo(resource_semaphore[type]);
+      break;
+    }
+    default: ;
+  }
   if (logging != MagickFalse)
     {
       (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"%s: %s/%s/%s",
@@ -1198,12 +1295,12 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
   assert(path != (const char *) NULL);
   status=MagickFalse;
   (void) LogMagickEvent(ResourceEvent,GetMagickModule(),"Relinquish %s",path);
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  if (resource_semaphore[FileResource] == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&resource_semaphore[FileResource]);
+  LockSemaphoreInfo(resource_semaphore[FileResource]);
   if (temporary_resources != (SplayTreeInfo *) NULL)
     status=DeleteNodeFromSplayTree(temporary_resources, (const void *) path);
-  UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[FileResource]);
   (void) CopyMagickString(cache_path,path,MaxTextExtent);
     AppendImageFormat("cache",cache_path);
   if (access_utf8(cache_path,F_OK) == 0)
@@ -1231,19 +1328,6 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 %      MagickBooleanType ResourceComponentGenesis(void)
 %
 */
-
-static inline MagickSizeType StringToSizeType(const char *string,
-  const double interval)
-{
-  double
-    value;
-
-  value=SiPrefixToDoubleInterval(string,interval);
-  if (value >= (double) MagickULLConstant(~0))
-    return(MagickULLConstant(~0));
-  return((MagickSizeType) value);
-}
-
 MagickExport MagickBooleanType ResourceComponentGenesis(void)
 {
   char
@@ -1252,6 +1336,9 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
   MagickSizeType
     memory;
 
+  ssize_t
+    i;
+
   ssize_t
     files,
     pages,
@@ -1260,8 +1347,9 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
   /*
     Set Magick resource limits.
   */
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    resource_semaphore=AllocateSemaphoreInfo();
+  for (i=0; i < (ssize_t) NumberOfResourceTypes; i++)
+    if (resource_semaphore[i] == (SemaphoreInfo *) NULL)
+      ActivateSemaphoreInfo(&resource_semaphore[i]);
   (void) SetMagickResourceLimit(WidthResource,resource_info.width_limit);
   limit=GetEnvironmentValue("MAGICK_WIDTH_LIMIT");
   if (limit != (char *) NULL)
@@ -1282,6 +1370,9 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
   pages=(-1);
 #if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
   pages=(ssize_t) sysconf(_SC_PHYS_PAGES);
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+  pages=pages/2;
+#endif
 #endif
   memory=(MagickSizeType) pages*pagesize;
   if ((pagesize <= 0) || (pages <= 0))
@@ -1400,15 +1491,20 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
 */
 MagickExport void ResourceComponentTerminus(void)
 {
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  ssize_t
+    i;
+
+  for (i=0; i < (ssize_t) NumberOfResourceTypes; i++)
+    if (resource_semaphore[i] == (SemaphoreInfo *) NULL)
+      ActivateSemaphoreInfo(&resource_semaphore[i]);
+  LockSemaphoreInfo(resource_semaphore[FileResource]);
   if (temporary_resources != (SplayTreeInfo *) NULL)
     temporary_resources=DestroySplayTree(temporary_resources);
   if (random_info != (RandomInfo *) NULL)
     random_info=DestroyRandomInfo(random_info);
-  UnlockSemaphoreInfo(resource_semaphore);
-  DestroySemaphoreInfo(&resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[FileResource]);
+  for (i=0; i < (ssize_t) NumberOfResourceTypes; i++)
+    DestroySemaphoreInfo(&resource_semaphore[i]);
 }
 
 /*
@@ -1447,9 +1543,9 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
     status;
 
   status=MagickTrue;
-  if (resource_semaphore == (SemaphoreInfo *) NULL)
-    ActivateSemaphoreInfo(&resource_semaphore);
-  LockSemaphoreInfo(resource_semaphore);
+  if (resource_semaphore[type] == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&resource_semaphore[type]);
+  LockSemaphoreInfo(resource_semaphore[type]);
   value=(char *) NULL;
   switch (type)
   {
@@ -1488,6 +1584,8 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       else
         resource_info.height_limit=MagickMin(limit,StringToSizeType(value,
           100.0));
+      resource_info.height_limit=MagickMin(resource_info.height_limit,
+        (MagickSizeType) MAGICK_SSIZE_MAX);
       break;
     }
     case ListLengthResource:
@@ -1562,6 +1660,8 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       else
         resource_info.width_limit=MagickMin(limit,StringToSizeType(value,
           100.0));
+      resource_info.width_limit=MagickMin(resource_info.width_limit,
+        (MagickSizeType) MAGICK_SSIZE_MAX);
       break;
     }
     default:
@@ -1572,6 +1672,6 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
   }
   if (value != (char *) NULL)
     value=DestroyString(value);
-  UnlockSemaphoreInfo(resource_semaphore);
+  UnlockSemaphoreInfo(resource_semaphore[type]);
   return(status);
 }
