@@ -17,13 +17,13 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -93,6 +93,7 @@
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/quantize.h"
@@ -195,6 +196,7 @@ static ssize_t
   DefineRegion(const short *,ExtentPacket *);
 
 static void
+  FreeNodes(IntervalTree *),
   InitializeHistogram(const Image *,ssize_t **,ExceptionInfo *),
   ScaleSpace(const ssize_t *,const MagickRealType,MagickRealType *),
   ZeroCrossHistogram(MagickRealType *,const MagickRealType,short *);
@@ -244,6 +246,21 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   const MagickRealType weighting_exponent,const MagickBooleanType verbose)
 {
 #define SegmentImageTag  "Segment/Image"
+#define ThrowClassifyException(severity,tag,label) \
+{\
+  for (cluster=head; cluster != (Cluster *) NULL; cluster=next_cluster) \
+  { \
+    next_cluster=cluster->next; \
+    cluster=(Cluster *) RelinquishMagickMemory(cluster); \
+  } \
+  if (squares != (MagickRealType *) NULL) \
+    { \
+      squares-=255; \
+      free_squares=squares; \
+      free_squares=(MagickRealType *) RelinquishMagickMemory(free_squares); \
+    } \
+  ThrowBinaryException(severity,tag,label); \
+}
 
   CacheView
     *image_view;
@@ -271,10 +288,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   MagickStatusType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register MagickRealType
+  MagickRealType
     *squares;
 
   size_t
@@ -289,9 +306,11 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  squares=(MagickRealType *) NULL;
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
+  exception=(&image->exception);
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -305,26 +324,25 @@ static MagickBooleanType Classify(Image *image,short **extrema,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *) AcquireMagickMemory(
+            cluster->next=(Cluster *) AcquireQuantumMemory(1,
               sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
           {
-            cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
+            cluster=(Cluster *) AcquireQuantumMemory(1,sizeof(*cluster));
             head=cluster;
           }
         if (cluster == (Cluster *) NULL)
-          ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+          ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
             image->filename);
         /*
           Initialize a new class.
         */
-        cluster->count=0;
+        (void) memset(cluster,0,sizeof(*cluster));
         cluster->red=red;
         cluster->green=green;
         cluster->blue=blue;
-        cluster->next=(Cluster *) NULL;
       }
     }
   }
@@ -333,18 +351,17 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       /*
         No classes were identified-- create one.
       */
-      cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
+      cluster=(Cluster *) AcquireQuantumMemory(1,sizeof(*cluster));
       if (cluster == (Cluster *) NULL)
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
       /*
         Initialize a new class.
       */
-      cluster->count=0;
+      (void) memset(cluster,0,sizeof(*cluster));
       cluster->red=red;
       cluster->green=green;
       cluster->blue=blue;
-      cluster->next=(Cluster *) NULL;
       head=cluster;
     }
   /*
@@ -353,14 +370,13 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   status=MagickTrue;
   count=0;
   progress=0;
-  exception=(&image->exception);
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *p;
 
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
@@ -368,28 +384,27 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
+      MagickPixelPacket
+        pixel;
+
+      pixel.red=(double) ScaleQuantumToChar(p->red);
+      pixel.green=(double) ScaleQuantumToChar(p->green);
+      pixel.blue=(double) ScaleQuantumToChar(p->blue);
       for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
-        if (((ssize_t) ScaleQuantumToChar(GetPixelRed(p)) >=
-             (cluster->red.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelRed(p)) <=
-             (cluster->red.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelGreen(p)) >=
-             (cluster->green.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelGreen(p)) <=
-             (cluster->green.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelBlue(p)) >=
-             (cluster->blue.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelBlue(p)) <=
-             (cluster->blue.right+SafeMargin)))
+        if ((pixel.red >= (double) (cluster->red.left-SafeMargin)) &&
+            (pixel.red <= (double) (cluster->red.right+SafeMargin)) &&
+            (pixel.green >= (double) (cluster->green.left-SafeMargin)) &&
+            (pixel.green <= (double) (cluster->green.right+SafeMargin)) &&
+            (pixel.blue >= (double) (cluster->blue.left-SafeMargin)) &&
+            (pixel.blue <= (double) (cluster->blue.right+SafeMargin)))
           {
             /*
               Count this pixel.
             */
             count++;
-            cluster->red.center+=(MagickRealType) ScaleQuantumToChar(GetPixelRed(p));
-            cluster->green.center+=(MagickRealType)
-              ScaleQuantumToChar(GetPixelGreen(p));
-            cluster->blue.center+=(MagickRealType) ScaleQuantumToChar(GetPixelBlue(p));
+            cluster->red.center+=pixel.red;
+            cluster->green.center+=pixel.green;
+            cluster->blue.center+=pixel.blue;
             cluster->count++;
             break;
           }
@@ -401,10 +416,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,
-          2*image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -497,13 +512,13 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       (void) FormatLocaleFile(stdout,"\n");
     }
   if (number_clusters > 256)
-    ThrowBinaryException(ImageError,"TooManyClusters",image->filename);
+    ThrowClassifyException(ImageError,"TooManyClusters",image->filename);
   /*
     Speed up distance calculations.
   */
   squares=(MagickRealType *) AcquireQuantumMemory(513UL,sizeof(*squares));
   if (squares == (MagickRealType *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   squares+=255;
   for (i=(-255); i <= 255; i++)
@@ -512,7 +527,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     Allocate image colormap.
   */
   if (AcquireImageColormap(image,number_clusters) == MagickFalse)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+    ThrowClassifyException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
   i=0;
   for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
@@ -531,24 +546,24 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   exception=(&image->exception);
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status) \
-    magick_threads(image,image,image->rows,1)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     Cluster
       *cluster;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -562,21 +577,21 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
+      MagickPixelPacket
+        pixel;
+
       SetPixelIndex(indexes+x,0);
+      pixel.red=(double) ScaleQuantumToChar(q->red);
+      pixel.green=(double) ScaleQuantumToChar(q->green);
+      pixel.blue=(double) ScaleQuantumToChar(q->blue);
       for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
       {
-        if (((ssize_t) ScaleQuantumToChar(q->red) >=
-             (cluster->red.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(q->red) <=
-             (cluster->red.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(q->green) >=
-             (cluster->green.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(q->green) <=
-             (cluster->green.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(q->blue) >=
-             (cluster->blue.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(q->blue) <=
-             (cluster->blue.right+SafeMargin)))
+        if ((pixel.red >= (double) (cluster->red.left-SafeMargin)) &&
+            (pixel.red <= (double) (cluster->red.right+SafeMargin)) &&
+            (pixel.green >= (double) (cluster->green.left-SafeMargin)) &&
+            (pixel.green <= (double) (cluster->green.right+SafeMargin)) &&
+            (pixel.blue >= (double) (cluster->blue.left-SafeMargin)) &&
+            (pixel.blue <= (double) (cluster->blue.right+SafeMargin)))
           {
             /*
               Classify this pixel.
@@ -594,7 +609,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
             ratio,
             sum;
 
-          register ssize_t
+          ssize_t
             j,
             k;
 
@@ -606,22 +621,18 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           {
             sum=0.0;
             p=image->colormap+j;
-            distance_squared=squares[(ssize_t) ScaleQuantumToChar(q->red)-
-              (ssize_t) ScaleQuantumToChar(GetPixelRed(p))]+
-              squares[(ssize_t) ScaleQuantumToChar(q->green)-
-              (ssize_t) ScaleQuantumToChar(GetPixelGreen(p))]+
-              squares[(ssize_t) ScaleQuantumToChar(q->blue)-
-              (ssize_t) ScaleQuantumToChar(GetPixelBlue(p))];
+            distance_squared=
+              squares[(ssize_t) (pixel.red-ScaleQuantumToChar(p->red))]+
+              squares[(ssize_t) (pixel.green-ScaleQuantumToChar(p->green))]+
+              squares[(ssize_t) (pixel.blue-ScaleQuantumToChar(p->blue))];
             numerator=distance_squared;
             for (k=0; k < (ssize_t) image->colors; k++)
             {
               p=image->colormap+k;
-              distance_squared=squares[(ssize_t) ScaleQuantumToChar(q->red)-
-                (ssize_t) ScaleQuantumToChar(GetPixelRed(p))]+
-                squares[(ssize_t) ScaleQuantumToChar(q->green)-
-                (ssize_t) ScaleQuantumToChar(GetPixelGreen(p))]+
-                squares[(ssize_t) ScaleQuantumToChar(q->blue)-
-                (ssize_t) ScaleQuantumToChar(GetPixelBlue(p))];
+              distance_squared=
+                squares[(ssize_t) (pixel.red-ScaleQuantumToChar(p->red))]+
+                squares[(ssize_t) (pixel.green-ScaleQuantumToChar(p->green))]+
+                squares[(ssize_t) (pixel.blue-ScaleQuantumToChar(p->blue))];
               ratio=numerator/distance_squared;
               sum+=SegmentPower(ratio);
             }
@@ -645,10 +656,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,
-          2*image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -699,7 +710,7 @@ static MagickBooleanType Classify(Image *image,short **extrema,
 static void ConsolidateCrossings(ZeroCrossing *zero_crossing,
   const size_t number_crossings)
 {
-  register ssize_t
+  ssize_t
     i,
     j,
     k,
@@ -872,7 +883,7 @@ static ssize_t DefineRegion(const short *extrema,ExtentPacket *extents)
 static void DerivativeHistogram(const MagickRealType *histogram,
   MagickRealType *derivative)
 {
-  register ssize_t
+  ssize_t
     i,
     n;
 
@@ -949,10 +960,10 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   MagickRealType
     threshold;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -968,7 +979,7 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
     Allocate histogram and extrema.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   GetMagickPixelPacket(image,pixel);
@@ -1003,9 +1014,9 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   */
   cluster=(Cluster *) NULL;
   head=(Cluster *) NULL;
-  (void) ResetMagickMemory(&red,0,sizeof(red));
-  (void) ResetMagickMemory(&green,0,sizeof(green));
-  (void) ResetMagickMemory(&blue,0,sizeof(blue));
+  (void) memset(&red,0,sizeof(red));
+  (void) memset(&green,0,sizeof(green));
+  (void) memset(&blue,0,sizeof(blue));
   while (DefineRegion(extrema[Red],&red) != 0)
   {
     green.index=0;
@@ -1019,13 +1030,13 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
         */
         if (head != (Cluster *) NULL)
           {
-            cluster->next=(Cluster *) AcquireMagickMemory(
+            cluster->next=(Cluster *) AcquireQuantumMemory(1,
               sizeof(*cluster->next));
             cluster=cluster->next;
           }
         else
           {
-            cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
+            cluster=(Cluster *) AcquireQuantumMemory(1,sizeof(*cluster));
             head=cluster;
           }
         if (cluster == (Cluster *) NULL)
@@ -1051,7 +1062,7 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
       /*
         No classes were identified-- create one.
       */
-      cluster=(Cluster *) AcquireMagickMemory(sizeof(*cluster));
+      cluster=(Cluster *) AcquireQuantumMemory(1,sizeof(*cluster));
       if (cluster == (Cluster *) NULL)
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
@@ -1079,30 +1090,27 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
+      MagickPixelPacket
+        pixel;
+
+      pixel.red=(double) ScaleQuantumToChar(p->red);
+      pixel.green=(double) ScaleQuantumToChar(p->green);
+      pixel.blue=(double) ScaleQuantumToChar(p->blue);
       for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
-        if (((ssize_t) ScaleQuantumToChar(GetPixelRed(p)) >=
-             (cluster->red.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelRed(p)) <=
-             (cluster->red.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelGreen(p)) >=
-             (cluster->green.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelGreen(p)) <=
-             (cluster->green.right+SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelBlue(p)) >=
-             (cluster->blue.left-SafeMargin)) &&
-            ((ssize_t) ScaleQuantumToChar(GetPixelBlue(p)) <=
-             (cluster->blue.right+SafeMargin)))
+        if ((pixel.red >= (double) (cluster->red.left-SafeMargin)) &&
+            (pixel.red <= (double) (cluster->red.right+SafeMargin)) &&
+            (pixel.green >= (double) (cluster->green.left-SafeMargin)) &&
+            (pixel.green <= (double) (cluster->green.right+SafeMargin)) &&
+            (pixel.blue >= (double) (cluster->blue.left-SafeMargin)) &&
+            (pixel.blue <= (double) (cluster->blue.right+SafeMargin)))
           {
             /*
               Count this pixel.
             */
             count++;
-            cluster->red.center+=(MagickRealType)
-              ScaleQuantumToChar(GetPixelRed(p));
-            cluster->green.center+=(MagickRealType)
-              ScaleQuantumToChar(GetPixelGreen(p));
-            cluster->blue.center+=(MagickRealType)
-              ScaleQuantumToChar(GetPixelBlue(p));
+            cluster->red.center+=pixel.red;
+            cluster->green.center+=pixel.green;
+            cluster->blue.center+=pixel.blue;
             cluster->count++;
             break;
           }
@@ -1221,10 +1229,10 @@ MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
 static void InitializeHistogram(const Image *image,ssize_t **histogram,
   ExceptionInfo *exception)
 {
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1296,7 +1304,7 @@ static void InitializeList(IntervalTree **list,ssize_t *number_nodes,
 
 static void MeanStability(IntervalTree *node)
 {
-  register IntervalTree
+  IntervalTree
     *child;
 
   if (node == (IntervalTree *) NULL)
@@ -1305,10 +1313,10 @@ static void MeanStability(IntervalTree *node)
   child=node->child;
   if (child != (IntervalTree *) NULL)
     {
-      register ssize_t
+      ssize_t
         count;
 
-      register MagickRealType
+      MagickRealType
         sum;
 
       sum=0.0;
@@ -1345,7 +1353,7 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
     *node,
     *root;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1364,12 +1372,15 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
   /*
     The root is the entire histogram.
   */
-  root=(IntervalTree *) AcquireMagickMemory(sizeof(*root));
+  root=(IntervalTree *) AcquireCriticalMemory(sizeof(*root));
   root->child=(IntervalTree *) NULL;
   root->sibling=(IntervalTree *) NULL;
   root->tau=0.0;
   root->left=0;
   root->right=255;
+  root->mean_stability=0.0;
+  root->stability=0.0;
+  (void) memset(list,0,TreeLength*sizeof(*list));
   for (i=(-1); i < (ssize_t) number_crossings; i++)
   {
     /*
@@ -1391,16 +1402,22 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
           {
             if (node == head)
               {
-                node->child=(IntervalTree *) AcquireMagickMemory(
+                node->child=(IntervalTree *) AcquireQuantumMemory(1,
                   sizeof(*node->child));
                 node=node->child;
               }
             else
               {
-                node->sibling=(IntervalTree *) AcquireMagickMemory(
+                node->sibling=(IntervalTree *) AcquireQuantumMemory(1,
                   sizeof(*node->sibling));
                 node=node->sibling;
               }
+            if (node == (IntervalTree *) NULL)
+              {
+                list=(IntervalTree **) RelinquishMagickMemory(list);
+                FreeNodes(root);
+                return((IntervalTree *) NULL);
+              }
             node->tau=zero_crossing[i+1].tau;
             node->child=(IntervalTree *) NULL;
             node->sibling=(IntervalTree *) NULL;
@@ -1411,9 +1428,15 @@ static IntervalTree *InitializeIntervalTree(const ZeroCrossing *zero_crossing,
         }
       if (left != head->left)
         {
-          node->sibling=(IntervalTree *) AcquireMagickMemory(
+          node->sibling=(IntervalTree *) AcquireQuantumMemory(1,
             sizeof(*node->sibling));
           node=node->sibling;
+          if (node == (IntervalTree *) NULL)
+            {
+              list=(IntervalTree **) RelinquishMagickMemory(list);
+              FreeNodes(root);
+              return((IntervalTree *) NULL);
+            }
           node->tau=zero_crossing[i+1].tau;
           node->child=(IntervalTree *) NULL;
           node->sibling=(IntervalTree *) NULL;
@@ -1506,7 +1529,7 @@ static MagickRealType OptimalTau(const ssize_t *histogram,const double max_tau,
     tau,
     value;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -1537,19 +1560,18 @@ static MagickRealType OptimalTau(const ssize_t *histogram,const double max_tau,
   zero_crossing=(ZeroCrossing *) AcquireQuantumMemory((size_t) count,
     sizeof(*zero_crossing));
   if (zero_crossing == (ZeroCrossing *) NULL)
-    return(0.0);
+    {
+      list=(IntervalTree **) RelinquishMagickMemory(list);
+      return(0.0);
+    }
   for (i=0; i < (ssize_t) count; i++)
     zero_crossing[i].tau=(-1.0);
   /*
     Initialize zero crossing list.
   */
-  derivative=(MagickRealType *) AcquireQuantumMemory(256,sizeof(*derivative));
-  second_derivative=(MagickRealType *) AcquireQuantumMemory(256,
+  derivative=(MagickRealType *) AcquireCriticalMemory(256*sizeof(*derivative));
+  second_derivative=(MagickRealType *) AcquireCriticalMemory(256*
     sizeof(*second_derivative));
-  if ((derivative == (MagickRealType *) NULL) ||
-      (second_derivative == (MagickRealType *) NULL))
-    ThrowFatalException(ResourceLimitFatalError,
-      "UnableToAllocateDerivatives");
   i=0;
   for (tau=max_tau; tau >= min_tau; tau-=delta_tau)
   {
@@ -1598,9 +1620,13 @@ static MagickRealType OptimalTau(const ssize_t *histogram,const double max_tau,
   */
   root=InitializeIntervalTree(zero_crossing,number_crossings);
   if (root == (IntervalTree *) NULL)
-    return(0.0);
+    {
+      zero_crossing=(ZeroCrossing *) RelinquishMagickMemory(zero_crossing);
+      list=(IntervalTree **) RelinquishMagickMemory(list);
+      return(0.0);
+    }
   /*
-    Find active nodes:  stability is greater (or equal) to the mean stability of
+    Find active nodes: stability is greater (or equal) to the mean stability of
     its children.
   */
   number_nodes=0;
@@ -1660,7 +1686,7 @@ static MagickRealType OptimalTau(const ssize_t *histogram,const double max_tau,
   average_tau=0.0;
   for (i=0; i < number_nodes; i++)
     average_tau+=list[i]->tau;
-  average_tau/=(MagickRealType) number_nodes;
+  average_tau*=PerceptibleReciprocal((MagickRealType) number_nodes);
   /*
     Relinquish resources.
   */
@@ -1694,7 +1720,6 @@ static MagickRealType OptimalTau(const ssize_t *histogram,const double max_tau,
 %      of pixels for each intensity of a particular color component.
 %
 */
-
 static void ScaleSpace(const ssize_t *histogram,const MagickRealType tau,
   MagickRealType *scale_histogram)
 {
@@ -1704,7 +1729,7 @@ static void ScaleSpace(const ssize_t *histogram,const MagickRealType tau,
     *gamma,
     sum;
 
-  register ssize_t
+  ssize_t
     u,
     x;
 
@@ -1780,7 +1805,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   short
@@ -1793,7 +1818,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
     Allocate histogram and extrema.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   for (i=0; i < MaxDimension; i++)
@@ -1807,7 +1832,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
           extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
           histogram[i]=(ssize_t *) RelinquishMagickMemory(histogram[i]);
         }
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+        ThrowBinaryImageException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename)
       }
   }
@@ -1817,12 +1842,12 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
   previous_colorspace=image->colorspace;
   (void) TransformImageColorspace(image,colorspace);
   InitializeHistogram(image,histogram,&image->exception);
-  (void) OptimalTau(histogram[Red],Tau,0.2,DeltaTau,
-    smooth_threshold == 0.0 ? 1.0 : smooth_threshold,extrema[Red]);
-  (void) OptimalTau(histogram[Green],Tau,0.2,DeltaTau,
-    smooth_threshold == 0.0 ? 1.0 : smooth_threshold,extrema[Green]);
-  (void) OptimalTau(histogram[Blue],Tau,0.2,DeltaTau,
-    smooth_threshold == 0.0 ? 1.0 : smooth_threshold,extrema[Blue]);
+  (void) OptimalTau(histogram[Red],Tau,0.2,DeltaTau,smooth_threshold == 0.0 ?
+    1.0 : smooth_threshold,extrema[Red]);
+  (void) OptimalTau(histogram[Green],Tau,0.2,DeltaTau,smooth_threshold == 0.0 ?
+    1.0 : smooth_threshold,extrema[Green]);
+  (void) OptimalTau(histogram[Blue],Tau,0.2,DeltaTau,smooth_threshold == 0.0 ?
+    1.0 : smooth_threshold,extrema[Blue]);
   /*
     Classify using the fuzzy c-Means technique.
   */
@@ -1872,7 +1897,7 @@ MagickExport MagickBooleanType SegmentImage(Image *image,
 static void ZeroCrossHistogram(MagickRealType *second_derivative,
   const MagickRealType smooth_threshold,short *crossings)
 {
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
