@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -128,7 +128,7 @@ static MagickBooleanType CropToFitImage(Image **image,
     geometry,
     page;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -164,8 +164,8 @@ static MagickBooleanType CropToFitImage(Image **image,
     if (max.y < extent[i].y)
       max.y=extent[i].y;
   }
-  geometry.x=(ssize_t) ceil(min.x-0.5);
-  geometry.y=(ssize_t) ceil(min.y-0.5);
+  geometry.x=CastDoubleToLong(ceil(min.x-0.5));
+  geometry.y=CastDoubleToLong(ceil(min.y-0.5));
   geometry.width=(size_t) floor(max.x-min.x+0.5);
   geometry.height=(size_t) floor(max.y-min.y+0.5);
   page=(*image)->page;
@@ -222,11 +222,11 @@ static void RadonProjection(const Image *image,MatrixInfo *source_matrix,
   MatrixInfo
     *swap;
 
-  register MatrixInfo
+  MatrixInfo
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     x;
 
   size_t
@@ -238,7 +238,7 @@ static void RadonProjection(const Image *image,MatrixInfo *source_matrix,
   {
     for (x=0; x < (ssize_t) GetMatrixColumns(p); x+=2*(ssize_t) step)
     {
-      register ssize_t
+      ssize_t
         i;
 
       ssize_t
@@ -298,7 +298,7 @@ static void RadonProjection(const Image *image,MatrixInfo *source_matrix,
 #endif
   for (x=0; x < (ssize_t) GetMatrixColumns(p); x++)
   {
-    register ssize_t
+    ssize_t
       y;
 
     size_t
@@ -338,7 +338,7 @@ static MagickBooleanType RadonTransform(const Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -389,10 +389,10 @@ static MagickBooleanType RadonTransform(const Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -446,10 +446,10 @@ static MagickBooleanType RadonTransform(const Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       i,
       x;
 
@@ -527,10 +527,10 @@ static void GetImageBackgroundColor(Image *image,const ssize_t offset,
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     if ((y >= offset) && (y < ((ssize_t) image->rows-offset)))
@@ -585,7 +585,7 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
   RectangleInfo
     geometry;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -735,15 +735,34 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
   assert(image != (Image *) NULL);
   page=image->page;
   rotations%=4;
-  if (rotations == 0)
-    return(CloneImage(image,0,0,MagickTrue,exception));
-  if ((rotations == 1) || (rotations == 3))
-    rotate_image=CloneImage(image,image->rows,image->columns,MagickTrue,
-      exception);
-  else
-    rotate_image=CloneImage(image,0,0,MagickTrue,exception);
+  rotate_image=(Image *) NULL;
+  rotate_view=(CacheView *) NULL;
+  switch (rotations)
+  {
+    case 0:
+    default:
+    {
+      rotate_image=CloneImage(image,0,0,MagickTrue,exception);
+      break;
+    }
+    case 2:
+    {
+      rotate_image=CloneImage(image,image->columns,image->rows,MagickTrue,
+        exception);
+      break;
+    }
+    case 1:
+    case 3:
+    {
+      rotate_image=CloneImage(image,image->rows,image->columns,MagickTrue,
+        exception);
+      break;
+    }
+  }
   if (rotate_image == (Image *) NULL)
     return((Image *) NULL);
+  if (rotations == 0)
+    return(rotate_image);
   /*
     Integral rotate the image.
   */
@@ -769,11 +788,11 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
       tile_width=image->columns;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
       #pragma omp parallel for schedule(static) shared(status) \
-        magick_number_threads(image,image,image->rows/tile_height,1)
+        magick_number_threads(image,rotate_image,image->rows/tile_height,1)
 #endif
       for (tile_y=0; tile_y < (ssize_t) image->rows; tile_y+=(ssize_t) tile_height)
       {
-        register ssize_t
+        ssize_t
           tile_x;
 
         if (status == MagickFalse)
@@ -783,19 +802,19 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           MagickBooleanType
             sync;
 
-          register const IndexPacket
+          const IndexPacket
             *magick_restrict indexes;
 
-          register const PixelPacket
+          const PixelPacket
             *magick_restrict p;
 
-          register IndexPacket
+          IndexPacket
             *magick_restrict rotate_indexes;
 
-          register PixelPacket
+          PixelPacket
             *magick_restrict q;
 
-          register ssize_t
+          ssize_t
             y;
 
           size_t
@@ -818,10 +837,10 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (ssize_t) width; y++)
           {
-            register const PixelPacket
+            const PixelPacket
               *magick_restrict tile_pixels;
 
-            register ssize_t
+            ssize_t
               x;
 
             if (status == MagickFalse)
@@ -844,7 +863,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             if ((indexes != (IndexPacket *) NULL) &&
                 (rotate_indexes != (IndexPacket *) NULL))
               {
-                register const IndexPacket
+                const IndexPacket
                   *magick_restrict tile_indexes;
 
                 tile_indexes=indexes+(height-1)*width+y;
@@ -885,26 +904,26 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
       #pragma omp parallel for schedule(static) shared(status) \
-        magick_number_threads(image,image,image->rows,1)
+        magick_number_threads(image,rotate_image,image->rows,1)
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
       {
         MagickBooleanType
           sync;
 
-        register const IndexPacket
+        const IndexPacket
           *magick_restrict indexes;
 
-        register const PixelPacket
+        const PixelPacket
           *magick_restrict p;
 
-        register IndexPacket
+        IndexPacket
           *magick_restrict rotate_indexes;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -967,11 +986,11 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
       tile_width=image->columns;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
       #pragma omp parallel for schedule(static) shared(status) \
-        magick_number_threads(image,image,image->rows/tile_height,1)
+        magick_number_threads(image,rotate_image,image->rows/tile_height,1)
 #endif
       for (tile_y=0; tile_y < (ssize_t) image->rows; tile_y+=(ssize_t) tile_height)
       {
-        register ssize_t
+        ssize_t
           tile_x;
 
         if (status == MagickFalse)
@@ -981,19 +1000,19 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           MagickBooleanType
             sync;
 
-          register const IndexPacket
+          const IndexPacket
             *magick_restrict indexes;
 
-          register const PixelPacket
+          const PixelPacket
             *magick_restrict p;
 
-          register IndexPacket
+          IndexPacket
             *magick_restrict rotate_indexes;
 
-          register PixelPacket
+          PixelPacket
             *magick_restrict q;
 
-          register ssize_t
+          ssize_t
             y;
 
           size_t
@@ -1016,10 +1035,10 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (ssize_t) width; y++)
           {
-            register const PixelPacket
+            const PixelPacket
               *magick_restrict tile_pixels;
 
-            register ssize_t
+            ssize_t
               x;
 
             if (status == MagickFalse)
@@ -1041,7 +1060,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             if ((indexes != (IndexPacket *) NULL) &&
                 (rotate_indexes != (IndexPacket *) NULL))
               {
-                register const IndexPacket
+                const IndexPacket
                   *magick_restrict tile_indexes;
 
                 tile_indexes=indexes+(width-1)-y;
@@ -1180,15 +1199,15 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
       area,
       displacement;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes,
       *magick_restrict shear_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       i;
 
     ShearDirection
@@ -1219,7 +1238,7 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
         displacement*=(-1.0);
         direction=LEFT;
       }
-    step=(ssize_t) floor((double) displacement);
+    step=CastDoubleToLong(floor((double) displacement));
     area=(MagickRealType) (displacement-step);
     step++;
     pixel=background;
@@ -1393,9 +1412,6 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
 #endif
   for (x=0; x < (ssize_t) width; x++)
   {
-    ssize_t
-      step;
-
     MagickPixelPacket
       pixel,
       source,
@@ -1405,20 +1421,23 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
       area,
       displacement;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes,
       *magick_restrict shear_indexes;
 
-    register ssize_t
+    ssize_t
       i;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict p,
       *magick_restrict q;
 
     ShearDirection
       direction;
 
+    ssize_t
+      step;
+
     if (status == MagickFalse)
       continue;
     p=GetCacheViewAuthenticPixels(image_view,x_offset+x,0,1,image->rows,
@@ -1441,7 +1460,7 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
         displacement*=(-1.0);
         direction=UP;
       }
-    step=(ssize_t) floor((double) displacement);
+    step=CastDoubleToLong(floor((double) displacement));
     area=(MagickRealType) (displacement-step);
     step++;
     pixel=background;
@@ -1618,11 +1637,12 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   /*
     Compute image size.
   */
-  bounds.width=image->columns+(ssize_t) floor(fabs(shear.x)*image->rows+0.5);
-  bounds.x=(ssize_t) ceil((double) image->columns+((fabs(shear.x)*image->rows)-
-    image->columns)/2.0-0.5);
-  bounds.y=(ssize_t) ceil((double) image->rows+((fabs(shear.y)*bounds.width)-
-    image->rows)/2.0-0.5);
+  bounds.width=image->columns+CastDoubleToLong(floor(fabs(shear.x)*
+    image->rows+0.5));
+  bounds.x=CastDoubleToLong(ceil((double) image->columns+((fabs(shear.x)*
+    image->rows)-image->columns)/2.0-0.5));
+  bounds.y=CastDoubleToLong(ceil((double) image->rows+((fabs(shear.y)*
+    bounds.width)-image->rows)/2.0-0.5));
   /*
     Surround image with border.
   */
@@ -1771,9 +1791,9 @@ MagickExport Image *ShearRotateImage(const Image *image,const double degrees,
   bounds.height=(size_t) floor(fabs((double) bounds.width*shear.y)+height+0.5);
   shear_width=(size_t) floor(fabs((double) bounds.height*shear.x)+
     bounds.width+0.5);
-  bounds.x=(ssize_t) floor((double) ((shear_width > bounds.width) ? width :
-    bounds.width-shear_width+2)/2.0+0.5);
-  bounds.y=(ssize_t) floor(((double) bounds.height-height+2)/2.0+0.5);
+  bounds.x=CastDoubleToLong(floor((double) ((shear_width > bounds.width) ?
+    width : bounds.width-shear_width+2)/2.0+0.5));
+  bounds.y=CastDoubleToLong(floor(((double) bounds.height-height+2)/2.0+0.5));
   /*
     Surround image with a border.
   */
