@@ -17,13 +17,13 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
 %  obtain a copy of the license at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %  unless required by applicable law or agreed to in writing, software        %
 %  distributed under the license is distributed on an "as is" basis,          %
 %  without warranties or conditions of any kind, either express or implied.   %
@@ -36,7 +36,7 @@
 */
 
 /*
-  include declarations.
+  Include declarations.
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
@@ -58,7 +58,12 @@
 #include "magick/utility-private.h"
 
 /*
-  static declarations.
+  Define declarations.
+*/
+#define CharsPerLine  0x14
+
+/*
+  Static declarations.
 */
 #if !defined(MAGICKCORE_HAVE_STRCASECMP) || !defined(MAGICKCORE_HAVE_STRNCASECMP)
 static const unsigned char
@@ -134,7 +139,6 @@ MagickExport char *AcquireString(const char *source)
     sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -163,7 +167,8 @@ MagickExport char *AcquireString(const char *source)
 %    o length: the string length.
 %
 */
-MagickExport StringInfo *AcquireStringInfo(const size_t length)
+
+static StringInfo *AcquireStringInfoContainer()
 {
   StringInfo
     *string_info;
@@ -171,15 +176,25 @@ MagickExport StringInfo *AcquireStringInfo(const size_t length)
   string_info=(StringInfo *) AcquireMagickMemory(sizeof(*string_info));
   if (string_info == (StringInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(string_info,0,sizeof(*string_info));
-  string_info->signature=MagickSignature;
+  (void) memset(string_info,0,sizeof(*string_info));
+  string_info->signature=MagickCoreSignature;
+  return(string_info);
+}
+
+MagickExport StringInfo *AcquireStringInfo(const size_t length)
+{
+  StringInfo
+    *string_info;
+
+  string_info=AcquireStringInfoContainer();
   string_info->length=length;
-  string_info->datum=(unsigned char *) NULL;
   if (~string_info->length >= (MaxTextExtent-1))
     string_info->datum=(unsigned char *) AcquireQuantumMemory(
       string_info->length+MaxTextExtent,sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memset(string_info->datum,0,(length+MagickPathExtent)*
+    sizeof(*string_info->datum));
   return(string_info);
 }
 
@@ -213,16 +228,12 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
   StringInfo
     *string_info;
 
-  string_info=AcquireStringInfo(0);
   if (~length < MaxTextExtent)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  string_info=AcquireStringInfoContainer();
   string_info->length=length;
-  if (string_info->datum == (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
-      MaxTextExtent,sizeof(*string_info->datum));
-  else
-    string_info->datum=(unsigned char *) ResizeQuantumMemory(string_info->datum,
-      length+MaxTextExtent,sizeof(*string_info->datum));
+  string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+    MaxTextExtent,sizeof(*string_info->datum));
   if (string_info->datum == (unsigned char *) NULL)
     {
       string_info=DestroyStringInfo(string_info);
@@ -230,6 +241,10 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
     }
   if (blob != (const void *) NULL)
     (void) memcpy(string_info->datum,blob,length);
+  else
+    (void) memset(string_info->datum,0,length*sizeof(*string_info->datum));
+  (void) memset(string_info->datum+length,0,MagickPathExtent*
+    sizeof(*string_info->datum));
   return(string_info);
 }
 
@@ -323,8 +338,10 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
     *clone_info;
 
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   clone_info=AcquireStringInfo(string_info->length);
+  (void) strcpy(clone_info->path,string_info->path);
+  (void) CloneString(&clone_info->name,string_info->name);
   if (string_info->length != 0)
     (void) memcpy(clone_info->datum,string_info->datum,string_info->length+1);
   return(clone_info);
@@ -364,9 +381,9 @@ MagickExport int CompareStringInfo(const StringInfo *target,
     status;
 
   assert(target != (StringInfo *) NULL);
-  assert(target->signature == MagickSignature);
+  assert(target->signature == MagickCoreSignature);
   assert(source != (StringInfo *) NULL);
-  assert(source->signature == MagickSignature);
+  assert(source->signature == MagickCoreSignature);
   status=memcmp(target->datum,source->datum,MagickMin(target->length,
     source->length));
   if (status != 0)
@@ -393,8 +410,8 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %
 %  The format of the ConcatenateMagickString method is:
 %
-%      size_t ConcatenateMagickString(char *destination,const char *source,
-%        const size_t length)
+%      size_t ConcatenateMagickString(char *magick_restrict destination,
+%        const char *magick_restrict source,const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -405,16 +422,16 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %    o length: the length of the destination string.
 %
 */
-MagickExport size_t ConcatenateMagickString(char *destination,
-  const char *source,const size_t length)
+MagickExport size_t ConcatenateMagickString(char *magick_restrict destination,
+  const char *magick_restrict source,const size_t length)
 {
-  register char
-    *q;
+  char
+    *magick_restrict q;
 
-  register const char
-    *p;
+  const char
+    *magick_restrict p;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -461,8 +478,8 @@ MagickExport size_t ConcatenateMagickString(char *destination,
 %
 %  The format of the ConcatenateString method is:
 %
-%      MagickBooleanType ConcatenateString(char **destination,
-%        const char *source)
+%      MagickBooleanType ConcatenateString(char **magick_restrict destination,
+%        const char *magick_restrict source)
 %
 %  A description of each parameter follows:
 %
@@ -471,8 +488,8 @@ MagickExport size_t ConcatenateMagickString(char *destination,
 %    o source: A character string.
 %
 */
-MagickExport MagickBooleanType ConcatenateString(char **destination,
-  const char *source)
+MagickExport MagickBooleanType ConcatenateString(
+  char **magick_restrict destination,const char *magick_restrict source)
 {
   size_t
     destination_length,
@@ -538,7 +555,7 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
     length;
 
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   assert(source != (const StringInfo *) NULL);
   length=string_info->length;
   if (~length < source->length)
@@ -618,7 +635,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
     }
   else
     {
-      register size_t
+      size_t
         i;
 
       ssize_t
@@ -628,7 +645,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
       for (i=0; i < length; i+=count)
       {
         count=read(file,string+i,(size_t) MagickMin(length-i,(size_t)
-          SSIZE_MAX));
+          MAGICK_SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -645,12 +662,9 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
     }
   string[length]='\0';
   file=close(file)-1;
-  string_info=AcquireStringInfo(0);
+  string_info=AcquireStringInfoContainer();
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
   string_info->length=length;
-  if (string_info->datum != (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) RelinquishMagickMemory( 
-      string_info->datum);
   string_info->datum=(unsigned char *) string;
   return(string_info);
 }
@@ -697,7 +711,6 @@ MagickExport char *ConstantString(const char *source)
     destination=(char *) AcquireQuantumMemory(length+1UL,sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -719,13 +732,12 @@ MagickExport char *ConstantString(const char *source)
 %  out exceeding the given pre-declared length.
 %
 %  The destination buffer is always null-terminated even if the string must be
-%  truncated.  The return value is the minimum of the source string length or
-%  the length parameter.
+%  truncated.  The return value is length of the string.
 %
 %  The format of the CopyMagickString method is:
 %
-%      size_t CopyMagickString(const char *destination,char *source,
-%        const size_t length)
+%      size_t CopyMagickString(const char *magick_restrict destination,
+%        char *magick_restrict source,const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -736,50 +748,39 @@ MagickExport char *ConstantString(const char *source)
 %    o length: the length of the destination string.
 %
 */
-MagickExport size_t CopyMagickString(char *destination,const char *source,
-  const size_t length)
+MagickExport size_t CopyMagickString(char *magick_restrict destination,
+  const char *magick_restrict source,const size_t length)
 {
-  register char
-    *q;
+  char
+    *magick_restrict q;
 
-  register const char
-    *p;
+  const char
+    *magick_restrict p;
 
-  register size_t
+  size_t
     n;
 
   p=source;
   q=destination;
   for (n=length; n > 4; n-=4)
   {
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
   }
-  if (n != 0)
-    for (n--; n != 0; n--)
-    {
-      *q=(*p++);
-      if (*q == '\0')
-        return((size_t) (p-source-1));
-      q++;
-    }
   if (length != 0)
-    *q='\0';
-  return((size_t) (p-source-1));
+    {
+      while (--n != 0)
+        if (((*q++)=(*p++)) == '\0')
+          return((size_t) (p-source-1));
+      *q='\0';
+    }
+  return((size_t) (p-source));
 }
 
 /*
@@ -834,11 +835,13 @@ MagickExport char *DestroyString(char *string)
 MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   if (string_info->datum != (unsigned char *) NULL)
     string_info->datum=(unsigned char *) RelinquishMagickMemory(
       string_info->datum);
-  string_info->signature=(~MagickSignature);
+  if (string_info->name != (char *) NULL)
+    string_info->name=DestroyString(string_info->name);
+  string_info->signature=(~MagickCoreSignature);
   string_info=(StringInfo *) RelinquishMagickMemory(string_info);
   return(string_info);
 }
@@ -867,7 +870,7 @@ MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 */
 MagickExport char **DestroyStringList(char **list)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(list != (char **) NULL);
@@ -911,10 +914,10 @@ MagickExport char *EscapeString(const char *source,const char escape)
   char
     *destination;
 
-  register char
+  char
     *q;
 
-  register const char
+  const char
     *p;
 
   size_t
@@ -1024,11 +1027,8 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
   assert(filename != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
-  string_info=AcquireStringInfo(0);
+  string_info=AcquireStringInfoContainer();
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
-  if (string_info->datum != (unsigned char *) NULL)
-    string_info->datum=(unsigned char *) RelinquishMagickMemory( 
-      string_info->datum);
   string_info->datum=FileToBlob(filename,extent,&string_info->length,exception);
   if (string_info->datum == (unsigned char *) NULL)
     {
@@ -1055,7 +1055,8 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
 %
 %  The format of the FormatMagickSize method is:
 %
-%      ssize_t FormatMagickSize(const MagickSizeType size,char *format)
+%      ssize_t FormatMagickSize(const MagickSizeType size,
+%        const MagickBooleanType by,char *format)
 %
 %  A description of each parameter follows:
 %
@@ -1069,6 +1070,10 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
 MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
   const MagickBooleanType bi,char *format)
 {
+  char
+    p[MaxTextExtent],
+    q[MaxTextExtent];
+
   const char
     **units;
 
@@ -1076,9 +1081,8 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
     bytes,
     length;
 
-  register ssize_t
-    i,
-    j;
+  ssize_t
+    i;
 
   ssize_t
     count;
@@ -1105,96 +1109,18 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
 #else
   length=(double) size;
 #endif
+  (void) FormatLocaleString(p,MaxTextExtent,"%.*g",GetMagickPrecision(),
+    length);
+  (void) FormatLocaleString(q,MaxTextExtent,"%.20g",length);
+  if (strtod(p,(char **) NULL) == strtod(q,(char **) NULL))
+    {
+      count=FormatLocaleString(format,MaxTextExtent,"%.20g%sB",length,units[0]);
+      return(count);
+    }
   for (i=0; (length >= bytes) && (units[i+1] != (const char *) NULL); i++)
     length/=bytes;
-  count=0;
-  for (j=2; j < 12; j++)
-  {
-    count=FormatLocaleString(format,MaxTextExtent,"%.*g%sB",(int) (i+j),length,
-      units[i]);
-    if (strchr(format,'+') == (char *) NULL)
-      break;
-  }
-  return(count);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%  F o r m a t M a g i c k T i m e                                            %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  FormatMagickTime() returns the specified time in the Internet date/time
-%  format and the length of the timestamp.
-%
-%  The format of the FormatMagickTime method is:
-%
-%      ssize_t FormatMagickTime(const time_t time,const size_t length,
-%        char *timestamp)
-%
-%  A description of each parameter follows.
-%
-%   o time:  the time since the Epoch (00:00:00 UTC, January 1, 1970),
-%     measured in seconds.
-%
-%   o length: the maximum length of the string.
-%
-%   o timestamp:  Return the Internet date/time here.
-%
-*/
-MagickExport ssize_t FormatMagickTime(const time_t time,const size_t length,
-  char *timestamp)
-{
-  ssize_t
-    count;
-
-  struct tm
-    gm_time,
-    local_time;
-
-  time_t
-    timezone;
-
-  assert(timestamp != (char *) NULL);
-  (void) ResetMagickMemory(&local_time,0,sizeof(local_time));
-  (void) ResetMagickMemory(&gm_time,0,sizeof(gm_time));
-#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
-  (void) localtime_r(&time,&local_time);
-#else
-  {
-    struct tm
-      *my_time;
-
-    my_time=localtime(&time);
-    if (my_time != (struct tm *) NULL)
-      (void) memcpy(&local_time,my_time,sizeof(local_time));
-  }
-#endif
-#if defined(MAGICKCORE_HAVE_GMTIME_R)
-  (void) gmtime_r(&time,&gm_time);
-#else
-  {
-    struct tm
-      *my_time;
-
-    my_time=gmtime(&time);
-    if (my_time != (struct tm *) NULL)
-      (void) memcpy(&gm_time,my_time,sizeof(gm_time));
-  }
-#endif
-  timezone=(time_t) ((local_time.tm_min-gm_time.tm_min)/60+
-    local_time.tm_hour-gm_time.tm_hour+24*((local_time.tm_year-
-    gm_time.tm_year) != 0 ? (local_time.tm_year-gm_time.tm_year) :
-    (local_time.tm_yday-gm_time.tm_yday)));
-  count=FormatLocaleString(timestamp,length,
-    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time.tm_year+1900,
-    local_time.tm_mon+1,local_time.tm_mday,local_time.tm_hour,
-    local_time.tm_min,local_time.tm_sec,(long) timezone);
+  count=FormatLocaleString(format,MaxTextExtent,"%.*g%sB",GetMagickPrecision(),
+    length,units[i]);
   return(count);
 }
 
@@ -1257,7 +1183,7 @@ MagickExport char *GetEnvironmentValue(const char *name)
 MagickExport unsigned char *GetStringInfoDatum(const StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   return(string_info->datum);
 }
 
@@ -1286,7 +1212,7 @@ MagickExport unsigned char *GetStringInfoDatum(const StringInfo *string_info)
 MagickExport size_t GetStringInfoLength(const StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   return(string_info->length);
 }
 
@@ -1295,6 +1221,35 @@ MagickExport size_t GetStringInfoLength(const StringInfo *string_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t S t r i n g I n f o N a m e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetStringInfoName() returns the name associated with the string.
+%
+%  The format of the GetStringInfoName method is:
+%
+%      const char *GetStringInfoName(const StringInfo *string_info)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+*/
+MagickExport const char *GetStringInfoName(const StringInfo *string_info)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickCoreSignature);
+  return(string_info->name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t S t r i n g I n f o P a t h                                         %
 %                                                                             %
 %                                                                             %
@@ -1315,7 +1270,7 @@ MagickExport size_t GetStringInfoLength(const StringInfo *string_info)
 MagickExport const char *GetStringInfoPath(const StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   return(string_info->path);
 }
 
@@ -1522,16 +1477,16 @@ MagickExport MagickBooleanType IsStringNotFalse(const char *value)
 MagickExport void PrintStringInfo(FILE *file,const char *id,
   const StringInfo *string_info)
 {
-  register const char
+  const char
     *p;
 
-  register size_t
+  size_t
     i,
     j;
 
   assert(id != (const char *) NULL);
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   p=(char *) string_info->datum;
   for (i=0; i < string_info->length; i++)
   {
@@ -1540,7 +1495,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
       break;
     p++;
   }
-  (void) FormatLocaleFile(file,"%s(%.20g): ",id,(double) string_info->length);
+  (void) FormatLocaleFile(file,"%s(%.20g):\n",id,(double) string_info->length);
   if (i == string_info->length)
     {
       for (i=0; i < string_info->length; i++)
@@ -1552,16 +1507,16 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
     Convert string to a HEX list.
   */
   p=(char *) string_info->datum;
-  for (i=0; i < string_info->length; i+=0x14)
+  for (i=0; i < string_info->length; i+=CharsPerLine)
   {
-    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (0x14*i));
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (CharsPerLine*i));
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       (void) FormatLocaleFile(file,"%02lx",(unsigned long) (*(p+j)) & 0xff);
       if ((j % 0x04) == 0)
         (void) fputc(' ',file);
     }
-    for ( ; j <= 0x14; j++)
+    for ( ; j <= CharsPerLine; j++)
     {
       (void) fputc(' ',file);
       (void) fputc(' ',file);
@@ -1569,7 +1524,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
         (void) fputc(' ',file);
     }
     (void) fputc(' ',file);
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       if (isprint((int) ((unsigned char) *p)) != 0)
         (void) fputc(*p,file);
@@ -1606,8 +1561,8 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
 MagickExport void ResetStringInfo(StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
-  (void) ResetMagickMemory(string_info->datum,0,string_info->length);
+  assert(string_info->signature == MagickCoreSignature);
+  (void) memset(string_info->datum,0,string_info->length);
 }
 
 /*
@@ -1621,10 +1576,10 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SanitizeString() returns an new string removes all characters except
+%  SanitizeString() returns a new string with all characters removed except
 %  letters, digits and !#$%&'*+-=?^_`{|}~@.[].
 %
-%  The returned string shoud be freed using DestoryString().
+%  Free the sanitized string with DestroyString().
 %
 %  The format of the SanitizeString method is:
 %
@@ -1642,19 +1597,19 @@ MagickExport char *SanitizeString(const char *source)
 
   const char
     *q;
-  
-  register char
+
+  char
     *p;
-  
+
   static char
-    whitelist[] =
+    allowlist[] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
       "$-_.+!*'(),{}|\\^~[]`\"><#%;/?:@&=";
-  
+
   sanitize_source=AcquireString(source);
   p=sanitize_source;
   q=sanitize_source+strlen(sanitize_source);
-  for (p+=strspn(p,whitelist); p != q; p+=strspn(p,whitelist))
+  for (p+=strspn(p,allowlist); p != q; p+=strspn(p,allowlist))
     *p='_';
   return(sanitize_source);
 }
@@ -1687,12 +1642,12 @@ MagickExport void SetStringInfo(StringInfo *string_info,
   const StringInfo *source)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   assert(source != (StringInfo *) NULL);
-  assert(source->signature == MagickSignature);
+  assert(source->signature == MagickCoreSignature);
   if (string_info->length == 0)
     return;
-  (void) ResetMagickMemory(string_info->datum,0,string_info->length);
+  (void) memset(string_info->datum,0,string_info->length);
   (void) memcpy(string_info->datum,source->datum,MagickMin(string_info->length,
     source->length));
 }
@@ -1727,7 +1682,7 @@ MagickExport void SetStringInfoDatum(StringInfo *string_info,
   const unsigned char *source)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   if (string_info->length != 0)
     (void) memcpy(string_info->datum,source,string_info->length);
 }
@@ -1760,7 +1715,7 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
   const size_t length)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   if (string_info->length == length)
     return;
   if (~length < MaxTextExtent)
@@ -1781,7 +1736,39 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   S e t S t r i n g I n f o D a t u m                                       %
+%   S e t S t r i n g I n f o N a m e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetStringInfoName() sets the name associated with the string.
+%
+%  The format of the SetStringInfoName method is:
+%
+%      void SetStringInfoName(StringInfo *string_info,const char *name)
+%
+%  A description of each parameter follows:
+%
+%    o string_info: the string info.
+%
+%    o name: the name.
+%
+*/
+MagickExport void SetStringInfoName(StringInfo *string_info,const char *name)
+{
+  assert(string_info != (StringInfo *) NULL);
+  assert(string_info->signature == MagickCoreSignature);
+  assert(name != (const char *) NULL);
+  string_info->name=ConstantString(name);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t S t r i n g I n f o P a t h                                         %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1803,7 +1790,7 @@ MagickExport void SetStringInfoLength(StringInfo *string_info,
 MagickExport void SetStringInfoPath(StringInfo *string_info,const char *path)
 {
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   assert(path != (const char *) NULL);
   (void) CopyMagickString(string_info->path,path,MaxTextExtent);
 }
@@ -1837,7 +1824,7 @@ MagickExport StringInfo *SplitStringInfo(StringInfo *string_info,
     *split_info;
 
   assert(string_info != (StringInfo *) NULL);
-  assert(string_info->signature == MagickSignature);
+  assert(string_info->signature == MagickCoreSignature);
   if (offset > string_info->length)
     return((StringInfo *) NULL);
   split_info=AcquireStringInfo(offset);
@@ -1916,13 +1903,13 @@ MagickExport char *StringInfoToHexString(const StringInfo *string_info)
   char
     *string;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -2004,11 +1991,11 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   char
     **argv;
 
-  register const char
+  const char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   *argc=0;
@@ -2126,15 +2113,17 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
   double
     *array;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
     Determine count of values, and check syntax.
   */
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   *count=0;
+  if (string == (char *) NULL)
+    return((double *) NULL);  /* no value found */
   i=0;
   p=string;
   while (*p != '\0')
@@ -2190,13 +2179,13 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
 %
 %  StringToken() looks for any one of given delimiters and splits the string
 %  into two separate strings by replacing the delimiter character found with a
-%  nul character.
+%  null character.
 %
 %  The given string pointer is changed to point to the string following the
 %  delimiter character found, or NULL.  A pointer to the start of the
 %  string is returned, representing the token before the delimiter.
 %
-%  In may ways this is equivent to the strtok() C library function, but with
+%  StringToken() is equivent to the strtok() C library method, but with
 %  multiple delimiter characters rather than a delimiter string.
 %
 %  The format of the StringToken method is:
@@ -2216,13 +2205,13 @@ MagickExport char *StringToken(const char *delimiters,char **string)
   char
     *q;
 
-  register char
+  char
     *p;
 
-  register const char
+  const char
     *r;
 
-  register int
+  int
     c,
     d;
 
@@ -2275,28 +2264,63 @@ MagickExport char *StringToken(const char *delimiters,char **string)
 %
 */
 MagickExport char **StringToList(const char *text)
+{
+  return(StringToStrings(text,(size_t *) NULL));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%  S t r i n g T o S t r i n g s                                              %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringToStrings() converts a text string into a list by segmenting the text
+%  string at each carriage return discovered.  The list is converted to HEX
+%  characters if any control characters are discovered within the text string.
+%
+%  The format of the StringToList method is:
+%
+%      char **StringToList(const char *text,size_t *lines)
+%
+%  A description of each parameter follows:
+%
+%    o text:  Specifies the string to segment into a list.
+%
+%    o count: Return value for the number of items in the list.
+%
+*/
+MagickExport char **StringToStrings(const char *text,size_t *count)
 {
   char
     **textlist;
 
-  register const char
+  const char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
     lines;
 
   if (text == (char *) NULL)
-    return((char **) NULL);
+    {
+      if (count != (size_t *) NULL)
+        *count=0;
+      return((char **) NULL);
+    }
   for (p=text; *p != '\0'; p++)
     if (((int) ((unsigned char) *p) < 32) &&
         (isspace((int) ((unsigned char) *p)) == 0))
       break;
   if (*p == '\0')
     {
-      register const char
+      const char
         *q;
 
       /*
@@ -2316,7 +2340,7 @@ MagickExport char **StringToList(const char *text)
         for (q=p; *q != '\0'; q++)
           if ((*q == '\r') || (*q == '\n'))
             break;
-        textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+MaxTextExtent,
+        textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+1,
           sizeof(**textlist));
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
@@ -2330,18 +2354,18 @@ MagickExport char **StringToList(const char *text)
   else
     {
       char
-        hex_string[MaxTextExtent];
+        hex_string[MagickPathExtent];
 
-      register char
+      char
         *q;
 
-      register ssize_t
+      ssize_t
         j;
 
       /*
         Convert string to a HEX list.
       */
-      lines=(size_t) (strlen(text)/0x14)+1;
+      lines=(size_t) (strlen(text)/CharsPerLine)+1;
       textlist=(char **) AcquireQuantumMemory((size_t) lines+1UL,
         sizeof(*textlist));
       if (textlist == (char **) NULL)
@@ -2349,22 +2373,26 @@ MagickExport char **StringToList(const char *text)
       p=text;
       for (i=0; i < (ssize_t) lines; i++)
       {
-        textlist[i]=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
+        size_t
+          length;
+
+        textlist[i]=(char *) AcquireQuantumMemory(2UL*MagickPathExtent,
           sizeof(**textlist));
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
-        (void) FormatLocaleString(textlist[i],MaxTextExtent,"0x%08lx: ",
-          (long) (0x14*i));
+        (void) FormatLocaleString(textlist[i],MagickPathExtent,"0x%08lx: ",
+          (long) (CharsPerLine*i));
         q=textlist[i]+strlen(textlist[i]);
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        length=strlen(p);
+        for (j=1; j <= (ssize_t) MagickMin(length,CharsPerLine); j++)
         {
-          (void) FormatLocaleString(hex_string,MaxTextExtent,"%02x",*(p+j));
-          (void) CopyMagickString(q,hex_string,MaxTextExtent);
+          (void) FormatLocaleString(hex_string,MagickPathExtent,"%02x",*(p+j));
+          (void) CopyMagickString(q,hex_string,MagickPathExtent);
           q+=2;
           if ((j % 0x04) == 0)
             *q++=' ';
         }
-        for ( ; j <= 0x14; j++)
+        for ( ; j <= CharsPerLine; j++)
         {
           *q++=' ';
           *q++=' ';
@@ -2372,7 +2400,7 @@ MagickExport char **StringToList(const char *text)
             *q++=' ';
         }
         *q++=' ';
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        for (j=1; j <= (ssize_t) MagickMin(length,CharsPerLine); j++)
         {
           if (isprint((int) ((unsigned char) *p)) != 0)
             *q++=(*p);
@@ -2381,8 +2409,14 @@ MagickExport char **StringToList(const char *text)
           p++;
         }
         *q='\0';
+        textlist[i]=(char *) ResizeQuantumMemory(textlist[i],(size_t) (q-
+          textlist[i]+1),sizeof(**textlist));
+        if (textlist[i] == (char *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
       }
     }
+  if (count != (size_t *) NULL)
+    *count=lines;
   textlist[i]=(char *) NULL;
   return(textlist);
 }
@@ -2445,7 +2479,7 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
 */
 MagickExport void StripString(char *message)
 {
-  register char
+  char
     *p,
     *q;
 
@@ -2511,7 +2545,7 @@ MagickExport MagickBooleanType SubstituteString(char **string,
   MagickBooleanType
     status;
 
-  register char
+  char
     *p;
 
   size_t
@@ -2554,9 +2588,9 @@ MagickExport MagickBooleanType SubstituteString(char **string,
       Replace string.
     */
     if (search_extent != replace_extent)
-      (void) CopyMagickMemory(p+replace_extent,p+search_extent,
+      (void) memmove(p+replace_extent,p+search_extent,
         strlen(p+search_extent)+1);
-    (void) CopyMagickMemory(p,replace,replace_extent);
+    (void) memcpy(p,replace,replace_extent);
     p+=replace_extent-1;
   }
   return(status);
