@@ -17,7 +17,7 @@
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -41,7 +41,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/property.h"
+#include "magick/artifact.h"
 #include "magick/blob.h"
 #include "magick/cache-view.h"
 #include "magick/color.h"
@@ -69,6 +69,7 @@
 #include "magick/montage.h"
 #include "magick/option.h"
 #include "magick/pixel-private.h"
+#include "magick/property.h"
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/random_.h"
@@ -202,6 +203,8 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   threshold_image=CloneImage(image,0,0,MagickTrue,exception);
   if (threshold_image == (Image *) NULL)
     return((Image *) NULL);
+  if ((width == 0) || (height == 0))
+    return(threshold_image);
   if (SetImageStorageClass(threshold_image,DirectClass) == MagickFalse)
     {
       InheritException(exception,&threshold_image->exception);
@@ -230,20 +233,20 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       channel_bias,
       channel_sum;
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p,
       *magick_restrict r;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict threshold_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     ssize_t
@@ -348,10 +351,10 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_AdaptiveThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -374,9 +377,8 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AutoThresholdImage() automatically selects a threshold and replaces each
-%  pixel in the image with a black pixel if the image intentsity is less than
-%  the selected threshold otherwise white.
+%  AutoThresholdImage() automatically performs image thresholding
+%  dependent on which method you specify.
 %
 %  The format of the AutoThresholdImage method is:
 %
@@ -406,7 +408,7 @@ static double KapurThreshold(const Image *image,const double *histogram,
     maximum_entropy,
     *white_entropy;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -503,7 +505,7 @@ static double OTSUThreshold(const Image *image,const double *histogram,
     *sigma,
     threshold;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -571,8 +573,7 @@ static double OTSUThreshold(const Image *image,const double *histogram,
   return(100.0*threshold/MaxIntensity);
 }
 
-static double TriangleThreshold(const Image *image,const double *histogram,
-  ExceptionInfo *exception)
+static double TriangleThreshold(const Image *image,const double *histogram)
 {
   double
     a,
@@ -588,7 +589,7 @@ static double TriangleThreshold(const Image *image,const double *histogram,
     y1,
     y2;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -600,7 +601,7 @@ static double TriangleThreshold(const Image *image,const double *histogram,
   /*
     Compute optimal threshold with triangle algorithm.
   */
-  (void) exception;
+  magick_unreferenced(image);
   start=0;  /* find start bin, first bin not zero count */
   for (i=0; i <= (ssize_t) MaxIntensity; i++)
     if (histogram[i] > 0.0)
@@ -672,6 +673,9 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   char
     property[MagickPathExtent];
 
+  const char
+    *artifact;
+
   double
     gamma,
     *histogram,
@@ -681,7 +685,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -704,10 +708,10 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register ssize_t
+    ssize_t
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
@@ -748,7 +752,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
     }
     case TriangleThresholdMethod:
     {
-      threshold=TriangleThreshold(image,histogram,exception);
+      threshold=TriangleThreshold(image,histogram);
       break;
     }
   }
@@ -762,6 +766,9 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   */
   (void) FormatLocaleString(property,MagickPathExtent,"%g%%",threshold);
   (void) SetImageProperty(image,"auto-threshold:threshold",property);
+  artifact=GetImageArtifact(image,"threshold:verbose");
+  if (IsStringTrue(artifact) != MagickFalse)
+    (void) FormatLocaleFile(stdout,"%.*g%%\n",GetMagickPrecision(),threshold);
   return(BilevelImage(image,QuantumRange*threshold/100.0));
 }
 
@@ -843,7 +850,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  if (IsGrayColorspace(image->colorspace) != MagickFalse)
+  if (IsGrayColorspace(image->colorspace) == MagickFalse)
     (void) SetImageColorspace(image,sRGBColorspace);
   /*
     Bilevel threshold image.
@@ -858,13 +865,13 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -905,7 +912,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
               SetPixelOpacity(q,(MagickRealType) GetPixelOpacity(q) <=
                 threshold ? 0 : QuantumRange);
             else
-              SetPixelAlpha(q,(MagickRealType) GetPixelAlpha(q) <= threshold ? 
+              SetPixelAlpha(q,(MagickRealType) GetPixelAlpha(q) <= threshold ?
                 OpaqueOpacity : TransparentOpacity);
           }
         if (((channel & IndexChannel) != 0) &&
@@ -922,10 +929,10 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_BilevelImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1050,13 +1057,13 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1096,10 +1103,10 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_BlackThresholdImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1172,10 +1179,10 @@ MagickExport MagickBooleanType ClampImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       q=image->colormap;
@@ -1202,13 +1209,13 @@ MagickExport MagickBooleanType ClampImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1244,9 +1251,10 @@ MagickExport MagickBooleanType ClampImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_ClampImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ClampImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ClampImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1469,7 +1477,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
     char
       *p;
 
-    register ssize_t
+    ssize_t
       i;
 
     /*
@@ -1859,7 +1867,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
     char
       token[MaxTextExtent];
 
-    register const char
+    const char
       *p;
 
     p=(char *)threshold_map;
@@ -1964,13 +1972,13 @@ printf("DEBUG levels  r=%u g=%u b=%u a=%u i=%u\n",
 #endif
     for (y=0; y < (ssize_t) image->rows; y++)
     {
-      register IndexPacket
+      IndexPacket
         *magick_restrict indexes;
 
-      register ssize_t
+      ssize_t
         x;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       if (status == MagickFalse)
@@ -1984,7 +1992,7 @@ printf("DEBUG levels  r=%u g=%u b=%u a=%u i=%u\n",
       indexes=GetCacheViewAuthenticIndexQueue(image_view);
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        register ssize_t
+        ssize_t
           threshold,
           t,
           l;
@@ -2051,9 +2059,10 @@ printf("DEBUG levels  r=%u g=%u b=%u a=%u i=%u\n",
             proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-          #pragma omp critical (MagickCore_OrderedPosterizeImageChannel)
+          #pragma omp atomic
 #endif
-          proceed=SetImageProgress(image,DitherImageTag,progress++,image->rows);
+          progress++;
+          proceed=SetImageProgress(image,DitherImageTag,progress,image->rows);
           if (proceed == MagickFalse)
             status=MagickFalse;
         }
@@ -2143,10 +2152,10 @@ MagickExport MagickBooleanType PerceptibleImageChannel(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
-      register ssize_t
+      ssize_t
         i;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       q=image->colormap;
@@ -2173,13 +2182,13 @@ MagickExport MagickBooleanType PerceptibleImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2215,9 +2224,11 @@ MagickExport MagickBooleanType PerceptibleImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_PerceptibleImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,PerceptibleImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,PerceptibleImageTag,progress,
+          image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -2368,13 +2379,13 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
         MagickBooleanType
           sync;
 
-        register IndexPacket
+        IndexPacket
           *magick_restrict indexes;
 
-        register ssize_t
+        ssize_t
           x;
 
-        register PixelPacket
+        PixelPacket
           *magick_restrict q;
 
         if (status == MagickFalse)
@@ -2417,9 +2428,10 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical (MagickCore_RandomThresholdImageChannel)
+            #pragma omp atomic
 #endif
-            proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+            progress++;
+            proceed=SetImageProgress(image,ThresholdImageTag,progress,
               image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
@@ -2446,13 +2458,13 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     const int
       id = GetOpenMPThreadId();
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -2548,10 +2560,10 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RandomThresholdImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -2677,13 +2689,13 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2723,10 +2735,10 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_WhiteThresholdImageChannel)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
