@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,6 +60,7 @@
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
+#include "magick/property.h"
 #include "magick/resource_.h"
 #include "magick/resize.h"
 #include "magick/statistic.h"
@@ -255,17 +256,17 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 #endif
   for (y=0; y < (ssize_t) extent.y; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict chop_indexes,
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -319,17 +320,17 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 #endif
   for (y=0; y < (ssize_t) (image->rows-(extent.y+extent.height)); y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict chop_indexes,
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -419,7 +420,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     *cmyk_image,
     *cmyk_images;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -447,13 +448,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     cmyk_view=AcquireAuthenticCacheView(cmyk_image,exception);
     for (y=0; y < (ssize_t) images->rows; y++)
     {
-      register const PixelPacket
+      const PixelPacket
         *magick_restrict p;
 
-      register ssize_t
+      ssize_t
         x;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
@@ -479,13 +480,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     cmyk_view=AcquireAuthenticCacheView(cmyk_image,exception);
     for (y=0; y < (ssize_t) images->rows; y++)
     {
-      register const PixelPacket
+      const PixelPacket
         *magick_restrict p;
 
-      register ssize_t
+      ssize_t
         x;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
@@ -511,13 +512,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     cmyk_view=AcquireAuthenticCacheView(cmyk_image,exception);
     for (y=0; y < (ssize_t) images->rows; y++)
     {
-      register const PixelPacket
+      const PixelPacket
         *magick_restrict p;
 
-      register ssize_t
+      ssize_t
         x;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
@@ -543,16 +544,16 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     cmyk_view=AcquireAuthenticCacheView(cmyk_image,exception);
     for (y=0; y < (ssize_t) images->rows; y++)
     {
-      register const PixelPacket
+      const PixelPacket
         *magick_restrict p;
 
-      register IndexPacket
+      IndexPacket
         *magick_restrict indexes;
 
-      register ssize_t
+      ssize_t
         x;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       p=GetCacheViewVirtualPixels(image_view,0,y,images->columns,1,exception);
@@ -747,16 +748,16 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 #endif
   for (y=0; y < (ssize_t) crop_image->rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict crop_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -817,7 +818,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 %  all the normal geometry flags for Crop.
 %
 %      Image *CropImageToTiles(const Image *image,
-%        const RectangleInfo *crop_geometry, ExceptionInfo *exception)
+%        const RectangleInfo *crop_geometry,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -829,14 +830,14 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 %
 */
 
-static inline double MagickRound(double x)
+static inline ssize_t PixelRoundOffset(double x)
 {
   /*
     Round the fraction to nearest integer.
   */
   if ((x-floor(x)) < (ceil(x)-x))
-    return(floor(x));
-  return(ceil(x));
+    return(CastDoubleToLong(floor(x)));
+  return(CastDoubleToLong(ceil(x)));
 }
 
 MagickExport Image *CropImageToTiles(const Image *image,
@@ -901,18 +902,18 @@ MagickExport Image *CropImageToTiles(const Image *image,
       {
         if ((flags & AspectValue) == 0)
           {
-            crop.y=(ssize_t) MagickRound((MagickRealType) (offset.y-
+            crop.y=PixelRoundOffset((MagickRealType) (offset.y-
               (geometry.y > 0 ? 0 : geometry.y)));
             offset.y+=delta.y;   /* increment now to find width */
-            crop.height=(size_t) MagickRound((MagickRealType) (offset.y+
+            crop.height=(size_t) PixelRoundOffset((MagickRealType) (offset.y+
               (geometry.y < 0 ? 0 : geometry.y)));
           }
         else
           {
-            crop.y=(ssize_t) MagickRound((MagickRealType) (offset.y-
+            crop.y=PixelRoundOffset((MagickRealType) (offset.y-
               (geometry.y > 0 ? geometry.y : 0)));
             offset.y+=delta.y;  /* increment now to find width */
-            crop.height=(size_t) MagickRound((MagickRealType) (offset.y+
+            crop.height=(size_t) PixelRoundOffset((MagickRealType) (offset.y+
               (geometry.y < 0 ? geometry.y : 0)));
           }
         crop.height-=crop.y;
@@ -921,18 +922,18 @@ MagickExport Image *CropImageToTiles(const Image *image,
         {
           if ((flags & AspectValue) == 0)
             {
-              crop.x=(ssize_t) MagickRound((MagickRealType) (offset.x-
+              crop.x=PixelRoundOffset((MagickRealType) (offset.x-
                 (geometry.x > 0 ? 0 : geometry.x)));
               offset.x+=delta.x;  /* increment now to find height */
-              crop.width=(size_t) MagickRound((MagickRealType) (offset.x+
+              crop.width=(size_t) PixelRoundOffset((MagickRealType) (offset.x+
                 (geometry.x < 0 ? 0 : geometry.x)));
             }
           else
             {
-              crop.x=(ssize_t) MagickRound((MagickRealType) (offset.x-
+              crop.x=PixelRoundOffset((MagickRealType) (offset.x-
                 (geometry.x > 0 ? geometry.x : 0)));
               offset.x+=delta.x;  /* increment now to find height */
-              crop.width=(size_t) MagickRound((MagickRealType) (offset.x+
+              crop.width=(size_t) PixelRoundOffset((MagickRealType) (offset.x+
                 (geometry.x < 0 ? geometry.x : 0)));
             }
           crop.width-=crop.x;
@@ -956,6 +957,8 @@ MagickExport Image *CropImageToTiles(const Image *image,
         {
           crop_image->page.width=geometry.width;
           crop_image->page.height=geometry.height;
+          crop_image->page.x-=geometry.x;
+          crop_image->page.y-=geometry.y;
         }
       return(crop_image);
     }
@@ -1084,14 +1087,14 @@ MagickExport Image *ExcerptImage(const Image *image,
 #endif
   for (y=0; y < (ssize_t) excerpt_image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict excerpt_indexes,
       *magick_restrict indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1191,6 +1194,7 @@ MagickExport Image *ExtentImage(const Image *image,
     exception);
   if (extent_image == (Image *) NULL)
     return((Image *) NULL);
+  (void) DeleteImageProfile(extent_image,"8bim");  /* delete clipping path */
   status=SetImageBackgroundColor(extent_image);
   if (status == MagickFalse)
     {
@@ -1280,16 +1284,16 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
 #endif
   for (y=0; y < (ssize_t) flip_image->rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict flip_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1409,19 +1413,19 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 #endif
   for (y=0; y < (ssize_t) flop_image->rows; y++)
   {
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict flop_indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1529,16 +1533,16 @@ static MagickBooleanType CopyImageRegion(Image *destination,const Image *source,
     MagickBooleanType
       sync;
 
-    register const IndexPacket
+    const IndexPacket
       *magick_restrict indexes;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict destination_indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     /*
@@ -1840,17 +1844,17 @@ MagickExport Image *SpliceImage(const Image *image,
 #endif
   for (y=0; y < (ssize_t) splice_geometry.y; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes,
       *magick_restrict splice_indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -1920,17 +1924,17 @@ MagickExport Image *SpliceImage(const Image *image,
   for (y=(ssize_t) (splice_geometry.y+splice_geometry.height);
        y < (ssize_t) splice_image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict indexes,
       *magick_restrict splice_indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2147,7 +2151,7 @@ MagickExport MagickBooleanType TransformImages(Image **images,
   MagickStatusType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   assert(images != (Image **) NULL);
@@ -2242,14 +2246,14 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict transpose_indexes,
       *magick_restrict indexes;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
@@ -2375,17 +2379,17 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
     MagickBooleanType
       sync;
 
-    register const PixelPacket
+    const PixelPacket
       *magick_restrict p;
 
-    register IndexPacket
+    IndexPacket
       *magick_restrict transverse_indexes,
       *magick_restrict indexes;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register PixelPacket
+    PixelPacket
       *magick_restrict q;
 
     if (status == MagickFalse)
