@@ -17,13 +17,13 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -106,7 +106,7 @@
   Declare type map.
 */
 static const char
-  *TypeMap = (const char *)
+  TypeMap[] =
     "<?xml version=\"1.0\"?>"
     "<typemap>"
     "  <type stealth=\"True\" name=\"fixed\" family=\"helvetica\"/>"
@@ -159,7 +159,7 @@ static MagickBooleanType
 
 static void *DestroyTypeNode(void *type_info)
 {
-  register TypeInfo
+  TypeInfo
     *p;
 
   p=(TypeInfo *) type_info;
@@ -198,7 +198,7 @@ static SplayTreeInfo *AcquireTypeCache(const char *filename,
   if (cache == (SplayTreeInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
+#if !MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
   {
     char
       *font_path,
@@ -275,23 +275,12 @@ static SplayTreeInfo *AcquireTypeCache(const char *filename,
 MagickExport const TypeInfo *GetTypeInfo(const char *name,
   ExceptionInfo *exception)
 {
-  const TypeInfo
-    *type_info;
-
   assert(exception != (ExceptionInfo *) NULL);
   if (IsTypeTreeInstantiated(exception) == MagickFalse)
     return((const TypeInfo *) NULL);
-  LockSemaphoreInfo(type_semaphore);
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    {
-      ResetSplayTreeIterator(type_cache);
-      type_info=(const TypeInfo *) GetNextValueInSplayTree(type_cache);
-      UnlockSemaphoreInfo(type_semaphore);
-      return(type_info);
-    }
-  type_info=(const TypeInfo *) GetValueFromSplayTree(type_cache,name);
-  UnlockSemaphoreInfo(type_semaphore);
-  return(type_info);
+    return((const TypeInfo *) GetRootValueFromSplayTree(type_cache));
+  return((const TypeInfo *) GetValueFromSplayTree(type_cache,name));
 }
 
 /*
@@ -329,7 +318,6 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-
 MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   const StyleType style,const StretchType stretch,const size_t weight,
   ExceptionInfo *exception)
@@ -337,17 +325,17 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   typedef struct _Fontmap
   {
     const char
-      *name,
-      *substitute;
+      name[17],
+      substitute[10];
   } Fontmap;
 
   const TypeInfo
     *type_info;
 
-  register const TypeInfo
+  const TypeInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -362,8 +350,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
       { "news gothic", "helvetica" },
       { "system", "courier" },
       { "terminal", "courier" },
-      { "wingdings", "symbol" },
-      { NULL, NULL }
+      { "wingdings", "symbol" }
     };
 
   size_t
@@ -485,7 +472,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   /*
     Check for table-based substitution match.
   */
-  for (i=0; fontmap[i].name != (char *) NULL; i++)
+  for (i=0; i < (ssize_t) (sizeof(fontmap)/sizeof(fontmap[0])); i++)
   {
     if (family == (const char *) NULL)
       {
@@ -502,7 +489,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   }
   if (type_info != (const TypeInfo *) NULL)
     {
-      (void) ThrowMagickException(exception,GetMagickModule(),TypeError,
+      (void) ThrowMagickException(exception,GetMagickModule(),TypeWarning,
         "FontSubstitutionRequired","`%s'",type_info->family);
       return(type_info);
     }
@@ -567,10 +554,10 @@ MagickExport const TypeInfo **GetTypeInfoList(const char *pattern,
   const TypeInfo
     **fonts;
 
-  register const TypeInfo
+  const TypeInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -641,7 +628,7 @@ extern "C" {
 
 static int TypeCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -660,10 +647,10 @@ MagickExport char **GetTypeList(const char *pattern,size_t *number_fonts,
   char
     **fonts;
 
-  register const TypeInfo
+  const TypeInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -736,6 +723,9 @@ MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_cache,
     extension[MaxTextExtent],
     name[MaxTextExtent];
 
+  FcBool
+    result;
+
   FcChar8
     *family,
     *file,
@@ -762,7 +752,7 @@ MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_cache,
     width,
     weight;
 
-  register ssize_t
+  ssize_t
     i;
 
   TypeInfo
@@ -772,9 +762,13 @@ MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_cache,
     Load system fonts.
   */
   (void) exception;
-  font_config=FcInitLoadConfigAndFonts();
+  result=FcInit();
+  if (result == 0)
+    return(MagickFalse);
+  font_config=FcConfigGetCurrent();
   if (font_config == (FcConfig *) NULL)
     return(MagickFalse);
+  FcConfigSetRescanInterval(font_config,0);
   font_set=(FcFontSet *) NULL;
   object_set=FcObjectSetBuild(FC_FULLNAME,FC_FAMILY,FC_STYLE,FC_SLANT,
     FC_WIDTH,FC_WEIGHT,FC_FILE,(char *) NULL);
@@ -783,7 +777,7 @@ MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_cache,
       pattern=FcPatternCreate();
       if (pattern != (FcPattern *) NULL)
         {
-          font_set=FcFontList(0,pattern,object_set);
+          font_set=FcFontList(font_config,pattern,object_set);
           FcPatternDestroy(pattern);
         }
       FcObjectSetDestroy(object_set);
@@ -808,9 +802,9 @@ MagickExport MagickBooleanType LoadFontConfigFonts(SplayTreeInfo *type_cache,
     type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
     if (type_info == (TypeInfo *) NULL)
       continue;
-    (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
+    (void) memset(type_info,0,sizeof(*type_info));
     type_info->path=ConstantString("System Fonts");
-    type_info->signature=MagickSignature;
+    type_info->signature=MagickCoreSignature;
     (void) CopyMagickString(name,"Unknown",MaxTextExtent);
     status=FcPatternGetString(font_set->fonts[i],FC_FULLNAME,0,&fullname);
     if ((status == FcResultMatch) && (fullname != (FcChar8 *) NULL))
@@ -895,13 +889,17 @@ static MagickBooleanType IsTypeTreeInstantiated(ExceptionInfo *exception)
       LockSemaphoreInfo(type_semaphore);
       if (type_cache == (SplayTreeInfo *) NULL)
         {
-          type_cache=AcquireTypeCache(MagickTypeFilename,exception);
+          SplayTreeInfo
+            *splay_tree;
+
+          splay_tree=AcquireTypeCache(MagickTypeFilename,exception);
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
-          (void) NTAcquireTypeCache(type_cache,exception);
+          (void) NTAcquireTypeCache(splay_tree,exception);
 #endif
 #if defined(MAGICKCORE_FONTCONFIG_DELEGATE)
-          (void) LoadFontConfigFonts(type_cache,exception);
+          (void) LoadFontConfigFonts(splay_tree,exception);
 #endif
+          type_cache=splay_tree;
         }
       UnlockSemaphoreInfo(type_semaphore);
     }
@@ -948,7 +946,7 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
   const TypeInfo
     **type_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1107,7 +1105,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
     /*
       Interpret XML.
     */
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -1117,7 +1115,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -1126,7 +1124,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1137,13 +1135,13 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetNextToken(q,&q,extent,token);
+          (void) GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
-              if (depth > 200)
+              if (depth > MagickMaxRecursionDepth)
                 (void) ThrowMagickException(exception,GetMagickModule(),
                   ConfigureError,"IncludeNodeNestedTooDeeply","`%s'",token);
               else
@@ -1186,9 +1184,9 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
         type_info=(TypeInfo *) AcquireMagickMemory(sizeof(*type_info));
         if (type_info == (TypeInfo *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-        (void) ResetMagickMemory(type_info,0,sizeof(*type_info));
+        (void) memset(type_info,0,sizeof(*type_info));
         type_info->path=ConstantString(filename);
-        type_info->signature=MagickSignature;
+        type_info->signature=MagickCoreSignature;
         continue;
       }
     if (type_info == (TypeInfo *) NULL)
@@ -1203,11 +1201,11 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
         type_info=(TypeInfo *) NULL;
         continue;
       }
-    GetNextToken(q,(const char **) NULL,extent,token);
+    (void) GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetNextToken(q,&q,extent,token);
-    GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
+    (void) GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'E':
