@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
 
     https://imagemagick.org/script/license.php
@@ -56,7 +56,7 @@ static inline wchar_t *create_wchar_path(const char *utf8)
     *wideChar;
 
   count=MultiByteToWideChar(CP_UTF8,0,utf8,-1,NULL,0);
-  if (count > MAX_PATH)
+  if ((count > MAX_PATH) && (NTLongPathsEnabled() == MagickFalse))
     {
       char
         buffer[MaxTextExtent];
@@ -74,7 +74,7 @@ static inline wchar_t *create_wchar_path(const char *utf8)
       if (count != 0)
         count=GetShortPathNameW(longPath,shortPath,MAX_PATH);
       longPath=(wchar_t *) RelinquishMagickMemory(longPath);
-      if (count < 5)
+      if ((count < 5) || (count >= MAX_PATH))
         return((wchar_t *) NULL);
       wideChar=(wchar_t *) AcquireQuantumMemory(count-3,sizeof(*wideChar));
       wcscpy(wideChar,shortPath+4);
@@ -115,7 +115,7 @@ static inline int access_utf8(const char *path,int mode)
 
 static inline FILE *fopen_utf8(const char *path,const char *mode)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(fopen(path,mode));
 #else
    FILE
@@ -165,7 +165,7 @@ typedef int
 
 static inline int open_utf8(const char *path,int flags,mode_t mode)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(open(path,flags,mode));
 #else
    int
@@ -185,35 +185,40 @@ static inline int open_utf8(const char *path,int flags,mode_t mode)
 
 static inline FILE *popen_utf8(const char *command,const char *type)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(popen(command,type));
 #else
-   FILE
-     *file;
+  FILE
+    *file;
 
-   wchar_t
-     *type_wide,
-     *command_wide;
+  int
+    length;
 
-   command_wide=create_wchar_path(command);
-   if (command_wide == (wchar_t *) NULL)
-     return((FILE *) NULL);
-   type_wide=create_wchar_path(type);
-   if (type_wide == (wchar_t *) NULL)
-     {
-       command_wide=(wchar_t *) RelinquishMagickMemory(command_wide);
-       return((FILE *) NULL);
-     }
-   file=_wpopen(command_wide,type_wide);
-   type_wide=(wchar_t *) RelinquishMagickMemory(type_wide);
-   command_wide=(wchar_t *) RelinquishMagickMemory(command_wide);
-   return(file);
+  wchar_t
+    *command_wide,
+    type_wide[5];
+
+  file=(FILE *) NULL;
+  length=MultiByteToWideChar(CP_UTF8,0,type,-1,type_wide,5);
+  if (length == 0)
+    return(file);
+  length=MultiByteToWideChar(CP_UTF8,0,command,-1,NULL,0);
+  if (length == 0)
+    return(file);
+  command_wide=(wchar_t *) AcquireQuantumMemory(length,sizeof(*command_wide));
+  if (command_wide == (wchar_t *) NULL)
+    return(file);
+  length=MultiByteToWideChar(CP_UTF8,0,command,-1,command_wide,length);
+  if (length != 0)
+    file=_wpopen(command_wide,type_wide);
+  command_wide=(wchar_t *) RelinquishMagickMemory(command_wide);
+  return(file);
 #endif
 }
 
 static inline int remove_utf8(const char *path)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(unlink(path));
 #else
    int
@@ -233,7 +238,7 @@ static inline int remove_utf8(const char *path)
 
 static inline int rename_utf8(const char *source,const char *destination)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(rename(source,destination));
 #else
    int
@@ -261,7 +266,7 @@ static inline int rename_utf8(const char *source,const char *destination)
 
 static inline int stat_utf8(const char *path,struct stat *attributes)
 {
-#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(stat(path,attributes));
 #else
    int
