@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -345,11 +345,11 @@ MagickExport unsigned char *Base64Decode(const char *source,size_t *length)
   int
     state;
 
-  register const char
+  const char
     *p,
     *q;
 
-  register size_t
+  size_t
     i;
 
   unsigned char
@@ -501,10 +501,10 @@ MagickExport char *Base64Encode(const unsigned char *blob,
   char
     *encode;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -580,7 +580,7 @@ MagickExport char *Base64Encode(const unsigned char *blob,
 */
 MagickExport void ChopPathComponents(char *path,const size_t components)
 {
-  register ssize_t
+  ssize_t
     i;
 
   for (i=0; i < (ssize_t) components; i++)
@@ -644,9 +644,12 @@ MagickExport void ExpandFilename(char *path)
     {
 #if defined(MAGICKCORE_POSIX_SUPPORT) && !defined(__OS2__)
       char
+#if defined(MAGICKCORE_HAVE_GETPWNAM_R)
+        buffer[MagickPathExtent],
+#endif
         username[MaxTextExtent];
 
-      register char
+      char
         *p;
 
       struct passwd
@@ -659,7 +662,16 @@ MagickExport void ExpandFilename(char *path)
       p=strchr(username,'/');
       if (p != (char *) NULL)
         *p='\0';
+#if !defined(MAGICKCORE_HAVE_GETPWNAM_R)
       entry=getpwnam(username);
+#else
+      struct passwd
+        pwd;
+
+      entry=(struct passwd *) NULL;
+      if (getpwnam_r(username,&pwd,buffer,sizeof(buffer),&entry) < 0)
+        return;
+#endif
       if (entry == (struct passwd *) NULL)
         return;
       (void) CopyMagickString(expand_path,entry->pw_dir,MaxTextExtent);
@@ -736,7 +748,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
     home_directory[MaxTextExtent],
     **vector;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -810,7 +822,7 @@ MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
       continue;
     if ((IsGlob(filename) == MagickFalse) && (*option != '@'))
       continue;
-    if (*option != '@')
+    if ((*option != '@') && (IsPathAccessible(option) == MagickFalse))
       {
         /*
           Generate file list from wildcard filename (e.g. *.jpg).
@@ -1125,8 +1137,6 @@ MagickExport ssize_t GetMagickPageSize(void)
   page_size=(ssize_t) sysconf(_SC_PAGE_SIZE);
 #elif defined(MAGICKCORE_HAVE_GETPAGESIZE)
   page_size=(ssize_t) getpagesize();
-#elif defined(MAGICKCORE_WINDOWS_SUPPORT)
-  page_size=NTGetPageSize();
 #endif
   if (page_size <= 0)
     page_size=4096;
@@ -1211,7 +1221,7 @@ MagickExport void GetPathComponent(const char *path,PathType type,
   char
     *q;
 
-  register char
+  char
     *p;
 
   size_t
@@ -1233,23 +1243,25 @@ MagickExport void GetPathComponent(const char *path,PathType type,
   if (type != SubcanonicalPath)
     {
       p=component+strlen(component)-1;
-      q=strrchr(component,'[');
-      if ((strlen(component) > 2) && (*p == ']') && (q != (char *) NULL) &&
-          ((q == component) || (*(q-1) != ']')) &&
-          (IsPathAccessible(path) == MagickFalse))
+      if ((strlen(component) > 2) && (*p == ']'))
         {
-          /*
-            Look for scene specification (e.g. img0001.pcd[4]).
-          */
-          *p='\0';
-          if ((IsSceneGeometry(q+1,MagickFalse) == MagickFalse) &&
-              (IsGeometry(q+1) == MagickFalse))
-            *p=']';
-          else
+          q=strrchr(component,'[');
+          if ((q != (char *) NULL) && ((q == component) || (*(q-1) != ']')) &&
+              (IsPathAccessible(path) == MagickFalse))
             {
-              subimage_length=(size_t) (p-q);
-              subimage_offset=(size_t) (q-component+1);
-              *q='\0';
+              /*
+                Look for scene specification (e.g. img0001.pcd[4]).
+              */
+              *p='\0';
+                if ((IsSceneGeometry(q+1,MagickFalse) == MagickFalse) &&
+                  (IsGeometry(q+1) == MagickFalse))
+                *p=']';
+              else
+                {
+                  subimage_length=(size_t) (p-q);
+                  subimage_offset=(size_t) (q-component+1);
+                  *q='\0';
+                }
             }
         }
     }
@@ -1400,11 +1412,11 @@ MagickExport char **GetPathComponents(const char *path,
   char
     **components;
 
-  register const char
+  const char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   if (path == (char *) NULL)
@@ -1561,7 +1573,7 @@ extern "C" {
 
 static int FileCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -1621,7 +1633,8 @@ MagickExport char **ListFiles(const char *directory,const char *pattern,
   while ((MagickReadDirectory(current_directory,buffer,&entry) == 0) &&
          (entry != (struct dirent *) NULL))
   {
-    if (*entry->d_name == '.')
+    if ((LocaleCompare(entry->d_name,".") == 0) ||
+        (LocaleCompare(entry->d_name,"..") == 0))
       continue;
     if ((IsPathDirectory(entry->d_name) > 0) ||
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
@@ -1643,7 +1656,7 @@ MagickExport char **ListFiles(const char *directory,const char *pattern,
           }
 #if defined(vms)
         {
-          register char
+          char
             *p;
 
           p=strchr(entry->d_name,';');
@@ -1733,7 +1746,15 @@ MagickExport void MagickDelay(const MagickSizeType milliseconds)
 #elif defined(__BEOS__)
   snooze(1000*milliseconds);
 #else
-# error "Time delay method not defined."
+  {
+    clock_t
+      time_end;
+
+    time_end=clock()+milliseconds*CLOCKS_PER_SEC/1000;
+    while (clock() < time_end)
+    {
+    }
+  }
 #endif
 }
 
@@ -1814,7 +1835,7 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
   MagickSizeType
     length;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1868,7 +1889,7 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
     RandomInfo
       *random_info;
 
-    register MagickOffsetType
+    MagickOffsetType
       j;
 
     ssize_t
