@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -179,6 +179,9 @@ MagickExport const char *GetMagickDelegates(void)
     defined(MAGICKCORE_WINDOWS_SUPPORT)
   "ps "
 #endif
+#if defined(MAGICKCORE_RAQM_DELEGATE)
+  "raqm "
+#endif
 #if defined(MAGICKCORE_RAW_R_DELEGATE)
   "raw "
 #endif
@@ -248,10 +251,35 @@ MagickExport const char *GetMagickFeatures(void)
   "OpenCL "
 #endif
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  "OpenMP "
+  "OpenMP"
+#if _OPENMP == 199810
+  "(1.0) "
+#elif _OPENMP == 200203
+  "(2.0) "
+#elif _OPENMP == 200505
+  "(2.5) "
+#elif _OPENMP == 200805
+  "(3.0) "
+#elif _OPENMP == 201107
+  "(3.1) "
+#elif _OPENMP == 201307
+  "(4.0) "
+#elif _OPENMP == 201511
+  "(4.5) "
+#elif _OPENMP == 201811
+  "(5.0) "
+#else
+  " "
+#endif
+#endif
+#if defined(MAGICKCORE_HAVE_TCMALLOC)
+  "TCMalloc "
 #endif
 #if defined(ZERO_CONFIGURATION_SUPPORT)
   "Zero-configuration "
+#endif
+#if (MAGICKCORE_QUANTUM_DEPTH == 64)
+  "Q64 (experimental, not for production) "
 #endif
   ;
 }
@@ -467,7 +495,7 @@ MagickExport const char *GetMagickReleaseDate(void)
 
 static unsigned int CRC32(const unsigned char *message,const size_t length)
 {
-  register ssize_t
+  ssize_t
     i;
 
   static MagickBooleanType
@@ -484,7 +512,7 @@ static unsigned int CRC32(const unsigned char *message,const size_t length)
   */
   if (crc_initial == MagickFalse)
     {
-      register unsigned int
+      unsigned int
         i;
 
       unsigned int
@@ -492,7 +520,7 @@ static unsigned int CRC32(const unsigned char *message,const size_t length)
 
       for (i=0; i < 256; i++)
       {
-        register ssize_t
+        ssize_t
           j;
 
         alpha=i;
@@ -510,7 +538,7 @@ static unsigned int CRC32(const unsigned char *message,const size_t length)
 
 MagickExport unsigned int GetMagickCoreSignature(const StringInfo *nonce)
 {
-  register unsigned char
+  unsigned char
     *p;
 
   StringInfo
@@ -530,7 +558,7 @@ MagickExport unsigned int GetMagickCoreSignature(const StringInfo *nonce)
   signature=MagickLibInterface;
   (void) memcpy(p,&signature,sizeof(signature));
   p+=sizeof(signature);
-  signature=1;  /* endianess */
+  signature=1;  /* endinaness */
   (void) memcpy(p,&signature,sizeof(signature));
   p+=sizeof(signature);
   SetStringInfoLength(version,p-GetStringInfoDatum(version));
