@@ -23,7 +23,7 @@
 %                               December 2004                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -219,7 +219,7 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
     subnode[MaxTextExtent],
     tag[MaxTextExtent];
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -284,7 +284,6 @@ MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
 %
 %  The format of the CanonicalXMLContent method is:
 %
-%
 %      char *CanonicalXMLContent(const char *content,
 %        const MagickBooleanType pedantic)
 %
@@ -303,10 +302,10 @@ MagickExport char *CanonicalXMLContent(const char *content,
     *base64,
     *canonical_content;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -437,7 +436,7 @@ MagickExport char *CanonicalXMLContent(const char *content,
 
 static char **DestroyXMLTreeAttributes(char **attributes)
 {
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -496,7 +495,7 @@ static void DestroyXMLTreeRoot(XMLTreeInfo *xml_info)
   char
     **attributes;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -606,7 +605,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
   MagickOffsetType
     offset;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -695,7 +694,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       (void) lseek(file,0,SEEK_SET);
       for (i=0; i < length; i+=count)
       {
-        count=read(file,xml+i,(size_t) MagickMin(length-i,SSIZE_MAX));
+        count=read(file,xml+i,(size_t) MagickMin(length-i,(size_t) MAGICK_SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -778,7 +777,7 @@ MagickExport XMLTreeInfo *GetNextXMLTreeTag(XMLTreeInfo *xml_info)
 MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag)
 {
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -847,7 +846,7 @@ MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
 MagickExport MagickBooleanType GetXMLTreeAttributes(const XMLTreeInfo *xml_info,
   SplayTreeInfo *attributes)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(xml_info != (XMLTreeInfo *) NULL);
@@ -1006,7 +1005,7 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
     subnode[MaxTextExtent],
     tag[MaxTextExtent];
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1077,7 +1076,7 @@ MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
 MagickExport const char **GetXMLTreeProcessingInstructions(
   XMLTreeInfo *xml_info,const char *target)
 {
-  register ssize_t
+  ssize_t
     i;
 
   XMLTreeRoot
@@ -1296,7 +1295,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     c,
     encoding;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1375,11 +1374,11 @@ static char *ParseEntities(char *xml,char **entities,int state)
     byte,
     c;
 
-  register char
+  char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1482,19 +1481,24 @@ static char *ParseEntities(char *xml,char **entities,int state)
                     offset=(ssize_t) (xml-p);
                     extent=(size_t) (offset+length+strlen(entity));
                     if (p != q)
-                      p=(char *) ResizeQuantumMemory(p,extent,sizeof(*p));
+                      {
+                        p=(char *) ResizeQuantumMemory(p,extent+1,sizeof(*p));
+                        p[extent]='\0';
+                      }
                     else
                       {
                         char
-                          *xml;
+                          *extent_xml;
 
-                        xml=(char *) AcquireQuantumMemory(extent,sizeof(*xml));
-                        if (xml != (char *) NULL)
+                        extent_xml=(char *) AcquireQuantumMemory(extent+1,
+                          sizeof(*extent_xml));
+                        if (extent_xml != (char *) NULL)
                           {
-                            memset(xml,0,extent*sizeof(*xml));
-                            (void) CopyMagickString(xml,p,extent*sizeof(*xml));
+                            memset(extent_xml,0,extent*sizeof(*extent_xml));
+                            (void) CopyMagickString(extent_xml,p,extent*
+                              sizeof(*extent_xml));
                           }
-                        p=xml;
+                        p=extent_xml;
                       }
                     if (p == (char *) NULL)
                       ThrowFatalException(ResourceLimitFatalError,
@@ -1504,7 +1508,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
                   }
                 if (entity != (char *) NULL)
                   (void) memmove(xml+length,entity+1,strlen(entity));
-                (void) strncpy(xml,entities[i],length);
+                (void) memcpy(xml,entities[i],length);
               }
         }
       else
@@ -1582,7 +1586,7 @@ static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,
 static MagickBooleanType ValidateEntities(char *tag,char *xml,
   const size_t depth,char **entities)
 {
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1614,7 +1618,7 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
   char
     *target;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1697,7 +1701,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
     *t,
     *v;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1727,6 +1731,8 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
         if ((isalpha((int) ((unsigned char) *n)) == 0) && (*n != '_'))
           break;
         xml=n+strcspn(n,XMLWhitespace);
+        if (*xml == '\0')
+          break;
         *xml=';';
         v=xml+strspn(xml+1,XMLWhitespace)+1;
         q=(*v);
@@ -1934,7 +1940,7 @@ static const char
 
 static inline MagickBooleanType IsSkipTag(const char *tag)
 {
-  register ssize_t
+  ssize_t
     i;
 
   i=0;
@@ -1962,10 +1968,10 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  register char
+  char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2135,7 +2141,9 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                 utf8=DestroyString(utf8);
                 return(&root->root);
               }
-            if ((ignore_depth == 0) && (IsSkipTag(tag) == MagickFalse))
+            if ((ignore_depth != 0) || (IsSkipTag(tag) != MagickFalse))
+              (void) DestroyXMLTreeAttributes(attributes);
+            else
               {
                 ParseOpenTag(root,tag,attributes);
                 (void) ParseCloseTag(root,tag,exception);
@@ -2375,8 +2383,7 @@ MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
   root->entities=(char **) AcquireMagickMemory(sizeof(predefined_entities));
   if (root->entities == (char **) NULL)
     return((XMLTreeInfo *) NULL);
-  (void) memcpy(root->entities,predefined_entities,
-    sizeof(predefined_entities));
+  (void) memcpy(root->entities,predefined_entities,sizeof(predefined_entities));
   root->root.attributes=sentinel;
   root->attributes=(char ***) root->root.attributes;
   root->processing_instructions=(char ***) root->root.attributes;
@@ -2485,7 +2492,7 @@ MagickExport XMLTreeInfo *PruneTagFromXMLTree(XMLTreeInfo *xml_info)
 MagickExport XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag,const char *value)
 {
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -2654,7 +2661,7 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
   const char
     *attribute;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -2756,11 +2763,11 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
   char
     *xml;
 
-  register char
+  char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
