@@ -18,13 +18,13 @@
 %                               March 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -40,12 +40,7 @@
 /*
   Include declarations.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <math.h>
-#include <locale.h>
+#include "wand/studio.h"
 #include "wand/MagickWand.h"
 #include "magick/colorspace-private.h"
 #include "magick/log.h"
@@ -79,7 +74,7 @@
 %
 %  The format of the ValidateColorspaces method is:
 %
-%      size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
+%      size_t ValidateColorspaces(ImageInfo *image_info,size_t *fails,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -821,13 +816,14 @@ static MagickBooleanType ValidateYUVToRGB()
   return(MagickTrue);
 }
 
-static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
+static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fails,
   ExceptionInfo *exception)
 {
   MagickBooleanType
     status;
 
   size_t
+    fail,
     test;
 
   /*
@@ -853,6 +849,7 @@ static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
      Y'CbCr       187.577791,  87.586330,  90.040886
   */
   (void) FormatLocaleFile(stdout,"validate colorspaces:\n");
+  fail=0;
   for (test=0; test < 26; test++)
   {
     CatchException(exception);
@@ -891,14 +888,15 @@ static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -920,7 +918,7 @@ static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
 %
 %      size_t ValidateCompareCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -936,7 +934,7 @@ static size_t ValidateColorspaces(ImageInfo *image_info,size_t *fail,
 %
 */
 static size_t ValidateCompareCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -949,13 +947,15 @@ static size_t ValidateCompareCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate compare command line program:\n");
   for (i=0; compare_options[i] != (char *) NULL; i++)
@@ -970,7 +970,7 @@ static size_t ValidateCompareCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=CompareImageCommand(image_info,number_arguments,arguments,
@@ -982,14 +982,15 @@ static size_t ValidateCompareCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1011,7 +1012,7 @@ static size_t ValidateCompareCommand(ImageInfo *image_info,
 %
 %      size_t ValidateCompositeCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1027,7 +1028,7 @@ static size_t ValidateCompareCommand(ImageInfo *image_info,
 %
 */
 static size_t ValidateCompositeCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -1040,13 +1041,15 @@ static size_t ValidateCompositeCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate composite command line program:\n");
   for (i=0; composite_options[i] != (char *) NULL; i++)
@@ -1062,7 +1065,7 @@ static size_t ValidateCompositeCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=CompositeImageCommand(image_info,number_arguments,arguments,
@@ -1074,14 +1077,15 @@ static size_t ValidateCompositeCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1103,7 +1107,7 @@ static size_t ValidateCompositeCommand(ImageInfo *image_info,
 %
 %      size_t ValidateConvertCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1119,7 +1123,7 @@ static size_t ValidateCompositeCommand(ImageInfo *image_info,
 %
 */
 static size_t ValidateConvertCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -1132,13 +1136,15 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate convert command line program:\n");
   for (i=0; convert_options[i] != (char *) NULL; i++)
@@ -1153,7 +1159,7 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=ConvertImageCommand(image_info,number_arguments,arguments,
@@ -1167,14 +1173,15 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
           GetMagickModule());
         (void) LogMagickEvent(ExceptionEvent,GetMagickModule(),
           "%s",exception->reason);
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1196,7 +1203,7 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
 %
 %      size_t ValidateIdentifyCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1212,7 +1219,7 @@ static size_t ValidateConvertCommand(ImageInfo *image_info,
 %
 */
 static size_t ValidateIdentifyCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -1225,14 +1232,16 @@ static size_t ValidateIdentifyCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
   (void) output_filename;
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate identify command line program:\n");
   for (i=0; identify_options[i] != (char *) NULL; i++)
@@ -1247,7 +1256,7 @@ static size_t ValidateIdentifyCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=IdentifyImageCommand(image_info,number_arguments,arguments,
@@ -1259,14 +1268,15 @@ static size_t ValidateIdentifyCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
           GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1288,7 +1298,7 @@ static size_t ValidateIdentifyCommand(ImageInfo *image_info,
 %
 %      size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1311,7 +1321,7 @@ static size_t ValidateIdentifyCommand(ImageInfo *image_info,
 #undef MagickCountTempFiles
 
 static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -1337,7 +1347,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -1348,18 +1358,18 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
     *blob;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate image formats in memory:\n");
-
 #ifdef MagickCountTempFiles
   (void) GetPathTemplate(path);
   /* Remove file template except for the leading "/path/to/magick-" */
   path[strlen(path)-17]='\0';
   (void) FormatLocaleFile(stdout," tmp path is '%s*'\n",path);
 #endif
-
   for (i=0; reference_formats[i].magick != (char *) NULL; i++)
   {
     magick_info=GetMagickInfo(reference_formats[i].magick,exception);
@@ -1387,7 +1397,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1406,7 +1416,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1417,7 +1427,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1432,7 +1442,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1449,7 +1459,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       ping_image=DestroyImage(ping_image);
@@ -1467,7 +1477,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1490,7 +1500,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1506,7 +1516,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           continue;
         }
@@ -1526,7 +1536,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1548,7 +1558,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
           if (exception->reason != (char *) NULL)
             (void) FormatLocaleFile(stdout,"    reason:%s\n",exception->reason);
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       difference_image=DestroyImage(difference_image);
@@ -1556,7 +1566,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
         {
           (void) FormatLocaleFile(stdout,"... fail (with distortion %g).\n",
             QuantumScale*distortion);
-          (*fail)++;
+          fail++;
           continue;
         }
 
@@ -1576,7 +1586,8 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1598,7 +1609,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
 %
 %      size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1614,7 +1625,7 @@ static size_t ValidateImageFormatsInMemory(ImageInfo *image_info,
 %
 */
 static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -1635,13 +1646,15 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate image formats on disk:\n");
   for (i=0; reference_formats[i].magick != (char *) NULL; i++)
@@ -1671,7 +1684,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1690,7 +1703,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1701,7 +1714,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1714,7 +1727,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1729,7 +1742,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       /*
@@ -1746,7 +1759,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1762,7 +1775,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1782,7 +1795,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       difference_image=DestroyImage(difference_image);
@@ -1790,7 +1803,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
         {
           (void) FormatLocaleFile(stdout,"... fail (with distortion %g).\n",
             QuantumScale*distortion);
-          (*fail)++;
+          fail++;
           continue;
         }
       (void) FormatLocaleFile(stdout,"... pass.\n");
@@ -1798,7 +1811,8 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -1820,7 +1834,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
 %
 %      size_t ValidateImportExportPixels(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1836,7 +1850,7 @@ static size_t ValidateImageFormatsOnDisk(ImageInfo *image_info,
 %
 */
 static size_t ValidateImportExportPixels(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   double
@@ -1850,7 +1864,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -1861,9 +1875,11 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
     *pixels;
 
   size_t
+    fail,
     test;
 
   (void) output_filename;
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,
     "validate the import and export of image pixels:\n");
@@ -1887,7 +1903,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       if (LocaleNCompare(reference_map[i],"cmy",3) == 0)
@@ -1904,7 +1920,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1917,7 +1933,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           pixels=(unsigned char *) RelinquishMagickMemory(pixels);
           reference_image=DestroyImage(reference_image);
           continue;
@@ -1932,7 +1948,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
            pixels=(unsigned char *) RelinquishMagickMemory(pixels);
           reference_image=DestroyImage(reference_image);
           continue;
@@ -1955,7 +1971,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           reference_image=DestroyImage(reference_image);
           continue;
         }
@@ -1972,7 +1988,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
           (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
           CatchException(exception);
-          (*fail)++;
+          fail++;
           continue;
         }
       difference_image=DestroyImage(difference_image);
@@ -1980,7 +1996,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
         {
           (void) FormatLocaleFile(stdout,"... fail (with distortion %g).\n",
             QuantumScale*distortion);
-          (*fail)++;
+          fail++;
           continue;
         }
       (void) FormatLocaleFile(stdout,"... pass.\n");
@@ -1988,7 +2004,8 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -2010,7 +2027,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
 %
 %      size_t ValidateMontageCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -2026,7 +2043,7 @@ static size_t ValidateImportExportPixels(ImageInfo *image_info,
 %
 */
 static size_t ValidateMontageCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -2039,13 +2056,15 @@ static size_t ValidateMontageCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate montage command line program:\n");
   for (i=0; montage_options[i] != (char *) NULL; i++)
@@ -2061,7 +2080,7 @@ static size_t ValidateMontageCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=MontageImageCommand(image_info,number_arguments,arguments,
@@ -2073,14 +2092,15 @@ static size_t ValidateMontageCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -2102,7 +2122,7 @@ static size_t ValidateMontageCommand(ImageInfo *image_info,
 %
 %      size_t ValidateStreamCommand(ImageInfo *image_info,
 %        const char *reference_filename,const char *output_filename,
-%        size_t *fail,ExceptionInfo *exception)
+%        size_t *fails,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -2118,7 +2138,7 @@ static size_t ValidateMontageCommand(ImageInfo *image_info,
 %
 */
 static size_t ValidateStreamCommand(ImageInfo *image_info,
-  const char *reference_filename,const char *output_filename,size_t *fail,
+  const char *reference_filename,const char *output_filename,size_t *fails,
   ExceptionInfo *exception)
 {
   char
@@ -2131,13 +2151,15 @@ static size_t ValidateStreamCommand(ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i,
     j;
 
   size_t
+    fail,
     test;
 
+  fail=0;
   test=0;
   (void) FormatLocaleFile(stdout,"validate stream command line program:\n");
   for (i=0; stream_options[i] != (char *) NULL; i++)
@@ -2152,7 +2174,7 @@ static size_t ValidateStreamCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     status=StreamImageCommand(image_info,number_arguments,arguments,
@@ -2164,14 +2186,15 @@ static size_t ValidateStreamCommand(ImageInfo *image_info,
       {
         (void) FormatLocaleFile(stdout,"... fail @ %s/%s/%lu.\n",
             GetMagickModule());
-        (*fail)++;
+        fail++;
         continue;
       }
     (void) FormatLocaleFile(stdout,"... pass.\n");
   }
   (void) FormatLocaleFile(stdout,
     "  summary: %.20g subtests; %.20g passed; %.20g failed.\n",(double) test,
-    (double) (test-(*fail)),(double) *fail);
+    (double) (test-fail),(double) fail);
+  fails+=fail;
   return(test);
 }
 
@@ -2266,7 +2289,7 @@ int main(int argc,char **argv)
     memory_resource,
     map_resource;
 
-  register ssize_t
+  ssize_t
     i;
 
   TimerInfo
