@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
 
     https://imagemagick.org/script/license.php
@@ -97,14 +97,24 @@ static const char
 static const char
   *convert_options[] =
   {
-    "-noop",
+    "",
     "-affine 1,0,0.785,1,0,0 -transform",
     "-black-threshold 20%",
     "-blur 0x0.5",
+    "-blur 0x1.0",
+    "-blur 0x2.0",
     "-border 6x6",
     "-canny 0x1+10%+80%",
+    "-channel red",
     "-charcoal 0x1",
+    "-chop 80x60+10+20",
     "-chop 8x6+20+30",
+    "-colorize 30%/20%/50%",
+    "-color-matrix '0,0,1,0,1,0,1,0,0'",
+    "-color-matrix '0.9 0 0, 0 0.9 0, 0 0 1.2'",
+    "-color-matrix '0.9,0,0,0,0.9,0,0,0,1.2'",
+    "-color-matrix '1,0,0,0,1,0,0,0,1'",
+    "-color-matrix '.22,.72,.07,.22,.72,.07,.22,.72,.07'",
     "-colors 16",
     "-colorspace CMYK",
     "-colorspace GRAY",
@@ -114,86 +124,122 @@ static const char
     "-colorspace YCbCr",
     "-colorspace YIQ",
     "-colorspace YUV",
-    "-contrast",
+    "+contrast +contrast +contrast",
+    "-contrast -contrast -contrast",
     "+contrast",
+    "-contrast",
     "-convolve 1,1,1,1,4,1,1,1,1",
-    "-colorize 30%/20%/50%",
     "-crop 17x9+10+10",
+    "-crop 60x70+10+10",
     "-cycle 200",
+    "-density 75x75 -resample 50x50",
     "-despeckle",
     "-draw \"rectangle 20,10 80,50\"",
     "-edge 0x1",
     "-emboss 0x1",
     "-enhance",
     "-equalize",
+    "-evaluate Abs 2%",
+    "-evaluate Add 233",
+    "-evaluate AddModulus 50%",
+    "-evaluate And 6",
+    "-evaluate Cosine 2",
+    "-evaluate Divide 0.7",
+    "-evaluate Exponential 1.0",
+    "-evaluate InverseLog 2",
+    "-evaluate Xor 233",
+    "-extent 1200x1200-100-100",
+    "-fill blue -fuzz 35% -opaque red",
+    "-fill blue -stroke gold -draw 'Bezier 400,200 1100,800 100,300'",
+    "-fill blue -stroke gold -draw 'polygon 400,200 1100,800 100,300'",
+    "-fill blue -stroke gold -draw 'rectangle 400,200 1100,800'",
+    "-fill blue -stroke gold -draw 'roundRectangle 400,200 1100,800 20,20'",
+    "-fill green -stroke gold -draw 'circle 800,500 1100,800'",
+    "-fill none -stroke gold -draw 'Bezier 400,200 1100,800 100,300'",
+    "-fill none -stroke gold -draw 'circle 800,500 1100,800'",
+    "-fill none -stroke gold -draw 'polygon 400,200 1100,800 100,300'",
+    "-fill none -stroke gold -draw 'rectangle 400,200 1100,800'",
+    "-fill none -stroke gold -draw 'roundRectangle 400,200 1100,800 20,20'",
     "-flip",
     "-flop",
     "-frame 15x15+3+3",
+    "-fuzz 35% -transparent red",
+    "-fuzz 5% -trim",
     "-fx \"(1.0/(1.0+exp(10.0*(0.5-u)))-0.006693)*1.0092503\"",
     "-gamma 1.6",
     "-gaussian 0x0.5",
+    "-gaussian 0x1.0",
+    "-gaussian 0x2.0",
     "-implode 0.5",
     "-implode -1",
     "-label Magick",
     "-lat 10x10-5%",
     "-level 10%,1.2,90%",
+    "-magnify",
     "-map netscape:",
+    "-median 1",
     "-median 2",
     "-modulate 110/100/95",
     "-monochrome",
     "-motion-blur 0x3+30",
     "-negate",
-    "+noise Uniform",
+    "-noise 1",
+    "-noise 2",
     "+noise Gaussian",
-    "+noise Multiplicative",
     "+noise Impulse",
     "+noise Laplacian",
+    "+noise Multiplicative",
     "+noise Poisson",
-    "-noise 2",
+    "+noise Random",
+    "+noise Uniform",
+    "-noop",
     "-normalize",
-    "-fill blue -fuzz 35% -opaque red",
-    "-ordered-dither 2x2",
     "-paint 0x1",
     "-raise 10x10",
     "-random-threshold 10%",
-    "-recolor '0.9 0 0, 0 0.9 0, 0 0 1.2'",
-    "-density 75x75 -resample 50x50",
+    "-random-threshold 20x80",
+    "-resize 100%",
     "-resize 10%",
+    "-resize 150%",
+    "-resize 150x75%",
     "-resize 50%",
     "-resize 50x150%",
-    "-resize 100%",
-    "-resize 150%",
-    "-resize 150x50%",
     "-roll +20+10",
     "-rotate 0",
-    "-rotate 45",
-    "-rotate 90",
+    "-rotate 15",
     "-rotate 180",
     "-rotate 270",
-    "-sample 5%",
-    "-sample 50%",
-    "-sample 50x150%",
+    "-rotate 45",
+    "-rotate 90",
     "-sample 100%",
     "-sample 150%",
     "-sample 150x50%",
-    "-scale 5%",
-    "-scale 50%",
-    "-scale 50x150%",
+    "-sample 50%",
+    "-sample 50x150%",
+    "-sample 5%",
     "-scale 100%",
     "-scale 150%",
     "-scale 150x50%",
+    "-scale 50%",
+    "-scale 50x150%",
+    "-scale 5%",
+    "-segment 0.5x0.25",
     "-segment 1x1.5",
     "-shade 30x30",
+    "-sharpen 0x0.5",
     "-sharpen 0x1.0",
+    "-sharpen 0x2.0",
     "-shave 10x10",
+    "-shear 25x20",
     "-shear 45x45",
     "-size 130x194",
     "-solarize 50%",
+    "-spread 1",
     "-spread 3",
     "-swirl 90",
     "-threshold 35%",
-    "-fuzz 35% -transparent red",
-    "-fuzz 5% -trim",
+    "-trim",
+    "-unsharp 0x0.5+20+1",
     "-unsharp 0x1.0+20+1",
     "-wave 25x150",
     "-white-threshold 80%",
